                processor 6502
;###
;### BitLive4 demo
;###  to be run on an unmodified Commodore 64
;###
;### Created by
;###  Pex "Mahoney" Tufvesson
;###
;### Released on the 13th of September 2003
;###
;
;###
;### Credits
;###
;
;Coding by: Pex Mahoney Tufvesson
;Music: Spellbound by Rob Hubbard
;The endscroller charset by Shaman of Taboo
;
;###
;### Inspiration from
;###
;
;Stable Raster IRQ by Fungus 1996
;Sprite multiplexing by Lasse rni
;Altered States by Taboo 1994, Endscroll code by MMS
;Commodore 64 Programmer's Reference Guide by Project64, Ville Muikkula.
;
;###
;### Tools used for creating the BitLive4-demo:
;###
;
;CodeWright 6.0b
;    text editor for win32.
;
;Photoshop 7.0
;    graphics editor for win32.
;
;PuCrunch 8.3.2002
;    by Pasi Ojala.
;    Packer for win32.
;
;XRay64 v0.1 beta3
;    by Jakub 'Prezes' Wozniakowski.
;    Graphics converter for win32.
;
;dasm v2.12.04
;    by Mattew Dillon
;    and Olaf 'Rhialto' Seibert.
;
;WinVice v1.12
;    Commodore64 emulator
;
;CCS64
;    Commodore64 emulator.
;
;...thanks to you all for making these wonderful
;tools. Without them, this demo would never be.
;
;###
;### How to "compile"
;###
;
;With any win32-computer, run the "a.bat" script file.
;
;This should produce the "white.prg"-file which is the demo.
;"white.sym" is the symbol table,
;"white.lst" is the assembly listing,
;"white.bin" is the non-crunched demo.
;Drag and drop this file into a running WinVice v1.12 window,
;press alt-w momentarily (twice) to warp through the decrunching, and
;the demo is running.
;
;The source code is the file "white.s".
;
;The other .bat-files are for crunching various images in the demo,
;and you'll find the source files for those in the "images" folder.
;
;###
;### What to do next?
;###
;
;Make your own Commodore 64 demo? Well, why not. With this
;source code, you'll get a head start.
;
;Anyway, do whatever you like with this source code. If you
;want to steal or borrow whatever routines or code, please
;go ahead!
;
;###
;### Where am I?
;###
;
;You'll probably get hold of me at
;http://mahoney.c64.org
;
;Or you could try an email to:
;mahoney@c64.org
;
;Download my remixes from http://remix.kwed.org - search for "Mahoney"
;
;
;Have a noise night!
;
;###
;### Pex "Mahoney" Tufvesson
;### http://mahoney.c64.org
;###

;Why is the source code a mess?
; - well, this demo uses 62kB memory in a 64kB computer.
;   which means, that some routines are split into different
;   parts of memory. The Commodore 64 has a lot of restrictions
;   on where to put graphics and sprites in memory, which means
;   that the code has to nice fill all the holes between the
;   graphics to fit in.
;
;   The 6510 processor sometimes uses a different number of
;   clock cycles for an instruction, depending on where in memory
;   is it. This is for instance when a brach instruction crosses
;   a page (a 256-byte area). Since almost all graphics routines
;   in this demo has to be completely in sync with the TV display,
;   some seemingly unwanted relocation of code (such as org $9202
;   directly in the code) has to be used.
;
;   That's why it's a mess. Plus that when it's enough, then it's
;   enough, There's a lot of other nice things to do in this world
;   besides cleaning up assembly language code that's already
;   working. Don't fix it if it ain't broke!
;
;Where is the start of the demo?
; - You'll find the address in the a.bat file. It's 7168=$1c00
;   the actual code is at the label "real_start:" -search for it!
;
;Are there any hidden parts in there?
; - No, they didn't fit in memory! ;)
;
;Where do the different parts in the demo start?
; - They are all at the labels "start_"
;   start_load:      The first part, typing "LOAD",
;                    with the "take off your clothes"-text.
;   start_scroll:    The spritescroll on top of the C64-start screen.
;   start_image:     The PressPlayOnTape-image with the zoomscroll
;                    at the bottom.
;   start_spr:       The part with 8 sprites expanded over the whole
;                    screen "BitLive4", "Remix64.com", "Mahoney" and
;                    "Melina".
;   start_line:      The line that wanders through the screen
;                    white to the left, black to the right.
;                    Actually implemented by changing the background
;                    colour $d020 with the right timing!
;   start_up:        The endscroller.
;   start_fade:      The text that fades the screen from light blue
;                    to black.
;   start_varglad1:  The "Var glad fast du r svensk" image.
;   start_varglad2:  The image of a woman's face (Melina) with fading.


 seg.u zp
 org $0000
zp_start:  
  dc.b 0,0  ;Probably some use these, $01 for bank switching, $02 used to be free

;This is the text that is displayed on-screen
;while decrunching the demo. It will be placed
;on screen first, since the demo is decrunched
;from start to end!
 seg code2
 org $06a8
 dc.b $20, $10, $0c, $05, $01, $13, $05, $20
 dc.b $17, $01, $09, $14, $20, $04, $05, $03
 dc.b $12, $15, $0e, $03, $08, $09, $0e, $07
 dc.b $20, $02, $09, $14, $20, $0c, $09, $16
 dc.b $05, $34, $2d, $04, $05, $0d, $0f, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $04, $0f, $17, $0e, $0c, $0f
 dc.b $01, $04, $20, $14, $08, $05, $20, $13
 dc.b $0f, $15, $12, $03, $05, $20, $03, $0f
 dc.b $04, $05, $20, $06, $12, $0f, $0d, $3a
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $08, $14, $14, $10, $3a, $2f
 dc.b $2f, $17, $17, $17, $2e, $05, $0a, $05
 dc.b $13, $0f, $0e, $2e, $13, $05, $2f, $0d
 dc.b $01, $08, $0f, $0e, $05, $19, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $03, $0f, $0e, $14, $01
 dc.b $03, $14, $20, $0d, $05, $3a, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $0d, $01, $08, $0f, $0e
 dc.b $05, $19, $00, $0c, $09, $16, $05, $14
 dc.b $2e, $13, $05, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $20, $20, $20, $20, $20, $20, $20, $20
 dc.b $a0, $a0, $a0, $a0, $a0, $a0, $a0, $a0
 dc.b $a0, $a0, $a0, $a0, $a0, $a0, $a0, $ff
 dc.b $d0, $d0, $e4, $d0, $e3, $d0, $d0, $d0
  nop
irq_load4_raster = $46
irq_load4:
  sta savea
  stx savex
  sty savey
  tsx           ; 2
  lda #$ff
  sta $d017
  lda #$18  ;spr data at $07f8
  lda $d018

  nop
  bit $24       ; 5

  lda #<irq_load4a
  sta $fffe
  lda #>irq_load4a
  sta $ffff

  lda #$c1         ; waste at least 12 cycles
  lda $d010      ; (up to 64 cycles delay allowed here)
  lda #%11110000
  sta $d015
  inc $d012     ; At this stage, $d012 has already been incremented by one.
  lda #1
  sta $d019     ; acknowledge the first raster interrupt
  cli           ; enable interrupts (the second interrupt can now occur)


;Total cycles = 2+5*9-1 = 46 cycles
;  ldy #9  ;#2
;  dey     ;#2
;  bne *-1 ;#3 if succeeds      ; delay

;cycles = 2+8*4 = 34
  lda #$01  ;#2
  lda $d028 ;spr #1 colour
  lda $d029 ;spr #2 colour
  lda $d02a ;spr #3 colour
  lda $d02b ;spr #4 colour
  lda $d02c ;spr #5 colour
  lda $d02d ;spr #6 colour
  lda #1
  lda $d027 ;#4 ;spr #0 colour
  lda $d02e ;spr #7 colour
  nop
  nop
  nop
  nop
  nop


  nop           ; The second interrupt will occur while executing these
  nop           ; two-cycle instructions.
  nop
  nop
  nop
  nop

.error:
;  inc $d020
  jmp .error



 seg zp
rows: dc.b 0
 seg code2

irq_load4a:
;Now, get the bytes away from the stack that we won't use
  pla           ; 4
  txa           ; 2
  pla           ; 4
  tya           ; 2
  pla           ; 4
  tsx           ; 2
  nop
  nop           ; 4
  nop           ; 2
  nop
  bit $24       ; 5


  lda #<irq_spr1
  lda $fffe
  lda #>irq_spr1
  lda $ffff
  ldx $d012
  nop
  bit $24       ; 6569, 63 cycles/line
  cpx $d012     ; The comparison cycle is executed CYCLES or CYCLES+1 cycles
                ; after the interrupt has occurred.
  beq *+2       ; Delay by one cycle if $d012 hadn't changed.
                ; Now exactly CYCLES+3 cycles have passed since the interrupt.



  lda #spr_raster
  lda $d012     ; restore original raster interrupt position
  ldx #1
  stx $d019     ; acknowledge the raster interrupt
;  lda #10
; sta $d020
  nop
  nop
  nop
  lda #1
  lda IRQ_finished
  nop
  nop
  nop
  bit $24
  lda #14
  sta $d020
  lda #5
  sta rows
  ldx #$c8 ;First row shall not be opened

.load4_loop:
  nop
  lda #$c8
  stx $d016
  sta $d016

  ldy #7
.de:
  dey
  bne .de
  nop
  ldx #$c0

  nop
  nop
  stx $d016
  sta $d016,y

;Badline
  stx $d016
  sta $d016
  nop
  nop
  ldx #$c0
  ldy #6
.de2:
  dey
  bne .de2
  nop
  nop
  bit $24

  stx $d016
  sta $d016
  ldy #8
.de3:
  dey
  bne .de3
  bit $24

  stx $d016
  sta $d016
  ldy #8
.de4:
  dey
  bne .de4
  bit $24

  stx $d016
  sta $d016
  ldy #8
.de5:
  dey
  bne .de5
  bit $24

  stx $d016
  sta $d016
  ldy #8
.de6:
  dey
  bne .de6
  bit $24

  stx $d016
  sta $d016
  ldy #5
.de7:
  dey
  bne .de7
  bit $24
  nop
;  bit $24

  dec rows
  bne .load4_loop

  bit $24
  bit $24
  stx $d016
  sta $d016

  dec $d019                       ;Acknowledge raster interrupt
  lda #%00011011
  sta $d011
  lda #<irq_load1                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_load1
  sta $ffff
  lda #irq_load1_raster
  sta $d012

  lda #1
  sta IRQ_finished

  ldx spr_cou2
.bolo:
  lda sintbl,x
  bpl .n
  eor #$ff
.n:
  eor #$7f
  clc
  adc #$79
  sta $d001,y
  txa
  clc
  adc #$f5
  tax
  iny
  iny
  cpy #$10
  bne .bolo

;  lda #$80
;  sta $d001
;  sta $d003
;  sta $d005
;  sta $d007
;  sta $d009
;  sta $d00b
;  sta $d00d
;  sta $d00f
  lda #$19
  sta $d000
  lda #$32
  sta $d002
  lda #$59
  sta $d004
  lda #$89
  sta $d006
  lda #$b9
  sta $d008
  lda #$e9
  sta $d00a
  lda #$19
  sta $d00c
  lda #$37
  sta $d00e
  lda #$ff
  sta $d015
  sta $d017
  sta $d01d
  lda #%11000000
  sta $d010


;  lda #"C" - $40 + charset_start
  lda takeoff+11
  sta $07f8
;  lda #"L" - $40 + charset_start
  lda takeoff+12
  sta $07f9
;  lda #"O" - $40 + charset_start
  lda takeoff+13
  sta $07fa
;  lda #"T" - $40 + charset_start
  lda takeoff+14
  sta $07fb
;  lda #"H" - $40 + charset_start
  lda takeoff+15
  sta $07fc
;  lda #"E" - $40 + charset_start
  lda takeoff+16
  sta $07fd
;  lda #"S" - $40 + charset_start
  lda takeoff+17
  sta $07fe
;  lda #"Z" + 2 - $40 + charset_start
  lda takeoff+18
  sta $07ff

  jsr music_play

  lda savea
  ldx savex
  ldy savey
  rti


nmi_load:
  sta nmi_savea
  lda $dd0d ;ack
  lda #$7f                    ;CIA interrupt off
  sta $dd0d
  lda nmi_savea
  rti

irq_load1_raster = $fa
irq_load1:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%10010011
  sta $d011
  lda #<irq_load1a                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_load1a
  sta $ffff
  lda #irq_load1a_raster
  sta $d012
  nop
  nop
  nop
  lda #14
  lda $d021
;  lda #15
;  sta $d020

  lda #$80  ;Don't display sprites
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f


  lda savea
  ldx savex
  ldy savey
  rti

irq_load1a_raster = $20
irq_load1a:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
screen_blank = *+1
  lda #%00011011
  sta $d011
  lda #<irq_load2                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_load2
  sta $ffff
  lda #irq_load2_raster
  sta $d012

  lda #$10
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f

  lda #$fe
  sta $d015
  sta $d017
  sta $d01d
  lda #$00
  sta $d01c ;multicolor
  sta $d01b
  lda #$28
  sta $d000
  lda #$57
  sta $d002
  lda #$88
  sta $d004
  lda #$b8
  sta $d006
  lda #$e9
  sta $d008
  lda #$1a
  sta $d00a
  lda #%11100000
  sta $d010

  lda #charset_start
  sta $07f8
;  lda #"T" - $40 + charset_start
  lda takeoff+0
  sta $07f9
;  lda #"A" - $40 + charset_start
  lda takeoff+1
  sta $07fa
;  lda #"K" - $40 + charset_start
  lda takeoff+2
  sta $07fb
;  lda #"E" - $40 + charset_start
  lda takeoff+3
  sta $07fc
;  lda #"O" - $40 + charset_start
  lda takeoff+4
  sta $07fd
;  lda #"F" - $40 + charset_start
  lda takeoff+5
  sta $07fe
;  lda #"F" - $40 + charset_start
  lda takeoff+6
  sta $07ff

  lda #1
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

  jsr load1a_movex

  lda savea
  ldx savex
  ldy savey
  rti

irq_load2_raster = $3d
irq_load2:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%00011011
  sta $d011
  lda #<irq_load4                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_load4
  sta $ffff
  lda #irq_load4_raster
  sta $d012

  lda #$49
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f

;  lda #%11000000
;  sta $d010
;  ldx #charset_start
;  stx $07f8
;  stx $07f9
;  stx $07fa
;  stx $07fb
;  lda #"Z" + 9 - $40 + charset_start
;  sta $07fc
;  lda #"Z" + 10 - $40 + charset_start
;  sta $07fd
;  lda #"Z" + 11 - $40 + charset_start
;  sta $07fe
;  sta $07ff


  jsr load3_movex

;  lda #"Y" - $40 + charset_start
  lda takeoff+7
  sta $07fc
;  lda #"O" - $40 + charset_start
  lda takeoff+8
  sta $07fd
;  lda #"U" - $40 + charset_start
  lda takeoff+9
  sta $07fe
;  lda #"R" - $40 + charset_start
  lda takeoff+10
  sta $07ff

;  lda #6
;  lda $d021
  lda #$0f
  sta $d015

  lda savea
  ldx savex
  ldy savey
  rti







load1a_movex:
;Now, move the sprites in the x-direction:
  lda #$00
  sta spr_x_msb
  lda #$19
  sta spr_x_lsb
  inc spr_cou
  inc spr_cou

  ldx spr_cou
  ldy #0

.xset_lop:
  lda costbl,x
  cmp #$80
  ror
  cmp #$80
  ror
;  cmp #$80
;  ror
  clc
;  adc #$10
  adc #$20

  clc
  adc spr_x_lsb
  sta $d002,y
  lda spr_x_msb
  adc #0
;  cmp #$ff
;  bne .nofix_for_left_border
;  lda $d004,y
;  sec
;  sbc #8
;  sta $d004,y
;  lda #$ff
;.nofix_for_left_border:
;  and #$01
  ror
  ror $d010

  lda sintbl,x
  cmp #$80
  ror
  cmp #$80
  ror
  cmp #$80
  ror
  cmp #$80
  ror
  clc
  adc #$10
  sta $d003,y

  txa
  clc
  adc #$e3
  tax

  lda spr_x_lsb
  clc
  adc #$1d
  sta spr_x_lsb
  lda spr_x_msb
  adc #0
  sta spr_x_msb

  cpy #6
  bne .no_space
  lda spr_x_lsb
  clc
  adc #$24
  sta spr_x_lsb
  lda spr_x_msb
  adc #0
  sta spr_x_msb
.no_space:

  iny
  iny
  cpy #$e
  bne .xset_lop


;  lda #5
;  sta $d020
  rts












load3_movex:
;Now, move the sprites in the x-direction:
  inc spr_cou2
  lda spr_cou2
  clc
  adc #$c8
  tax
  lda sintbl,x
  clc
  adc #$80

  clc
  adc #$58
  sta spr_x_lsb
  lda #$ff
  adc #0
  sta spr_x_msb



  ldx spr_cou2
  txa
  bpl .otherhalf
  asl
  tax
  lda costbl,x
  cmp #$80
  ror
  clc
  adc #$40
  jmp .cont
.otherhalf:
  asl
  tax
  lda costbl,x
  eor #$ff
  cmp #$80
  ror
  clc
  adc #$c0
;  lda #0
;  jmp .cont
.cont:
  clc
  adc spr_x_lsb
  sta spr_x_lsb
  lda spr_x_msb
  adc #0
  sta spr_x_msb

  ldy #0

.xset3_lop:
  lda spr_x_lsb
  sta $d008,y
  lda spr_x_msb
  cmp #$ff
  bne .nofix_for_left_border
  lda $d008,y
  sec
  sbc #8
  sta $d008,y
  lda #$ff
.nofix_for_left_border:
;  and #$01
  ror
  ror $d010

  lda spr_x_lsb
  clc
  adc #$31
  sta spr_x_lsb
  lda spr_x_msb
  adc #0
  sta spr_x_msb

  iny
  iny
  cpy #$8
  bne .xset3_lop

;  lda $d010
;  ror
;  ror
;  and #%00110000
;  ora #%11000000
;  sta $d010

;  lda #5
;  sta $d020
  rts










start_load: subroutine
  lda #$7f       ; turn off the CIA interrupts
  sta $dc0d
  sta $dd0d ;Clear NMI
  sei
  cld
  ldx #$ff
  txs
  lda #14
  sta $d020
  lda #14
  sta $d021
;Clear the takeoff-text:
  ldx #0
  lda #charset_start
.l:
  sta takeoff,x
  inx
  cpx #takeoff_end-takeoff
  bne .l
.wait_screen:
  lda $d012
  cmp #$a0
  bne .wait_screen
  lda #$35
  sta $01
  lda #%00000011
  sta $dd00
  lda #$15  ;spr data at $07f8
  sta $d018
  lda #$c8
  sta $d016

  lda #<irq_load1
  sta $fffe
  lda #>irq_load1
  sta $ffff
  lda #<nmi_load                       ;Set NMI, both the hardware
  sta $fffa                       ;KERNAL
  lda #>nmi_load
  sta $fffb
  lda #$1b
  sta $d011     ; set the raster interrupt location
  lda #irq_load1_raster
  sta $d012
  lda #0
  sta part_finished

  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d
  ldx #1
  stx $d01a     ; enable raster interrupt
  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts
  ldy #$00
  sty $d015     ; turn off all sprites

  jsr music_play

  ldx #0
.clr_screen:  
  lda text,x
  and #$3f
  ora #$80
  sta $0400,x
  lda #' +$80
;  lda #' 
  sta $0500,x
  sta $0600,x
  sta $06f7,x
  lda #6
  sta $d800,x
  sta $d900,x
  sta $da00,x
  sta $daf8,x
  inx
  bne .clr_screen

  jsr cursor_init

already_started_flag = *+1
  lda #0
  bne .already_started

  ldx #79
.pr:
  lda textpressspace,x
  and #$3f
  ora #$80
  sta $0400+400,x
  dex
  bpl .pr

.nokey
  lda #$ff
.wai7:
  cmp $d012
  bne .wai7
  lda #$08
  sta cursor_wait
  jsr force_cursor

  dec cursor_erase_press_space
  bne .noclr
  jsr clr_pressspace
.noclr:

  ldx #10
.waiit:
  dex
  bne .waiit

  lda $dc01
  and #$10
  bne .nokey

  jsr clr_pressspace

.already_started:
  lda #1
  sta already_started_flag
  lda #$c0
  sta spr_cou2
  sta spr_cou
  cli


  lda #0
  sta IRQ_finished
  sta part_finished

.wai:
  lda IRQ_finished
  beq .wai

  lda #0
  sta IRQ_finished
  jsr load_cursor
  lda part_finished
  beq .wai


;  jmp .wai
;waiting_for_Godot:
;  jmp waiting_for_Godot

  jmp next_demo_part

clr_pressspace:
  ldx #79
  lda #' +$80
.clrpr:
  sta $0400+400,x
  dex
  bpl .clrpr
  rts

 seg zp
load_cursor_cou: dc.b 0
cursor: dc.b 0,0
cursor_poi: dc.b 0
cursor_wait: dc.b 0
cursor_on: dc.b 0
cursor_erase_press_space: dc.b 0
 seg code2

cursor_init:
  lda #$f0
  sta cursor
  sta cursor_erase_press_space
  lda #$04
  sta cursor+1
  lda #0
  sta load_cursor_cou
  sta cursor_poi
  sta takeoff_poi
  lda #$80
  sta takeoff_cou
  lda #$48
  sta cursor_wait
  lda #1
  sta cursor_on
  rts

 seg zp
takeoff_cou: dc.b 0
takeoff_poi: dc.b 0
 seg code2

.print_takeoff:
  dec takeoff_cou
  bne .rts2

  lda takeoff_poi
  bmi .erase

  lda #$12
  sta takeoff_cou
  ldx takeoff_poi
  lda takeoff_text,x
  sta takeoff,x
  inx
  cpx #19
  bne .nowait2
  lda #$90
  sta takeoff_cou
.nowait2:
  cpx #20
  bne .noend
  ldx #$80
.noend:
  stx takeoff_poi
  rts

.erase:
  and #$7f
  tax
  lda #$03
  sta takeoff_cou

  lda #charset_start
  sta takeoff,x
  inx
  cpx #19
  bne .noend2
  lda #1
  sta part_finished
.noend2:
  txa
  ora #$80
  sta takeoff_poi


;  lda #1
;  sta part_finished
.rts2:
  rts

load_cursor:
  lda cursor_on
  beq .print_takeoff
force_cursor:
  inc load_cursor_cou
  lda load_cursor_cou
  cmp #17
  bne .no_blink
  ldy #0
  sty load_cursor_cou
  lda (cursor),y
  eor #$80
  sta (cursor),y
.no_blink:

  dec cursor_wait
  bne .rts
  ldx cursor_poi
.more:
  lda cursor_text,x
  cmp #$ff
  bne .nowrap
  ldx #0
  lda #$f0
  sta cursor
  lda #$04
  sta cursor+1
  lda #0
  sta cursor_on
  rts
;  lda cursor_text,x
.nowrap:
  bpl .nowait
  cmp #$fe
  bne .no_disableblink
  ldy #$80
  sty load_cursor_cou
.no_disableblink:
  and #$7f
  sta cursor_wait
  jmp .cend
.nowait:
  cmp #$7d
  bne .no_blank
;  lda #14
;  sta screen_colour
  lda #%00001011
  sta screen_blank
  inx
  jmp .more
.no_blank:
  cmp #$7e
  bne .no_noblank
;  lda #6
;  sta screen_colour
  lda #%00011011
  sta screen_blank
  inx
  jmp .more
.no_noblank:
  cmp #$0d
  bne .no_enter
  ldy #0
  lda #' +$80
  sta (cursor),y
  inx
  lda cursor_text,x
  clc
  adc cursor
  sta cursor
  lda cursor+1
  adc #0
  sta cursor+1
  inx
  jmp .more
.no_enter:
  ldy #0
  and #$3f
  ora #$80
  sta (cursor),y
  inc cursor
  bne .nowr3
  inc cursor+1
.nowr3:
  inx
  jmp .more
.cend:
  inx
  stx cursor_poi

.rts:
  rts
  
cursor_text:
  dc.b "L",$93,"O",$b2,"A",$99,"D",$c4
  dc.b $0d,40+36,"PRESS PLAY ON TAPE",$fe
  dc.b $0d,22,"OK",$82
  dc.b $7d,$fe     ;Screen blanked
  dc.b $7e,$0d,40+38,"SEARCHING"
  dc.b $0d,31,"FOUND PPOT BEN ROB",$fe
  dc.b $0d,22,"LOADING"
  dc.b $ff

takeoff:
  dc.b charset_start,charset_start,charset_start,charset_start
  dc.b charset_start,charset_start,charset_start
  dc.b charset_start,charset_start,charset_start,charset_start
  dc.b charset_start,charset_start,charset_start,charset_start
  dc.b charset_start,charset_start,charset_start,charset_start
takeoff_end:
  dc.b 0 ;This one gets overwritten by accident... hmmm

takeoff_text:
  dc.b #"T" - $40 + charset_start
  dc.b #"A" - $40 + charset_start
  dc.b #"K" - $40 + charset_start
  dc.b #"E" - $40 + charset_start
  dc.b #"O" - $40 + charset_start
  dc.b #"F" - $40 + charset_start
  dc.b #"F" - $40 + charset_start
  dc.b #"Y" - $40 + charset_start
  dc.b #"O" - $40 + charset_start
  dc.b #"U" - $40 + charset_start
  dc.b #"R" - $40 + charset_start
  dc.b #"C" - $40 + charset_start
  dc.b #"L" - $40 + charset_start
  dc.b #"O" - $40 + charset_start
  dc.b #"T" - $40 + charset_start
  dc.b #"H" - $40 + charset_start
  dc.b #"E" - $40 + charset_start
  dc.b #"S" - $40 + charset_start
  dc.b #"Z" + 2 - $40 + charset_start

;###
;### drawsprites
;###

 seg zp
drawspr_poi:  dc.b 0,0
drawspr_src:  dc.b 0,0
drawspr_char: dc.b 0
drawspr_y:    dc.b 0
drawspr_wait: dc.b 0
one_left:     dc.b 0  ;Makes the ytables show twice.
 seg code2

empty_ytable:
  ds.b $20,$ff

spr_drawsprites_init: subroutine
  stx drawspr_char

  lda #$10
  sta drawspr_wait
  lda #$3e
  sta drawspr_poi+1
  lda #0
  sta drawspr_poi

.calc_src:
  lda #0
  sta drawspr_src
  ldx drawspr_char
  lda spr_text,x
  ror
  ror drawspr_src
  ror
  ror drawspr_src
  and #$3f
  sta drawspr_src+1
  lda #0
  sta drawspr_y
.rts:
  rts

spr_drawsprites:
  ldy drawspr_y
  bmi .wait_for_a_while
  dec drawspr_wait
  bne .rts
  inc drawspr_wait
  lda (drawspr_src),y
  sta (drawspr_poi),y  
  iny
  lda (drawspr_src),y
  sta (drawspr_poi),y  
  iny
  lda (drawspr_src),y
  sta (drawspr_poi),y  
  iny
  cpy #63
  bne .nofin
;  ldy #0
  lda drawspr_poi
  clc
  adc #$40
  sta drawspr_poi
  lda drawspr_poi+1
  adc #0
  sta drawspr_poi+1
  inc drawspr_char
  lda drawspr_char
  tax
  and #$07
  bne .calc_src
;We are done with painting all 8 sprites.
  lda #$80
  sta drawspr_y
  lda #$00
  sta drawspr_wait
  lda #0
  sta drawspr_char
  rts
.nofin:
  sty drawspr_y
.hrts:
  rts

.wait_for_a_while:
  dec drawspr_wait
  bne .rts
  cpy #$80
  bne .enn
  lda spr_version_number
  bpl .hrts  ;Don't erase the sprites. Just do nothing.

  iny
  sty drawspr_y
  lda #$40
  sta drawspr_wait
  lda #>spr_to_draw_in
  sta drawspr_poi+1
  lda #<spr_to_draw_in
  sta drawspr_poi
.enn:
  inc drawspr_wait

  ldy #0
  lda #0
  sta (drawspr_poi),y
  iny
  sta (drawspr_poi),y
  iny
  sta (drawspr_poi),y

  lda drawspr_poi
  clc
  adc #$40
  sta drawspr_poi
  lda drawspr_poi+1
  adc #0
  sta drawspr_poi+1

  cmp #2+>spr_to_draw_in
  bne .noennd
  lda drawspr_poi
  sec
  sbc #$fd
  sta drawspr_poi
  tay
  lda drawspr_poi+1
  sbc #$01
  sta drawspr_poi+1
  cpy #$3f
  bne .noennd
  lda #1
  sta part_finished
  rts
.noennd:
  
.ok:
  stx drawspr_char
  rts

;###
;###
;###

real_start:
  jsr music_init
  jsr random_init
  lda #1
  sta constant_one
  lda #$1a
  sta constant_0x1f

;global_init:
  lda #0          ;Start from the beginning of the scroll
  sta pos_textcou

;Testing!

  lda #0
  sta demo_loop_cou

  lda #0
  sta part_inverted
;  jmp start_spr


next_demo_part:
demo_loop:
  sei
  lda #$35
  sta $01
  ldx demo_loop_cou
  lda demo_loop_data,x
  bpl .nodemowrap
  ldx #0
  lda demo_loop_data,x
.nodemowrap:
  inx
  stx demo_loop_cou

  cmp #0
  bne .noscroll
  lda #1
  sta music_enable
  lda #14
  sta part_bgcol ;for spr below...!
  jmp start_scroll

.noscroll:
  cmp #1
  bne .noline1
  lda #1
  sta part_bgcol ;for spr below...!
  lda #$00
  sta line_x_spd_msb
  lda #$40
  sta line_x_spd_lsb
  lda #$00
  sta line_x_lsb
  lda #$80
  sta line_x_msb
  lda #0
  sta line_xadd_lsb
  lda #$d0
  sta line_xadd_msb
  lda #$0c
  sta line_finito
  lda #$28
  sta line_xadd_spd_lsb
  lda #$00
  sta line_xadd_spd_msb
  lda #0
  sta line_cou
  lda #$20
  sta line_cou2
  jmp start_line

.noline1:
  cmp #2
  bne .nobitlive4
;Bit7: '0'=end by ytables-wrap the second time
;Bit7: '1'=end by erasing the sprites one by one row
;Bit5-3: "000"=Bounce
;Bit5-3: "001"=Starting in the middle, growing and down
;Bit2-0: "000"=Mahoney
;Bit2-0: "001"=Be Nice
;Bit2-0: "010"=BitLive4
;Bit2-0: "011"=Comeback64
;Bit2-0: "100"=Melina
;Bit2-0: "101"=Remix64.com
;Bit2-0: "110"=c64power
;Bit2-0: "111"=commodore
;  lda #%00000100
;  lda #%00001100 ;Melina
  lda #%00001010 ;bitlive4
;  lda #%10000010 ;bitlive4
;  lda #%00000000
;  lda random
  sta spr_version_number
  jsr new_random
  jmp start_spr 

.nobitlive4:
;ERROR:
  cmp #3
  bne .noload
  jmp start_load

.noload:
  cmp #4
  bne .noremix64
  lda #%10000101
  sta spr_version_number
  jsr new_random
  jmp start_spr 

.noremix64:
  cmp #5
  bne .noimage
  jmp start_image

.noimage:
  cmp #6
  bne .nomahoney
  lda #0
  sta part_bgcol
  lda #%10010000
  sta spr_version_number
  jmp start_spr 

.nomahoney:

  cmp #7
  bne .nomelina
  lda #0
  sta part_bgcol
  lda #%00001100
  sta spr_version_number
  jmp start_spr 

.nomelina:
  cmp #8
  bne .noupscroll
  jmp start_up 

.noupscroll:
  cmp #9
  bne .nofaderut
  jmp start_fade

.nofaderut:

  cmp #10
  bne .noline2
;  lda #$00
; sta line_x_spd_msb
;  lda #$00
; sta line_x_spd_lsb
;  lda #$00
; sta line_x_lsb
;  lda #$24
; sta line_x_msb
;Startpos i x-led
  lda #$ff
  sta line_xadd_lsb
  lda #$0d
  sta line_xadd_msb
  lda #$ea
  sta line_finito

  lda #$e2
  sta line_xadd_spd_lsb
  lda #$ff
  sta line_xadd_spd_msb

  lda #$6c
  sta line_cou
  lda #$68
  sta line_cou2
  jmp start_line

.noline2:
  cmp #11
  bne .novarglad1
  jmp start_varglad1

.novarglad1:
  cmp #12
  bne .novarglad2
  jmp start_varglad2

.novarglad2:

  lda #$0b
  sta $d011
  lda #0
  sta $d015
.ever:
  lda #14
  sta $d020
  lda #6
  sta $d020
  lda #0
  sta $d020
  ldx #4
.g:
  dex
  bne .g


  jmp .ever

;This is a list with the order of the parts.
;The decoding of this list is above.
demo_loop_data:
  dc.b 3,0,2,9,5,1,12,4,10,6,11,7,8,$ff


;Get a good seed for the random number generation:
random_init:
  lda #$80
  sta $d412 ;choose noise waveform on SID
  sta $d40f ;$81xx as frequency
  lda $d012 ;read raster position
  eor $d011 ;raster MSB
  eor $dc00
  eor $dc01
  eor $dc04
  eor $dc05
  eor $dc06
  eor $dc07
  eor $dc08
  eor $dd00
  eor $dd01
  eor $dd04
  eor $dd05
  eor $dd06
  eor $dd07
  eor $dd08
  eor $d41b ;read noise from SID
  sta random
  rts


new_random:
  lda random
  and #$b8

  ldx #5
  ldy #0
.loop:
  asl
  bcc .bit_clr
  iny
.bit_clr:
  dex
  bne .loop
.no_clr:
  tya
  lsr
  lda random
  rol
  sta random
  rts

random:
  dc.b 0

;###
;### pudecrunch for the multicol4bitmap image
;###

 subroutine
 seg zp
LZPOS: dc.b 0,0 ;2 ZeroPage temporaries
bitstr: dc.b 0  ;1 temporary (does not need to be ZP)
 seg code2

; SHORT+IRQLOAD   354 bytes
; no rle =~   -83 bytes -> 271
; fixed params =~ -48 bytes -> 306
;     223 bytes

SHORT = 0 ;1  ; assume file is ok
IRQLOAD = 0 ;1


decrunch:
  ldx #>(multicol4bitmap_pu+2)
  ldy #<(multicol4bitmap_pu+2)

pu:
  ; Call with X = HI of packed data, Y = LO of packed data
  ; Returns exec address in X = HI and Y = LO
  ; Carry will be set for error, cleared for OK

  sty INPOS
  stx INPOS+1

#if SHORT
  ldx #5
222$  jsr getbyt  ; skip 'p', 'u', endAddr HI&LO, leave starting escape in A
  dex
  bne 222$
#else
  jsr getbyt  ; 'p'
  cmp #112
  beq 9$
  sec   ; error
  jmp eof2
9$  jsr getbyt  ; 'u'
  cmp #117
  beq 8$
  sec   ; error
  jmp eof2
8$  jsr getbyt  ; skip endAddr
  jsr getbyt
  jsr getbyt
#endif
  sta esc+1 ; starting escape

  jsr getbyt  ; read startAddr
  sta OUTPOS
  jsr getbyt
  sta OUTPOS+1
  jsr getbyt  ; read # of escape bits
  sta escB0+1
  sta escB1+1
  lda #8
  sec
  sbc escB1+1
  sta noesc+1 ; 8-escBits

  jsr getbyt
  sta mg+1  ; maxGamma + 1
  lda #9
  sec
  sbc mg+1  ; 8 - maxGamma == (8 + 1) - (maxGamma + 1)
  sta longrle+1
  jsr getbyt
  sta mg1+1 ; (1<<maxGamma)
  asl
  clc
  sbc #0
  sta mg21+1  ; (2<<maxGamma) - 1
  jsr getbyt
  sta elzpb+1

  jsr getbyt  ; exec address
  sta lo+1  ; lo
  jsr getbyt
  sta hi+1  ; hi

  jsr getbyt  ; rleUsed
  ldx #0
  tay
0$  beq 1$    ; Y == 0 ?
  jsr getbyt
  sta table,x
  inx
  dey
  bne 0$
1$  ; setup bit store - $80 means empty
  lda #$80
  sta bitstr
  jmp main

getbyt:
  jsr getnew
  lda bitstr
  ror
  rts


newesc  ldy esc+1 ; remember the old code (top bits for escaped byte)
escB0 ldx #2    ; ** PARAMETER  0..8
  jsr getchkf ; get & save the new escape code
  sta esc+1
  tya   ; pre-set the bits
  ; Fall through and get the rest of the bits.
noesc ldx #6    ; ** PARAMETER  8..0
  jsr getchkf
  jsr putch ; output the escaped/normal byte
  ; Fall through and check the escape bits again
main  ldy #0    ; Reset to a defined state
  tya   ; A = 0
escB1 ldx #2    ; ** PARAMETER  0..8
  jsr getchkf ; X = 0
esc cmp #0
  bne noesc
  ; Fall through to packed code

  jsr getval  ; X = 0
  sta LZPOS ; xstore - save the length for a later time
  lsr   ; cmp #1  ; LEN == 2 ? (A is never 0)
  bne lz77  ; LEN != 2  -> LZ77
  ;tya    ; A = 0
  jsr get1bit ; X = 0
  lsr   ; bit -> C, A = 0
  bcc lz77_2  ; A=0 -> LZPOS+1
  ;***FALL THRU***

  ; e..e01
  jsr get1bit ; X = 0
  lsr   ; bit -> C, A = 0
  bcc newesc  ; e..e010
  ;***FALL THRU***

  ; e..e011
srle  iny   ; Y is 1 bigger than MSB loops
  jsr getval  ; Y is 1, get len, X = 0
  sta LZPOS ; xstore - Save length LSB
mg1 cmp #64   ; ** PARAMETER 63-64 -> C clear, 64-64 -> C set..
  bcc chrcode ; short RLE, get bytecode

longrle ldx #2    ; ** PARAMETER  111111xxxxxx
  jsr getbits ; get 3/2/1 more bits to get a full byte, X = 0
  sta LZPOS ; xstore - Save length LSB

  jsr getval  ; length MSB, X = 0
  tay   ; Y is 1 bigger than MSB loops

chrcode jsr getval  ; Byte Code, X = 0
  tax   ; this is executed most of the time anyway
  lda table-1,x ; Saves one jump if done here (loses one txa)

  cpx #32   ; 31-32 -> C clear, 32-32 -> C set..
  bcc 1$    ; 1..31, we got the right byte from the table

  ; Ranks 32..64 (11111xxxxx), get byte..
  txa   ; get back the value (5 valid bits)
  ldx #3
  jsr getbits ; get 3 more bits to get a full byte, X = 0

1$  ldx LZPOS ; xstore - get length LSB
  inx   ; adjust for cpx#$ff;bne -> bne
dorle jsr putch
  dex
  bne dorle ; xstore 0..255 -> 1..256
  dey
  bne dorle ; Y was 1 bigger than wanted originally
mainbeq beq main  ; reverse condition -> jump always


lz77  jsr getval  ; X = 0
mg21  cmp #127  ; ** PARAMETER  Clears carry (is maximum value)
  bne noeof
  ; EOF
eof
#if SHORT == 0
  ; EOF
eof clc
eof2
#endif
#if IRQLOAD
  ;jsr endloader
lo = .+1
hi = .+2
  jmp $aaaa
#else
hi  ldx #0
lo  ldy #0
  rts
#endif


noeof sbc #0    ; C is clear -> subtract 1  (1..126 -> 0..125)
elzpb ldx #0    ; ** PARAMETER (more bits to get)
  jsr getchkf ; clears Carry, X = 0

lz77_2  sta LZPOS+1 ; offset MSB
  jsr getbyte ; clears Carry, X = 0
  ; Note: Already eor:ed in the compressor..
  ;eor #255 ; offset LSB 2's complement -1 (i.e. -X = ~X+1)
  adc OUTPOS  ; -offset -1 + curpos (C is clear)
  ldx LZPOS ; xstore = LZLEN (read before it's overwritten)
  sta LZPOS

  lda OUTPOS+1
  sbc LZPOS+1 ; takes C into account
  sta LZPOS+1 ; copy X+1 number of chars from LZPOS to OUTPOS
  ;ldy #0   ; Y was 0 originally, we don't change it

  inx   ; adjust for cpx#$ff;bne -> bne
lzloop  lda (LZPOS),y ; using abs,y is 3 bytes longer, only 1 cycle/byte faster
  jsr putch ; Note: must be copied forwards!
  iny   ; Y does not wrap because X=0..255 and Y initially 0
  dex
  bne lzloop  ; X loops, (256,1..255)
  beq mainbeq ; jump through another beq (-1 byte, +3 cycles)


getnew:
  pha   ; 1 Byte/3 cycles

#if IRQLOAD
  ;jsr READBYTE ; should not change X or Y, return byte in A
  ; An implementation would first check if any bytes are available
  ; and if not, transfer a block from drive.
#else
;  sei
;  lda #$34
;  sta $01
INPOS = *+1
  lda multicol4bitmap_pu+2  ; ** PARAMETER
;  pha
;  inc $01
;  pla
;  cli
  inc INPOS
  bne 0$
  inc INPOS+1
#endif
0$  sec
  rol   ; Shift out the next bit and
      ;  shift in C=1 (last bit marker)
  sta bitstr  ; bitstr initial value = $80 == empty
  pla   ; 1 Byte/4 cycles
  rts
  ; 25+12 = 37

; getval : Gets a 'static huffman coded' value
; ** Scratches X, returns the value in A **
getval  inx   ; X <- 1
  txa   ; set the top bit (value is 1..255)
gv0 asl bitstr
  bne 1$
  jsr getnew
1$  bcc getchk  ; got 0-bit
  inx
mg  cpx #7    ; ** PARAMETER unary code maximum length + 1
  bne gv0
  beq getchk  ; inverse condition -> jump always
  ; getval: 18 bytes
  ; 15 + 17*n + 6+15*n+12 + 36*n/8 = 33 + 32*n + 36*n/8 cycles

; getbits: Gets X bits from the stream
; ** Scratches X, returns the value in A **
getbyte ldx #7
get1bit inx   ;2
getbits asl bitstr
  bne 1$
  jsr getnew
1$  rol   ;2
getchk  dex   ;2    more bits to get ?
getchkf bne getbits ;2/3
  clc   ;2    return carry cleared
  rts   ;6+6


OUTPOS = *+1    ; ZP
putch sta $aaaa ; ** parameter
  inc OUTPOS  ; ZP
  bne 0$
  inc OUTPOS+1  ; ZP
0$
decrunch_slow:  ;This is used for "delaying" the drawing of the
                ;PressPlayOnTape image
  pha
  lda #$90
.slower:
  clc
  adc #1
  bne .slower
  pla
  rts



table dc.b 0,0,0,0,0,0,0
  dc.b 0,0,0,0,0,0,0,0
  dc.b 0,0,0,0,0,0,0,0
  dc.b 0,0,0,0,0,0,0,0


;###
;###
;###

;This is the part with the zoom scroller and the
;press play on tape-image

start_image: subroutine
  lda #$7f       ; turn off the CIA interrupts
  sta $dc0d
  sta $dd0d ;Clear NMI
  sei
  cld
  ldx #$ff
  txs
  lda #0
  sta $d020
  sta $d021
  lda #$35
  sta $01
  lda #$15  ;spr data at $07f8
  sta $d018

  jsr justplay_init
  jsr image_scroller_init
  ldx #0
.copy_sprs:
  lda Sprites_for_the_ppot_image,x
  sta $7000,x
  lda Sprites_for_the_ppot_image+$100,x
  sta $7100,x
  inx
  bne .copy_sprs

  lda #0
  sta imaclr_poi
  lda #$40
  sta imaclr_poi+1
  ldy #0
  lda #0
  ldx #$1c
.filop:
  sta (imaclr_poi),y
  iny
  bne .filop
  inc imaclr_poi+1
  dex
  bne .filop

;  ldx #0
;  lda #' 
;.l3:
;  sta $0400,x
;  sta $0500,x
;  sta $0600,x
;  sta $06f7,x
;  inx
;  bne .l3

  lda #$18
  sta $d011

  lda #%00101000
  sta $d011
  lda #$15
  sta $d018     ;Screen at $6000
  lda #%00000011
  sta $dd00
  lda #$c8
  sta $d016


  ldx #0
;  lda #$0
  lda #$f
.l:
  sta $d800,x
  sta $d900,x
  sta $da00,x
;  sta $daf7,x
  inx
  bne .l

  ldx #0
  lda #$bc
;  lda #$0
.l2:
  sta $6000,x
  sta $6100,x
  sta $6200,x
;  sta $62f7,x
  inx
  bne .l2

  ldx #$f0
  lda #0
.l4:
  sta $daf8,x
  sta $62f8,x
  inx
  cpx #$f0
  bne .l4


  lda #$00
  sta image_wait
  sta image_wait+1

  sei
  lda #<irq_image2
  sta $fffe
  lda #>irq_image2
  sta $ffff
  lda #<nmi_load                  ;Set NMI
  sta $fffa                       
  lda #>nmi_load
  sta $fffb
  lda #$0b
  sta $d011     ; set the raster interrupt location
  lda #irq_image2_raster
  sta $d012
  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d
  ldx #1
  stx $d01a     ; enable raster interrupt
  ldy #$00
  sty $d015     ; turn off all sprites
  lda #0
  sta IRQ_finished
  sta part_finished

  lda #%00101000
  sta $d011
  lda #%10000000
  sta $d018     ;Screen at $6000
  lda #%00011000
  sta $d016

  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts
  cli

  ldx #$08
  stx img0a_raster

  ldy #$89
.next_row:
  lda #$80
.notyet:
  cmp $d012
  bne .notyet

  lda #$88
.notyet2:
  cmp $d012
  bne .notyet2

  stx img0a_raster

  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop


  dey
  bne .waa
  ldy #$04
  inx
.waa:
  cpx #$30
  bne .next_row

  lda #$48  ;opcode for pha
  sta decrunch_slow
  jsr decrunch
  jmp ppot_image_faderut

 seg zp
image_wait: dc.b 0,0
imaclr_poi: dc.b 0,0
 seg code2

irq_image0_raster = $00
irq_image0:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%00111000
  sta $d011
  lda #<irq_image0a                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_image0a
  sta $ffff
img0a_raster = *+1
  lda #irq_image0a_raster
  sta $d012

  lda #$0
  sta $d020
  sta $d021

  lda #$08
  sta $d00f

  lda #$18
  sta $d000
  lda #$48
  sta $d002
  lda #$78
  sta $d004
  lda #$a8
  sta $d006
  lda #$d8
  sta $d008
  lda #$08
  sta $d00a
  lda #$38
  sta $d00c
  lda #$18
  sta $d00e
  lda #%01100000
  sta $d010

  lda #%10000000
  sta $d018     ;Screen at $6000

  lda #$1a
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d

  lda #$ff  ;8 sprites visible
  sta $d015

  ldx #$c0
  stx $63f8
  inx
  stx $63f9
  inx
  stx $63fa
  inx
  stx $63fb
  inx
  stx $63fc
  inx
  stx $63fd
  inx
  stx $63fe
  inx
  stx $63ff

  lda #$00
  sta $d017
  lda #$ff
  sta $d01d
  lda #%00011000
  sta $d016

  lda #$b
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

  lda #%00000010
  sta $dd00

  jsr image_scroller
  jsr image_scroller_setx


  lda savea
  ldx savex
  ldy savey
  rti


irq_image0a_raster = $08
irq_image0a:
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%00111000
  sta $d011
  lda #<irq_image1                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_image1
  sta $ffff
  lda #irq_image1_raster
  sta $d012

  lda #$0
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

  jsr music_play

  lda savea
  ldx savex
  ldy savey
  rti


;  ds.b $c0,0
 org $14f9
irq_image1_raster = $c5
irq_image1:
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #<irq_image1a                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_image1a
  sta $ffff
  inc $d012
  cli
  lda #$2
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
.roll:
  sta $d021
  jmp .roll

irq_image1a:
  pla
  pla
  pla

  dec $d019                       ;Acknowledge raster interrupt
  lda #<irq_image0                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_image0
  sta $ffff
  lda #irq_image0_raster
  sta $d012
  lda image_scroll_xposes+8
  sta $d010

;  nop
;  nop
;  nop


  bit $00
  bit $00
;  lda #$5
;  sta $d020
;  sta $d021

  lda #$84
  sta $d018
  lda #$cc
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f
;  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
; nop
  bit $24

  ldx #$0
  stx $d02e
;  stx $d020
;  stx $d021

  lda #$18
  sta $d011
  lda #%00000011
  sta $dd00

  lda image_scroll_xposes+0
  sta $d000
  lda image_scroll_xposes+1
  sta $d002
  lda image_scroll_xposes+2
  sta $d004
  lda image_scroll_xposes+3
  sta $d006
  lda image_scroll_xposes+4
  sta $d008
  lda image_scroll_xposes+5
  sta $d00a
  lda image_scroll_xposes+6
  sta $d00c
  lda image_scroll_xposes+7
  sta $d00e

  lda #$ff
  sta $d015
  sta $d01d
  lda #0
  sta $3fff

  lda #$c8
  sta $d016


  lda #$15  ;spr data at $07f8
  sta $d018

  ldx #$90
  stx $63f8
  inx
  stx $63f9
  inx
  stx $63fa
  inx
  stx $63fb
  inx
  stx $63fc
  inx
  stx $63fd
  inx
  stx $63fe
  inx
  stx $63ff

  lda #$ff
  sta $d017
;  lda #$ff
  sta $d01d

  lda #1
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  sta $d02b
  sta $d02c
  sta $d02d
  nop
  nop

  ldx #10
.e:
  dex
  bne .e

  bit $24
  ldy #0
  ldx #0
.ilop:

  dec $d016
  inc $d016
  sty $d017
ydistpoi = * + 1
  lda imagescroll_ydist,x
  sta $d017
  lda image_d011,x
  sta $d011
  lda #1
  sta $d02e,y
;  nop
  inx
  cpx #96
  bne .ilop

;  lda #1
;  sta $d020
;  sta $d021
;  ldx #9
;.e3:
; dex
; bne .e3
;  nop
;  nop
;  nop
;  lda #0
;  sta $d020
;  sta $d021

  nop
  lda #1
  sta IRQ_finished

  lda savea
  ldx savex
  ldy savey
  rti

irq_image2_raster = $f9
irq_image2:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #$13
  sta $d011
  lda #<irq_image0                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_image0
  sta $ffff
  lda #irq_image0_raster
  sta $d012
;  lda #%00000010
;  sta $dd00

  lda savea
  ldx savex
  ldy savey
  rti

;###
;###
;###

;spr_tableno: dc.b 0
;spr_table_poi: dc.b 0,0
;spr_tmp3: dc.b 0
; seg zp
;spr_tmp4: dc.b 0,0
; seg code2

spr_calc_rubberboing: subroutine
  lda #0
  sta spr_tableno
  lda #<spr_ytables
  sta spr_table_poi
  lda #>spr_ytables
  sta spr_table_poi+1

.looop:
  lda spr_tableno
  tax
  lda #0
  sta spr_tmp4
  sta spr_tmp4+1

  lda #0

.nextbyte:
  ldy #8
.newtbl_loop:
  lda sintbl,x
  bpl .ok
  lda #0
.ok:
  clc
  ror
  clc
  adc #$27
;  clc
  adc spr_tmp4
  sta spr_tmp4
  lda spr_tmp4+1
  adc #0
  cmp #$03
  bne .j
  lda #0
  asl spr_tmp3
  jmp .c
.j:
  sec
  rol spr_tmp3
.c:
  sta spr_tmp4+1

  inx
  dey
  bne .newtbl_loop

  lda spr_tmp3
  sta (spr_table_poi),y
  inc spr_table_poi
  bne .n1
  inc spr_table_poi+1
.n1:

  lda spr_table_poi
  and #$1f
  bne .nextbyte

;;Increase pointers to the next table, and loop
;  lda spr_table_poi
;  clc
;  adc #$20
;  sta spr_table_poi
;  lda spr_table_poi+1
;  adc #0
;  sta spr_table_poi+1

  inc spr_tableno
  beq .endall
  jmp .looop
.endall:
  rts

;########################################

textdisplay_init:
  lda #$0b
  sta $d011     ; set the raster interrupt location
  lda #<irq_text2
  sta $fffe
  lda #>irq_text2
  sta $ffff
  lda #<nmi_spr                       ;Set NMI, both the hardware
  sta $fffa                       ;KERNAL
  lda #>nmi_spr
  sta $fffb
  lda #irq_text2_raster
  sta $d012
  lda #0
  sta part_finished
  lda #$00
  sta $d01b

  lda #>textdisplay_7fff
  sta .7fpoi+2
  lda #<textdisplay_7fff
  sta .7fpoi+1
  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d
  ldx #1
  stx $d01a     ; enable raster interrupt
  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts
  cli
  rts

irq_text0_raster = $08
irq_text0:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%00011011
  sta $d011
  lda #<irq_textdisplay_border    ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_textdisplay_border
  sta $ffff
;  lda #irq_textdisplay_border_raster
;  sta $d012
  lda #%00000011
  sta $dd00

  lda #$15
  sta $d018

  ldx #0
  stx textdisplay_rownr
  ldx #0
  jsr textdisplay_set

.7fpoi:
  lda textdisplay_7fff
  sta $7fff
  beq .nomore7f
  inc .7fpoi+1
  bne .now7
  inc .7fpoi+2
.now7:
.nomore7f:

  lda savea
  ldx savex
  ldy savey
  rti


irq_text2_raster = $f9
irq_text2:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%00010011
  sta $d011
  lda #<irq_text0                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_text0
  sta $ffff
  lda #irq_text0_raster
  sta $d012
  lda #%00000011
  sta $dd00
;  lda #4
;  sta $d020
  jsr music_play

  lda savea
  ldx savex
  ldy savey
  rti


;This is used when we want to play music
;while initializing or decruncing and displaying
;nothing on the screen.
justplay_init:
  sei
  lda $d011
  and #$7f
  sta $d011
  lda #<irq_music                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_music
  sta $ffff
  lda #irq_music_raster
  sta $d012
  lda #<nmi_spr                       ;Set NMI, both the hardware
  sta $fffa                       ;KERNAL
  lda #>nmi_spr
  sta $fffb
  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d
  ldx #1
  stx $d01a     ; enable raster interrupt
  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts
  cli
  rts


irq_music_raster = $e0
irq_music:
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  jsr music_play
  lda savea
  ldx savex
  ldy savey
  rti


;###
;###
;###


 subroutine
  seg jumptable
  org $1800

;This is the jumptable for the 48 different
;subroutines in the line drawing. Each subroutine makes
;d020 to be written on a different x position
jumptable_lsb:
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowfe
  dc.b <rowff
  dc.b <row00
  dc.b <row01
  dc.b <row02
  dc.b <row03
  dc.b <row04
  dc.b <row05
  dc.b <row06
  dc.b <row07
  dc.b <row08
  dc.b <row09
  dc.b <row10
  dc.b <row11
  dc.b <row12
  dc.b <row13
  dc.b <row14
  dc.b <row15
  dc.b <row16
  dc.b <row17
  dc.b <row18
  dc.b <row19
  dc.b <row20
  dc.b <row21
  dc.b <row22
  dc.b <row23
  dc.b <row24
  dc.b <row25
  dc.b <row26
  dc.b <row27
  dc.b <row28
  dc.b <row29
  dc.b <row30
  dc.b <row31
  dc.b <row32
  dc.b <row33
  dc.b <row34
  dc.b <row35
  dc.b <row36
  dc.b <row37
  dc.b <row38
  dc.b <row39
  dc.b <row40
  dc.b <row41
  dc.b <row42
  dc.b <row43
  dc.b <row44
  dc.b <row45
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46
  dc.b <row46

jumptable_msb:
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowfe
  dc.b >rowff
  dc.b >row00
  dc.b >row01
  dc.b >row02
  dc.b >row03
  dc.b >row04
  dc.b >row05
  dc.b >row06
  dc.b >row07
  dc.b >row08
  dc.b >row09
  dc.b >row10
  dc.b >row11
  dc.b >row12
  dc.b >row13
  dc.b >row14
  dc.b >row15
  dc.b >row16
  dc.b >row17
  dc.b >row18
  dc.b >row19
  dc.b >row20
  dc.b >row21
  dc.b >row22
  dc.b >row23
  dc.b >row24
  dc.b >row25
  dc.b >row26
  dc.b >row27
  dc.b >row28
  dc.b >row29
  dc.b >row30
  dc.b >row31
  dc.b >row32
  dc.b >row33
  dc.b >row34
  dc.b >row35
  dc.b >row36
  dc.b >row37
  dc.b >row38
  dc.b >row39
  dc.b >row40
  dc.b >row41
  dc.b >row42
  dc.b >row43
  dc.b >row44
  dc.b >row45
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46
  dc.b >row46

  seg sinus
  org $1a00
;A sinus table, 256 bytes long, from -126 to 126
sintbl:         dc.b 0,3,6,9,12,15,18,21,24,27,30,33,36,39,42,45
                dc.b 48,51,54,57,59,62,65,67,70,73,75,78,80,82,85,87
                dc.b 89,91,94,96,98,100,102,103,105,107,108,110,112,113,114,116
                dc.b 117,118,119,120,121,122,123,123,124,125,125,126,126,126,126,126
costbl:         dc.b 126,126,126,126,126,126,125,125,124,123,123,122,121,120,119,118
                dc.b 117,116,114,113,112,110,108,107,105,103,102,100,98,96,94,91
                dc.b 89,87,85,82,80,78,75,73,70,67,65,62,59,57,54,51
                dc.b 48,45,42,39,36,33,30,27,24,21,18,15,12,9,6,3
                dc.b 0,-3,-6,-9,-12,-15,-18,-21,-24,-27,-30,-33,-36,-39,-42,-45
                dc.b -48,-51,-54,-57,-59,-62,-65,-67,-70,-73,-75,-78,-80,-82,-85,-87
                dc.b -89,-91,-94,-96,-98,-100,-102,-103,-105,-107,-108,-110,-112,-113,-114,-116
                dc.b -117,-118,-119,-120,-121,-122,-123,-123,-124,-125,-125,-126,-126,-126,-126,-126
                dc.b -126,-126,-126,-126,-126,-126,-125,-125,-124,-123,-123,-122,-121,-120,-119,-118
                dc.b -117,-116,-114,-113,-112,-110,-108,-107,-105,-103,-102,-100,-98,-96,-94,-91
                dc.b -89,-87,-85,-82,-80,-78,-75,-73,-70,-67,-65,-62,-59,-57,-54,-51
                dc.b -48,-45,-42,-39,-36,-33,-30,-27,-24,-21,-18,-15,-12,-9,-6,-3
                dc.b 0,3,6,9,12,15,18,21,24,27,30,33,36,39,42,45
                dc.b 48,51,54,57,59,62,65,67,70,73,75,78,80,82,85,87
                dc.b 89,91,94,96,98,100,102,103,105,107,108,110,112,113,114,116
                dc.b 117,118,119,120,121,122,123,123,124,125,125,126,126,126,126,126


MAXSPR          = 25            ;Maximum number of sprites

 seg zp
numsprites: dc.b 0    ;Number of sprites that the main program wants
                      ;to pass to the sprite sorter
sprupdateflag: dc.b 0 ;Main program must write a nonzero value here
                      ;when it wants new sprites to be displayed
sortedsprites: dc.b 0 ;Number of sorted sprites for the raster
                      ;interrupt
tempvariable: dc.b 0  ;Just a temp variable used by the raster
                      ;interrupt
sprirqcounter: dc.b 0 ;Sprite counter used by the interrupt

sortorder:     ds.b MAXSPR+2 ;Order-table for sorting. Needs as many bytes
sortorderlast: ds.b MAXSPR+2 ;as there are sprites.

part_inverted: dc.b 0

savea: dc.b 0
savex: dc.b 0
savey: dc.b 0

nmi_savea: dc.b 0
nmi_savex: dc.b 0
nmi_savey: dc.b 0


whitex: ;dc.b 0
line_x_lsb: dc.b 0
whitex_poi: ;dc.b 0,0
line_x_msb: dc.b 0
line_x_spd_lsb: dc.b 0
whitex_colcou: ;dc.b 0
line_x_spd_msb: dc.b 0

fld_tmp: ;dc.b 0
line_xadd_lsb: dc.b 0
line_xadd_msb: dc.b 0
line_xadd_spd_lsb: dc.b 0
line_xadd_spd_msb: dc.b 0

line_end: dc.b 0

line_rcol: dc.b 0
line_lcol: dc.b 0

line_cou: dc.b 0
line_cou2: dc.b 0

;Bit 2-0 determines the text in the sprites
;Bit 5-3 determines the bounce
spr_version_number: dc.b 0

spr_x_lsb: dc.b 0
spr_x_msb: dc.b 0
;spr_xadd_lsb: dc.b 0
;spr_xadd_msb: dc.b 0
spr_cou: dc.b 0
spr_cou2: dc.b 0
spr_tmp: dc.b 0

spr_lsb: dc.b 0
spr_ytables_cou: dc.b 0
spr_ytables_mempoi_msb: dc.b 0
spr_sprcol: dc.b 0

;Used for timing, making a lda #1 takes an extra clock cycle
constant_one: dc.b 0
constant_0x1f: dc.b 0

part_finished: dc.b 0
IRQ_finished: dc.b 0
demo_loop_cou: dc.b 0
countdown_to_finished: dc.b 0

;###
;### start
;###

        ;Main program
 seg code
 org $1c00
raster = 12     ; start of raster interrupt

start:
  jmp real_start

;###
;### start spr
;###

spr_raster = 13     ; start of raster interrupt

;This is the large sprite-stretch routine
start_spr:
  lda #$7f       ; turn off the CIA interrupts
  sta $dc0d
  sta $dd0d ;Clear NMI
  sei
  cld
  ldx #$ff
  txs
  lda #%00000011
  sta $dd00
  ldx #0
part_bgcol = * + 1
  lda #14
  sta $d020
  sta $d021
;  lda part_bgcol
  bne .nowhite
  ldx #1
.nowhite:
  stx spr_sprcol
  lda #$35
  sta $01

  ldx #0
.clrsc:
  lda part_bgcol
  sta $d800,x
  sta $d900,x
  sta $da00,x
  sta $daf7,x
  inx
  bne .clrsc

  ldx spr_sprcol
  stx spr_spr0col
  stx spr_spr1col
  stx spr_spr2col
  stx spr_spr3col
  stx spr_spr4col
  stx spr_spr5col
  stx spr_spr6col
  stx spr_spr7col
  lda spr_version_number
  and #$07
  cmp #$04
  bne .no_hearts
  ldx #2
  stx spr_spr0col
  stx spr_spr7col
  jsr justplay_init
  jmp .gon
.no_hearts:
  cmp #$02
  bne .nobitlive
  ldx #1
  stx spr_spr0col
  stx spr_spr1col
  stx spr_spr2col
  stx spr_spr7col
  jsr justplay_init
  jmp .gon
.nobitlive:
  cmp #$05
  bne .no_remix64
  ldx #2
  stx spr_spr0col
  stx spr_spr1col
  stx spr_spr2col
  stx spr_spr3col
  stx spr_spr4col
  ldx #10
  stx spr_spr5col
  stx spr_spr6col
  jsr justplay_init
  jmp .gon
.no_remix64:
  cmp #$00
  bne .no_mahoney
  ldx #1
  stx spr_spr0col
  ldx #7
  stx spr_spr1col
  ldx #3
  stx spr_spr2col
  ldx #14
  stx spr_spr3col
  ldx #11
  stx spr_spr4col
  ldx #13
  stx spr_spr5col
  ldx #2
  stx spr_spr6col
  ldx #15
  stx spr_spr7col
  jsr textdisplay_init

  lda #$60  ;opcode for rts -> decrunch is fastest
  sta decrunch_slow

  ldx #>(pex_young_pu+2)
  ldy #<(pex_young_pu+2)
  jsr pu
  jsr pexyoung_init

.no_mahoney:

.gon:
  lda spr_version_number
  and #$07
  asl
  asl
  asl
  tax
  jsr spr_drawsprites_init

;Empty the y_tables:
  ldx #0
  lda #$ff
.ll:
  sta spr_ytables+$0000,x
  sta spr_ytables+$0100,x
  sta spr_ytables+$0200,x
  sta spr_ytables+$0300,x
  sta spr_ytables+$0400,x
  sta spr_ytables+$0500,x
  sta spr_ytables+$0600,x
  sta spr_ytables+$0700,x
  sta spr_ytables+$0800,x
  sta spr_ytables+$0900,x
  sta spr_ytables+$0a00,x
  sta spr_ytables+$0b00,x
  sta spr_ytables+$0c00,x
  sta spr_ytables+$0d00,x
  sta spr_ytables+$0e00,x
  sta spr_ytables+$0f00,x
  sta spr_ytables+$1000,x
  sta spr_ytables+$1100,x
  sta spr_ytables+$1200,x
  sta spr_ytables+$1300,x
  sta spr_ytables+$1400,x
  sta spr_ytables+$1500,x
  sta spr_ytables+$1600,x
  sta spr_ytables+$1700,x
  sta spr_ytables+$1800,x
  sta spr_ytables+$1900,x
  sta spr_ytables+$1a00,x
  sta spr_ytables+$1b00,x
  sta spr_ytables+$1c00,x
  sta spr_ytables+$1d00,x
  sta spr_ytables+$1e00,x
  sta spr_ytables+$1f00,x
  inx
  bne .ll

  lda #nof_ytables
  sta spr_ytables_cou
  lda #<spr_ytables ;LSB
  sta spr_newbittable_poi+1 ;#6
  lda #>spr_ytables ;MSB
  sta spr_newbittable_poi+2 ;#6

  lda #>ytable
  sta spr_ytables_mempoi_msb
  lda #1
  sta one_left

  jsr spr_precalc_spr_ytables
  jsr spr_move

  ldx #0
  lda #$ff
.ll2:
  sta ytable+$0000,x
  sta ytable+$0100,x
  sta ytable+$0200,x
  sta ytable+$0300,x
  sta ytable+$0400,x
  sta ytable+$0500,x
  sta ytable+$0600,x
  sta ytable+$0700,x
  sta ytable+$0800,x
  sta ytable+$0900,x
  sta ytable+$0a00,x
  sta ytable+$0b00,x
  sta ytable+$0c00,x
  sta ytable+$0d00,x
  sta ytable+$0e00,x
  sta ytable+$0f00,x
  inx
  bne .ll2

;Empty the sprites that the letters are drawn in:
  lda #0
  ldx #0
.clrspr:
  sta spr_to_draw_in,x
  sta spr_to_draw_in+$ff,x
  inx
  bne .clrspr

  lda #$f0
.notyeet:
  cmp $d012
  bne .notyeet

  sei
  ldy #$00
  sty $d015     ; turn off all sprites
  lda #%11110010
  sta $d018     ;Screen at $4000 - bitmap at $6000
  lda #%11001000
  sta $d016
  lda #%00000011
  sta $dd00

  lda part_bgcol
  sta $d020

  ldx #$f8  ;Sprite at $3e00
  stx $07f8 ;sprite 0f
  inx       ;Sprite at $3e40
  stx $07f9 ;sprite 1f
  inx
  stx $07fa ;sprite 2f
  inx
  stx $07fb ;sprite 3f
  inx
  stx $07fc ;sprite 4f
  inx
  stx $07fd ;sprite 5f
  inx
  stx $07fe ;sprite 6f
  inx
  stx $07ff ;sprite 7f

  lda #$0b
  sta $d011     ; set the raster interrupt location

  lda #<irq_spr1
  sta $fffe
  lda #>irq_spr1
  sta $ffff
  lda #<nmi_spr                       ;Set NMI, both the hardware
  sta $fffa                       ;KERNAL
  lda #>nmi_spr
  sta $fffb
  lda #spr_raster
  sta $d012
  lda #0
  sta part_finished
  lda #$80
  sta countdown_to_finished
  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d
  ldx #1
  stx $d01a     ; enable raster interrupt

  lda #$10
  sta $d001 ;sprite 0y
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f

;  lda #$f3
;  sta $d000
;  clc
;  adc #$37
;  ldx #02
;.xlop:
;  sta $d000,x
;  clc
;  adc #$2f
;  inx
;  inx
;  cpx #$10
;  bne .xlop

  ldy #$ff
  sty $d015     ; turn on all sprites
  lda #charset_start
  sta $03f8 ;sprite 0f -cleared
  sta $03f9 ;sprite 0f -cleared
  sta $03fa ;sprite 0f -cleared
  sta $03fb ;sprite 0f -cleared
  sta $03fc ;sprite 0f -cleared
  sta $03fd ;sprite 0f -cleared
  sta $03fe ;sprite 0f -cleared
  sta $03ff ;sprite 0f -cleared

  lda #$00
  sta $d017 ;enlarge y
  lda #$ff
  sta $d01d ;enlarge x
  lda #0
  sta spr_cou

  jsr music_play

  lda #0
  sta IRQ_finished
  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts
  cli
forever_spr:
  lda IRQ_finished
  beq forever_spr
  lda #0
  sta IRQ_finished

  jsr spr_move
  jsr spr_drawsprites
  jsr music_play

  lda part_finished
  beq forever_spr
;Now, switch to another part
  jmp next_demo_part

spr_text:
  dc.b "M" - $40 + charset_start
  dc.b "A" - $40 + charset_start
  dc.b "H" - $40 + charset_start
  dc.b "O" - $40 + charset_start
  dc.b "N" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "Y" - $40 + charset_start
  dc.b "Z" + 2 - $40 + charset_start

  dc.b "B" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b charset_start
  dc.b "N" - $40 + charset_start
  dc.b "I" - $40 + charset_start
  dc.b "C" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "Z" + 2 - $40 + charset_start

  dc.b "B" - $40 + charset_start
  dc.b "I" - $40 + charset_start
  dc.b "T" - $40 + charset_start
  dc.b "L" - $40 + charset_start
  dc.b "I" - $40 + charset_start
  dc.b "V" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "Z" + 4 - $40 + charset_start

  dc.b "Z" + 5 - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "B" - $40 + charset_start
  dc.b "A" - $40 + charset_start
  dc.b "C" - $40 + charset_start
  dc.b "K" - $40 + charset_start
  dc.b "Z" + 3 - $40 + charset_start
  dc.b "Z" + 4 - $40 + charset_start

  dc.b "Z" + 7 - $40 + charset_start
  dc.b "M" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "L" - $40 + charset_start
  dc.b "I" - $40 + charset_start
  dc.b "N" - $40 + charset_start
  dc.b "A" - $40 + charset_start
  dc.b "Z" + 7 - $40 + charset_start

  dc.b "R" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "M" - $40 + charset_start
  dc.b "I" - $40 + charset_start
  dc.b "X" - $40 + charset_start
  dc.b "Z" + 3 - $40 + charset_start
  dc.b "Z" + 4 - $40 + charset_start
  dc.b "Z" + 5 - $40 + charset_start

  dc.b "C" -$40 + charset_start
  dc.b "Z" + 3 - $40 + charset_start
  dc.b "Z" + 4 - $40 + charset_start
  dc.b "P" - $40 + charset_start
  dc.b "O" - $40 + charset_start
  dc.b "W" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "R" - $40 + charset_start

  dc.b "Z" + 5 - $40 + charset_start
  dc.b "M" - $40 + charset_start
  dc.b "O" - $40 + charset_start
  dc.b "D" - $40 + charset_start
  dc.b "O" - $40 + charset_start
  dc.b "R" - $40 + charset_start
  dc.b "E" - $40 + charset_start
  dc.b "Z" + 6 - $40 + charset_start

spr_precalc_spr_ytables:
;Bit 5-3 determines the bounce
  lda spr_version_number
  clc
  ror
  clc
  ror
  clc
  ror
  and #$03

  cmp #0
  bne .no0
  jsr spr_calc_bouce_contant_height
  jmp .spr_clean
.no0:
  cmp #1
  bne .no1
  jsr spr_calc_expy
  jmp .spr_clean
.no1:
  cmp #2
  bne .no2
  jsr spr_calc_rubberboing
  jmp .spr_clean
.no2:


.spr_clean:
  lda #<spr_ytables
  sta spr_table_poi
  lda spr_newbittable_poi+1
  lda #>spr_ytables
  sta spr_table_poi+1
  lda spr_newbittable_poi+2
  lda #nof_ytables
  sta spr_ytables_cou
  rts

 seg zp
spr_tableno: dc.b 0
spr_table_poi: dc.b 0,0
spr_tmp3: dc.b 0
 seg code

;########################################

spr_calc_bouce_contant_height:
  lda #0
  sta spr_tableno
  lda #<spr_ytables
  sta spr_table_poi
  lda #>spr_ytables
  sta spr_table_poi+1

.looop:
  lda spr_tableno
  clc
  adc #$18
  tax
  lda sintbl,x
  clc
  adc #$80
  clc
  ror
  clc
  ror
  sec
  sbc #2
  sta spr_tmp3

  ldx spr_tableno
  lda sintbl,x
  bmi .noxor
  eor #$ff
.noxor:  
;  ror
  clc
  adc #$80
  clc
  adc spr_tmp3
  tax
  inx

;  lda spr_cou
;  ror
;  and #$7f
;  tax
;  inx

  ldy #0
.newtbl_loop:
  txa
  sec
  sbc #$8
  bcc .draw_from_here
;still no sprite:
  tax
  lda #$ff
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny
  jmp .newtbl_loop

.draw_from_here:
  clc
  adc #$8
  tax
  lda ypos_to_bits_table,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_mid,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

  lda ypos_to_bits_table_end2,x
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny

.blank_the_rest:
  cpy #$20
  beq .endthis   
  lda #$ff
;  sta spr_ytables,y
  sta (spr_table_poi),y
  iny
  jmp .blank_the_rest

.endthis:
;Increase pointers to the next table, and loop
  lda spr_table_poi
  clc
  adc #$20
  sta spr_table_poi
  lda spr_table_poi+1
  adc #0
  sta spr_table_poi+1

  inc spr_tableno
  beq .endall
  jmp .looop
.endall:
  rts

;########################################

 seg zp

;Do this as self-modifying code instead... faster
;spr_expy_speed: dc.b 0
spr_table_tmp: dc.b 0
spr_tmp4: dc.b 0
spr_tmp4_old: dc.b 0
spr_first: dc.b 0
spr_yskip: dc.b 0
 seg code

spr_calc_expy:
  lda #0
  sta spr_tableno
  lda #<spr_ytables
  sta spr_table_poi
  lda #>spr_ytables
  sta spr_table_poi+1

.loop_expy:
  lda spr_tableno
  eor #$ff
;  clc
;  ror
  sta spr_expy_speed

  lda #0
  sta spr_yskip
  lda spr_tableno
  cmp #$c0
  bcs .noysk
  tax
  lda sintbl,x
  clc
  adc #$80
  clc
  ror
;  clc
;  ror
;  clc
;  ror
  sta spr_yskip
.noysk:

  lda #0
  sta spr_tmp3
  sta spr_tmp4
  sta spr_tmp4_old

;;First line - ignore:
;  lda #$ff
;  sta (spr_table_poi),y
;  inc spr_table_poi

  ldy spr_yskip
  ldx #0
.skip_pos:
  tya
  sec
  sbc #$8
  bcc .draw_from_here2
;still no sprite:
  tay
  lda #$ff
  sta (spr_table_poi,x)
  inc spr_table_poi
  jmp .skip_pos

.draw_from_here2:
  clc
  adc #$8
  and #$7
  eor #$7
  tay
  iny
  ldx #0
  lda #$ff
;  sta (spr_table_poi,x)
  sta spr_table_tmp
  jmp .thisbyte



.loop_bytes:
  ldy #8

.thisbyte:
  lda spr_tmp3
  clc
spr_expy_speed = *+1
  adc #0 ;Self modifying - spr_expy_speed
;  adc spr_expy_speed
  sta spr_tmp3

  lda spr_tmp4
  adc #0
  sta spr_tmp4
  cmp spr_tmp4_old
  beq .one
;  bvs .one

.zero:
  sta spr_tmp4_old
;  lda spr_tmp4
  cmp #21
  beq .nomore
  asl spr_table_tmp
  dey
  bne .thisbyte
  jmp .contt

.one:
  sec
  rol spr_table_tmp

.cont:
  dey
  bne .thisbyte

.contt:
  lda spr_table_tmp
  sta (spr_table_poi,x)
  inc spr_table_poi
;Done at .next_table below
;  bne .noms
;  inc spr_table_poi+1
;.noms:

  lda spr_table_poi
  and #$1f
  bne .loop_bytes
  jmp .next_table



.nomore:
;  lda #0
;  sta spr_expy_speed

;Finish the byte that we were working on...
.thisbyte_end:
  sec
  rol spr_table_tmp
  dey
  bne .thisbyte_end

  lda spr_table_tmp
  sta (spr_table_poi,x)
  inc spr_table_poi
;Done at .next_table below
;  bne .noms_end
;  inc spr_table_poi+1
;.noms_end:

  lda spr_table_poi
  and #$1f
  beq .next_table

;...and now fill out the rest of the bytes with #$ff

.loop_bytes_end:
  lda #$ff
  sta (spr_table_poi,x)
  inc spr_table_poi
;Done at .next_table below
;  bne .noms_end2
;  inc spr_table_poi+1
;.noms_end2:

  lda spr_table_poi
  and #$1f
  bne .loop_bytes_end

.next_table:
;Check to see if we wrapped:
  lda spr_table_poi
  bne .noms_end3
  inc spr_table_poi+1
.noms_end3:


  inc spr_tableno
  beq .end_expy
  jmp .loop_expy
.end_expy:
  rts




;########################################

spr_move:
  dec spr_ytables_cou
  bne .nowrap
  lda one_left
  beq .empty
  lda spr_version_number
  bmi .no_end_this_way  ;This part is ended by erase, not by empty ytable
  lda #0
  sta one_left
.no_end_this_way
  lda #<spr_ytables ;LSB
  sta spr_newbittable_poi+1 ;#6
  lda #>spr_ytables ;MSB
  sta spr_newbittable_poi+2 ;#6
  lda #nof_ytables
  sta spr_ytables_cou
  jmp .nowrap
.empty:
  lda #<empty_ytable ;LSB
  sta spr_newbittable_poi+1 ;#6
  lda #>empty_ytable ;MSB
  sta spr_newbittable_poi+2 ;#6
  lda #1
  sta spr_ytables_cou
  dec countdown_to_finished
  bne .nowrap
  lda #1
  sta part_finished

.nowrap:
;Get the first ytable-byte into the temp-reg
  ldy #0
  lda spr_newbittable_poi+1
  sta spr_table_poi
  lda spr_newbittable_poi+2
  sta spr_table_poi+1
  lda (spr_table_poi),y
;  lda spr_ytables
  sta spr_tmp
  asl spr_tmp ;shift the first bit out, this isn't used

  inc spr_cou

;Modify the code to get the ytable we use on this frame from correct position
  lda spr_ytables_mempoi_msb
  sta yta_00+2
  sta yta_01+2
  sta yta_02+2
  sta yta_03+2
  sta yta_04+2
  sta yta_05+2
  sta yta_06+2
  sta yta_07+2
  sta yta_08+2
  sta yta_09+2
  sta yta_0a+2
  sta yta_0b+2
  sta yta_0c+2
  sta yta_0d+2
  sta yta_0e+2
  sta yta_0f+2

  clc
  adc #1
  cmp #>ytable_end
  bne .nowraa
  lda #>ytable
.nowraa:
  sta spr_ytables_mempoi_msb

;Now, move the sprites in the x-direction:
  lda #$ff
  sta spr_x_msb
  lda #$f0
  sta spr_x_lsb

  ldx spr_cou
  ldy #0

.xset_lop:
  lda costbl,x
  cmp #$80
  ror
  cmp #$80
  ror
  clc
  adc #$20

  clc
  adc spr_x_lsb
  sta $d000,y
  lda spr_x_msb
  adc #0
  cmp #$ff
  bne .nofix_for_left_border
  lda $d000,y
  sec
  sbc #8
  sta $d000,y
  lda #$ff
.nofix_for_left_border:
;  and #$01
  ror
  ror d010_minus_one+1

  txa
  clc
  adc #$18
  tax

  lda spr_x_lsb
  clc
  adc #$2a
  sta spr_x_lsb
  lda spr_x_msb
  adc #0
  sta spr_x_msb

  iny
  iny
  cpy #$10
  bne .xset_lop


;  lda #5
;  sta $d020
  rts

ypos_to_bits_table:
  dc.b $77,$bb,$dd,$ee,$f7,$fb,$fd,$fe
;  dc.b $fe,$fd,$fb,$f7,$ee,$dd,$bb,$77
ypos_to_bits_table_mid:
  dc.b $77,$bb,$dd,$ee,$77,$bb,$dd,$ee
;  dc.b $ee,$dd,$bb,$77,$ee,$dd,$bb,$77
;ypos_to_bits_table_end:
;  dc.b $7f,$bf,$df,$ef,$77,$bb,$dd,$ee
;;  dc.b $ee,$dd,$bb,$77,$ef,$df,$bf,$7f
ypos_to_bits_table_end2:
  dc.b $ff,$ff,$ff,$ff,$7f,$bf,$df,$ef
;  dc.b $ef,$df,$bf,$7f,$ff,$ff,$ff,$ff



nmi_spr:
  sta nmi_savea
;Should we save tsx here? Dunno...
  lda $dd0d ;ack
  lda #$7f                    ;CIA interrupt off
  sta $dd0d
  lda nmi_savea
  rti



irq_spr1:
  sta savea
  stx savex
  sty savey
  tsx           ; 2
  lda #$ff
  sta $d017
  lda #$18  ;spr data at $07f8
  sta $d018

  nop
  bit $24       ; 5

  lda #<irq_spr2
  sta $fffe
  lda #>irq_spr2
  sta $ffff

d010_minus_one:
  lda #$c1         ; waste at least 12 cycles
  sta $d010      ; (up to 64 cycles delay allowed here)
  lda #$ff
  sta $d015
  inc $d012     ; At this stage, $d012 has already been incremented by one.
  lda #1
  sta $d019     ; acknowledge the first raster interrupt
  cli           ; enable interrupts (the second interrupt can now occur)


;Total cycles = 2+5*9-1 = 46 cycles
;  ldy #9  ;#2
;  dey     ;#2
;  bne *-1 ;#3 if succeeds      ; delay

;cycles = 2+8*4 = 34
spr_spr0col = *+1
  lda #1  ;#2
  sta $d027
spr_spr1col = *+1
  lda #1
  sta $d028
spr_spr2col = *+1
  lda #1
  sta $d029
spr_spr3col = *+1
  lda #1
  sta $d02a
spr_spr4col = *+1
  lda #1
  sta $d02b
spr_spr5col = *+1
  lda #1
  sta $d02c
spr_spr6col = *+1
  lda #1
  sta $d02d


;spr_sprcol = *+1
;  lda #$01  ;#2
; sta $d028 ;spr #1 colour
; sta $d029 ;spr #2 colour
; sta $d02a ;spr #3 colour
; sta $d02b ;spr #4 colour
;heart3addr = *+1
; sta $d02c ;spr #5 colour
;heart4addr = *+1
; sta $d02d ;spr #6 colour
;hearts_col = *+1
;  lda #1
;heart1addr = *+1
; sta $d027 ;#4 ;spr #0 colour
;heart2addr = *+1
; sta $d02e ;spr #7 colour
;spr2sta:
;  lda $d028
;spr3sta:
;  lda $d029
;  nop


  nop           ; The second interrupt will occur while executing these
  nop           ; two-cycle instructions.
  nop
  nop
  nop
  nop

.error:
;  inc $d020
  jmp .error

irq_spr2:
;Now, get the bytes away from the stack that we won't use
 pla           ; 4
 txa           ; 2
 pla           ; 4
 tya           ; 2
 pla           ; 4
 tsx           ; 2
spr_spr7col = *+1
  lda #1
  sta $d02e
 nop
 bit $24       ; 5

  lda #<irq_spr1
  sta $fffe
  lda #>irq_spr1
  sta $ffff
  ldx $d012
  nop
  bit $24       ; 6569, 63 cycles/line
  cpx $d012     ; The comparison cycle is executed CYCLES or CYCLES+1 cycles
                ; after the interrupt has occurred.
  beq *+2       ; Delay by one cycle if $d012 hadn't changed.
                ; Now exactly CYCLES+3 cycles have passed since the interrupt.



  lda #spr_raster
  sta $d012     ; restore original raster interrupt position
  ldx #1
  stx $d019     ; acknowledge the raster interrupt
;  lda #10
; sta $d020
  nop
  nop
  nop
  lda #1
  sta IRQ_finished
  nop
  nop
  nop
  nop
  nop
  bit $24
  nop

  nop
  bit $24
  ldy #0
  ldx #$0 ;#2
;$d018 bit 7-4 sets video bank


.sprlo:
  dec $d016
  inc $d016
  sty $d017
yta_00:
  lda ytable,x
  sta $d017
  asl spr_tmp
  rol
yta_01:
  sta ytable,x
  nop
  lda #$1e
; lda #$16
  sta $d011
  inx

  dec $d016
  inc $d016
  sty $d017
yta_02:
  lda ytable,x
  sta $d017
  asl spr_tmp
  rol
yta_03:
  sta ytable,x
  nop
; lda #$1f
  lda #$17
  sta $d011
  inx

  dec $d016
  inc $d016
  sty $d017
yta_04:
  lda ytable,x
  sta $d017
  asl spr_tmp
  rol
yta_05:
  sta ytable,x
  txa
; lda #$18
  and #$08
  eor #$18
; lda #$10 ;Using this as #$10 disables the screen completely
  sta $d011

  dec $d016
  inc $d016
  sty $d017
yta_06:
  lda ytable+1,x
  sta $d017
  asl spr_tmp
  rol
yta_07:
  sta ytable+1,x
  inx
  lda #$19
; lda #$11
  sta $d011
  inx

  dec $d016
  inc $d016
  sty $d017
yta_08:
  lda ytable,x
  sta $d017
  asl spr_tmp
  rol
yta_09:
  sta ytable,x
  nop
  inc spr_newbittable_poi+1 ;#6
  inx

  dec $d016
  inc $d016
  sty $d017
yta_0a:
  lda ytable,x
  sta $d017
  asl spr_tmp
  rol
yta_0b:
  sta ytable,x
  nop
  lda #$1b
; lda #$13
  sta $d011
  inx

  dec $d016
  inc $d016
  sty $d017
yta_0c:
  lda ytable,x
  sta $d017
  asl spr_tmp
  rol
yta_0d:
  sta ytable,x
spr_newbittable_poi:
  lda spr_ytables ;#4
;  sta spr_tmp ;#3
  dc.b $8d,spr_tmp,$00 ;sta spr_tmp (absolute) #4

  inx

  dec $d016
  inc $d016
  sty $d017
yta_0e:
  lda ytable,x
  sta $d017,y
  asl spr_tmp
  rol
yta_0f:
  sta ytable,x
  inx
  nop

  beq .sprloend
  jmp .sprlo

.sprloend:


.spr_end_screen:

; lda #$18  ;spr data at $07f8
  lda #$08  ;spr data at $03f8
  dec $d016
  sta $d018
  inc $d016
  lda #0
  sta $d017
  lda #$ff
  sta $d017

  lda #spr_raster
  sta $d012     ; restore original raster interrupt position
  lda #$1b  ;normal 25-row
;  lda #$7b ;normal 25-row
  sta $d011
  ldx #1
  stx $d019     ; acknowledge the raster interrupt

  dec $d016
  inc $d016

;This is done since we don't have time to take
;care of incrementing msb of the ytable-pointer
;in the screen-synced loop:
  lda spr_newbittable_poi+1
  bne .noaddmsb
  inc spr_newbittable_poi+2
.noaddmsb:

;  nop
;  nop

;  lda #6
; sta $d020

  lda savea
  ldy savey
  ldx savex
  rti





;###
;### start line
;###

;This is the routine that makes the screen go from white to black
;or vice versa by drawing a line. This is done by self modifying
;code with extreme caution with timing.

  seg code
start_line: subroutine
  lda #$7f       ; turn off the CIA interrupts
  sta $dc0d
  sta $dd0d ;Clear NMI
  sei
  cld
  ldx #$ff
  txs
; lda #$00
; sta $d020
; sta $d021
  lda #$35
  sta $01
  jsr music_play
  lda #<irq1
  sta $fffe
  lda #>irq1
  sta $ffff
  lda #<nmi_line_end                       ;Set NMI, both the hardware
  sta $fffa                       ;KERNAL
  lda #>nmi_line_end
  sta $fffb
  lda #$0b
  sta $d011     ; set the raster interrupt location
  lda #raster
  sta $d012
  lda #0
  sta part_finished

  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d
  ldx #1
  stx $d01a     ; enable raster interrupt
  lda $dc0d     ; acknowledge CIA interrupts
  lda $dd0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts
  ldy #$00
  sty $d015     ; turn off all sprites

;  lda #$00
; sta line_x_spd_msb
;  lda #$40
; sta line_x_spd_lsb
;  lda #$00
; sta line_x_lsb
;  lda #$80
; sta line_x_msb
;  lda #0
;  sta line_xadd_lsb
; lda #$d0
; sta line_xadd_msb
;  lda #$28
;  sta line_xadd_spd_lsb
; lda #$00
; sta line_xadd_spd_msb
;  lda #0
; sta line_cou
;  lda #$20
; sta line_cou2



  lda #$40
  sta line_end ;Not used anymore
  lda #0        ;Not fully implemented
  sta line_rcol
  lda #1
  sta line_lcol
  jsr line_move

  lda #0
; sta $d020
; sta $d021
  sta IRQ_finished

  cli
forever:
  lda IRQ_finished
  beq forever
  lda #0
  sta IRQ_finished

  jsr line_move
  jsr music_play
  lda part_finished
  beq forever
  lda #1
  sta part_inverted
  jmp next_demo_part

music_init:
;  rts
;;  lda #$01
;;  sta $c00f
  lda #0
  jmp $e000

music_enable: dc.b 0

music_play:
  lda music_enable
  beq .rrts
  jmp $e012
.rrts:
  rts

line_move:
  inc line_cou
  ldx line_cou
  ldy #00
  lda sintbl,x
  cmp #$80
  ror
;  cmp #$80
; ror
  sta line_x_spd_lsb
  bpl .no_neg
  ldy #$ff
.no_neg:
  sty line_x_spd_msb

  inc line_cou2
  inc line_cou2
  ldx line_cou2
  ldy #00
  lda sintbl,x
  sta line_x_lsb
  bpl .no_neg2
  ldy #$ff
.no_neg2:
  sty line_x_msb

  clc
  rol line_x_lsb
  rol line_x_msb
  clc
  rol line_x_lsb
  rol line_x_msb
  clc
  rol line_x_lsb
  rol line_x_msb
  clc
  rol line_x_lsb
  rol line_x_msb
  clc
  rol line_x_lsb
  rol line_x_msb

  lda line_xadd_lsb
  clc
  adc line_xadd_spd_lsb
  sta line_xadd_lsb

  lda line_xadd_msb
  adc line_xadd_spd_msb
  sta line_xadd_msb

line_finito = *+1
  cmp #$0c
  bne .no_finito
.finito:
  lda #1
  sta part_finished

.no_finito:

  lda line_x_lsb
  clc
  adc line_xadd_lsb
  sta line_x_lsb

  lda line_x_msb
  adc #$80
  clc
  adc line_xadd_msb
  sta line_x_msb
  rts




;NMI breaks the IRQ-infinite loop, no need to save registers

nmi_line_end:
  lda #0
  sta $d020

; lda #3
; sta $d020
; sta $d021
  lda $dd0d ;ack
  lda #$7f                    ;CIA interrupt off
  sta $dd0d

  lda #1
  sta IRQ_finished

  pla ;Remove the NMI-returnaddr+status from the stack
  pla
  pla

  lda savea ;Instead, return as we would from the IRQ
  ldy savey
  ldx savex
  rti

; lda nmi_savea
; ldx nmi_savex
; ldy nmi_savey
;  pla
; rti




irq1:
; irq (event)   ; > 7 + at least 2 cycles of last instruction (9 to 16 total)
; pha           ; 3
; txa           ; 2
; pha           ; 3
; tya           ; 2
; pha           ; 3
; tsx           ; 2
; lda $0104,x   ; 4
; and #xx       ; 2
; beq           ; 3
; jmp ($314)    ; 5
                ; ---
                ; 38 to 45 cycles delay at this stage

 sta savea
; pha           ; 3
 txa           ; 2
 stx savex
; pha           ; 3
 tya           ; 2
 sty savey
; pha           ; 3
 tsx           ; 2
 nop
 nop           ; 4
 nop           ; 2
 bit $24       ; 3
 nop
 bit $24       ; 5

  lda #<irq2
  sta $fffe
  lda #>irq2
  sta $ffff
  nop           ; waste at least 12 cycles
  nop           ; (up to 64 cycles delay allowed here)
  nop
  nop
  nop
  nop
  inc $d012     ; At this stage, $d012 has already been incremented by one.
  lda #1
  sta $d019     ; acknowledge the first raster interrupt
  cli           ; enable interrupts (the second interrupt can now occur)
  ldy #9
  dey
  bne *-1       ; delay
  nop           ; The second interrupt will occur while executing these
  nop           ; two-cycle instructions.
  nop
  nop
  nop
  nop

.error:
  inc $d020
  jmp .error

;  lda savea
; ldy savey
; ldx savex
; rti
;  jmp $ea81     ; return to the auxiliary raster interrupt
;  pla
; tay
; pla
; tax
; pla
; rti


; Main raster interrupt
irq2:
; irq (event)   ; 7 + 2 or 3 cycles of last instruction (9 or 10 total)
; pha           ; 3
; txa           ; 2
; pha           ; 3
; tya           ; 2
; pha           ; 3
; tsx           ; 2
; lda $0104,x   ; 4
; and #xx       ; 2
; beq           ; 3
; jmp (cinv)    ; 5
                ; ---
                ; 38 or 39 cycles delay at this stage

;Now, get the bytes away from the stack that we won't use
; lda savea
 pla           ; 4
 txa           ; 2
; ldx savex
 pla           ; 4
 tya           ; 2
; ldy savey
 pla           ; 4
 tsx           ; 2
 nop
 nop           ; 4
 nop           ; 2
 nop
 bit $24       ; 5

  lda #<irq1
  sta $fffe
  lda #>irq1
  sta $ffff
  ldx $d012
  nop
  bit $24       ; 6569, 63 cycles/line
  cpx $d012     ; The comparison cycle is executed CYCLES or CYCLES+1 cycles
                ; after the interrupt has occurred.
  beq *+2       ; Delay by one cycle if $d012 hadn't changed.
                ; Now exactly CYCLES+3 cycles have passed since the interrupt.
  dex
  dex
  stx $d012     ; restore original raster interrupt position
  ldx #1
  stx $d019     ; acknowledge the raster interrupt
  nop
  nop

;  ldy #2
;  dey
;  bne *-1       ; delay
  lda #0
  sta $d020
  nop
;  lda #10
; bit $24
  bit $24

;ticks_to_live = 63 * 200
ticks_to_live = 63 * 272

;Setup NMI-timer for endline
;NMI_trigger:
  lda #<ticks_to_live
  sta $dd04
  lda #>ticks_to_live
  sta $dd05
  lda $dd0d ;ack NMI-timer irq
;  lda #%10011011 ;Start timer
  lda #%10011001 ;Start timer
  sta $dd0e
  lda #$81       ;enable NMI from timer a IRQ
  sta $dd0d

  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  bit $24

  lda #$ff
  sta line_end

  ldx line_x_msb
  lda jumptable_lsb,x
  sta j1+1
  lda jumptable_msb,x
  sta j1+2

  ldy line_x_lsb
  ldx line_x_msb
j1:
  jmp $0000


endirq:
  ldy #2
  dey
  bne *-1       ; delay
  lda #0
  sta $d020
;  jsr line_move

  lda savea
  ldy savey
  ldx savex
  rti

;  jmp $ea81     ; return to the auxiliary raster interrupt
;  pla
; tay
; pla
; tax
; pla
; rti

rowfe: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  lda #0
  sta $d020
  nop
  nop
  nop
  bit $24
  nop
  nop
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

rowff: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020
  dec $d020
  bit $24
  nop
  nop
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row00: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020
  lda line_rcol
  sta $d020
  nop
  nop
  nop
;  bit $24 ;#3
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row01: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020
  nop
  dec $d020
  bit $24
  nop
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row02: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020
  bit $24
  dec $d020
  nop
  nop
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row03: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020
  nop
  nop
  dec $d020
  bit $24
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row04: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020
  nop
  bit $24
  dec $d020
  nop
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row05: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;6 cycles pause
  dc.b $ce,line_end,$00     ;DEC absolute 3/6
  dec $d020   ;1+5
  lda line_end
  lda line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row06: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;7 cycles pause
  nop
  nop
  bit $24
  dec $d020   ;0+5
  dec line_end
  nop ;beq .end_now
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row07: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;8 cycles pause
  dc.b $ce,line_end,$00     ;DEC absolute 3/6
  nop ;beq .end_now ;#2 not taken
  lda #0
  sta $d020
  nop
  nop
  nop
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row08: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;8 cycles pause
  nop
  dec line_end              ;DEC Z-page #5
;dc.b $ce,line_end,$00      ;DEC absolute #6
  nop ;beq .end_now ;#2 not taken
  lda #0
  sta $d020
  bit $24
  nop
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row09: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;8 cycles pause
  nop
;  dec line_end             ;DEC Z-page #5
  dc.b $ce,line_end,$00     ;DEC absolute #6
  nop ;beq .end_now ;#2 not taken
  lda #0
  sta $d020
  nop
  nop
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row10: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;8 cycles pause
  nop
  nop
  dec line_end              ;DEC Z-page #5
;  dc.b $ce,line_end,$00      ;DEC absolute #6
  nop ;beq .end_now ;#2 not taken
  lda #0
  sta $d020
  bit $24
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row11: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;8 cycles pause
  nop
  nop
;  dec line_end             ;DEC Z-page #5
  dc.b $ce,line_end,$00     ;DEC absolute #6
  nop ;beq .end_now ;#2 not taken
  lda #0
  sta $d020
  nop
  tya
  clc
  adc line_x_spd_lsb
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row12: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;8 cycles pause
  nop
  nop
  nop
  dec line_end              ;DEC Z-page #5
;  dc.b $ce,line_end,$00      ;DEC absolute #6
  nop ;beq .end_now ;#2 not taken
  lda #0
  sta $d020
  tya
  clc
; adc line_x_spd_lsb          ;#3
  dc.b $6d,line_x_spd_lsb,$00 ;ADC absolute #4
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row13: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;8 cycles pause
  nop
  nop
  nop
;  dec line_end             ;DEC Z-page #5
  dc.b $ce,line_end,$00     ;DEC absolute #6
  nop ;beq .end_now ;#2 not taken
  lda #0
  sta $d020
  tya
  clc
  adc line_x_spd_lsb          ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;ADC absolute #4
  tay
  txa
  adc line_x_spd_msb
  tax
  lda jumptable_lsb,x
  sta .j+1
  lda jumptable_msb,x
  sta .j+2
  txa
.j:
  jmp $0000
.end_now:
  jmp endirq

row14: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  nop ;#2
  nop ;#2
  nop ;#2
  clc ;#2
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda #0 ;#2
  sta $d020 ;#4
  tya ;#2
; adc line_x_spd_lsb ;#3
  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row15: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  nop ;#2
;  nop  ;#2
  bit $24 ;#3
  clc ;#2
;  nop  ;#2
;  dec $d020 ;#5
  lda #0 ;#2
  sta $d020 ;#4
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row16: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
; adc line_x_spd_lsb ;#3
  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
;  nop  ;#2
  nop ;#2
  nop ;#2
  nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row17: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  bit $24 ;#3
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row18: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row19: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  bit $24 ;#3
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row20: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  nop ;#2
  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row21: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  nop ;#2
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
;  dec line_end ;#5
  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
; nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row22: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  nop ;#2
  nop ;#2
  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
; nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row23: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
;  dec line_end ;#5
  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  nop ;#2
  nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row24: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  bit $24 ;#3
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row25: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
;  dec line_end ;#5
  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row26: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  bit $24 ;#3
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row27: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
;  dec line_end ;#5
  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row28: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#6
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  bit $24 ;#3
  nop ;#2
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row29: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  bit $24 ;#3
  lda jumptable_lsb,x ;#6
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  nop ;#2
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row30: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#6
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  bit $24 ;#3
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row31: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#6
  nop ;#2
  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  sta .j+1 ;#4
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row32: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  bit $24 ;#3
  nop ;#2
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row33: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  nop ;#2
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row34: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  bit $24 ;#3
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row35: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#6
  sta .j+1 ;#4
  nop ;#2
  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  lda jumptable_msb,x ;#6
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row36: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  nop ;#2
;  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  bit $24 ;#3
  nop ;#2
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row37: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  bit $24 ;#3
;  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  nop ;#2
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row38: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  nop ;#2
  nop ;#2
;  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  bit $24 ;#3
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row39: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  nop ;#2
; nop ;#2
  bit $24 ;#3
;  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  sta .j+2 ;#4
  txa ;#2
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row40: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  sta .j+2 ;#4
  txa ;#2
;  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  nop ;#2
  bit $24 ;#3
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row41: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
;  dec line_end ;#5
  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  sta .j+2 ;#4
  txa ;#2
;  bit $24 ;#3
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  nop ;#2
  nop ;#2
;  bit $24 ;#3
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row42: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  sta .j+2 ;#4
  txa ;#2
;  bit $24 ;#3
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
  nop ;#2
  bit $24 ;#3
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row43: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  sta .j+2 ;#4
  txa ;#2
  bit $24 ;#3
; nop ;#2
;  dec $d020 ;#5
  lda #0 ;#2
  sta $d020 ;#4
  nop ;#2
  nop ;#2
;  bit $24 ;#3
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row44: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  sta .j+2 ;#4
  txa ;#2
;  bit $24 ;#3
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
; nop ;#2
; nop ;#2
  bit $24 ;#3
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row45: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  sta .j+2 ;#4
  txa ;#2
  bit $24 ;#3
; nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
; nop ;#2
  nop ;#2
;  bit $24 ;#3
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row46: subroutine
;Can use A for fun, Y=line_x_lsb, X=line_x_msb
  inc $d020 ;#5
  clc ;#2
  tya ;#2
  adc line_x_spd_lsb ;#3
;  dc.b $6d,line_x_spd_lsb,$00 ;#4
  tay ;#2
  txa ;#2
  adc line_x_spd_msb ;#3
  tax ;#3
  dec line_end ;#5
;  dc.b $ce,line_end,$00 ;#6
  nop ;beq .end_now ;#2 not taken
  lda jumptable_lsb,x ;#4
  sta .j+1 ;#4
  lda jumptable_msb,x ;#4
  sta .j+2 ;#4
  txa ;#2
  bit $24 ;#3
; nop ;#2
  nop ;#2
  nop ;#2
  dec $d020 ;#5
;  lda #0 ;#2
; sta $d020 ;#4
; nop ;#2
;  bit $24 ;#3
.j:
  jmp $0000 ;#3
.end_now:
  jmp endirq

row47:
  jmp endirq





;###
;### start scroll
;###

;A sprite-multiplexed scroller over a normal text display.

start_scroll: SUBROUTINE
  lda #$7f       ; turn off the CIA interrupts
  sta $dc0d
  sta $dd0d ;Clear NMI
  sei
  cld
  ldx #$ff
  txs
  lda #14
  sta $d020
  sta $d021
; ldx #$0
; lda #' 
;.clrscr:
; sta $0400,x
; sta $0500,x
; sta $0600,x
; sta $0700,x
; inx
; bne .clrscr    

  lda #0
  sta part_finished
  sta $d017
  jsr initsprites             ;Init the multiplexing-system

  lda #$ff ;spr X-enlarge
  sta $d01d
  jsr IRQ_init

  jsr setpos_init

  jsr clr_init
  cli


;###
;### mainloop
;###

mainloop:
  inc sprupdateflag               ;Signal to IRQ: sort the
                                 ;sprites
waitloop:
  lda sprupdateflag               ;Wait until the flag turns back
  bne waitloop                    ;to zero


;                ldx #MAXSPR-1
;moveloop:       lda $e040,x                     ;Move the sprites with some
;                and #$03                        ;random speeds
;                sec
;                adc sprx,x
;       sec
;       sbc #$02
;                sta sprx,x
;                lda $e050,x
;                and #$01
;                sec
;                adc spry,x
;                sta spry,x
;                dex
;                bpl moveloop
;  dec $d021
  jsr setpos
;  inc $d021
;  dec $d021

  jsr music_play
  jsr screen_clear
;  inc $d021

  lda part_finished
  beq mainloop
  jmp next_demo_part

IRQ_idle:       dec $d019
                jmp $ea81

;###
;### screen_clear
;###

 seg zp
clear_wait: dc.b 0
clear_x: dc.b 0
clear_y: dc.b 0
clear_spdx: dc.b 0
clear_spdy: dc.b 0
clear_poi: dc.b 0,0
clear_poi2: dc.b 0,0
clear_tmp: dc.b 0
 seg code

clr_init:
  lda #$ff
  sta clear_wait
  lda #1
  sta clear_spdy
  lda #0
  sta clear_spdx
  sta clear_y
  lda #39
  sta clear_x

  lda #$d8
  sta clear_poi+1
  lda #0
  sta clear_poi

  lda #2
  sta screen_spd
.rrrts:
  rts

screen_clear:
  dec clear_wait
  bne .rrrts

screen_spd = * + 1
  lda #2
  sta clear_wait

  lda #14
  ldy clear_x
  bmi .rrrts  ;don't draw outside the screen
  sta (clear_poi),y

  lda clear_poi
  sta clear_poi2
  lda clear_poi+1
  and #$03
  clc
  adc #$04
  sta clear_poi2+1
  lda #' +$80
  sta (clear_poi2),y

  lda clear_spdy
  bpl .add

  dec clear_y
  lda clear_y
  cmp #$ff
  bne .noedge
  dec clear_x
  lda #0
  sta clear_y
  lda #1
  sta clear_spdy
  rts
.noedge:
  lda clear_poi
  sec
  sbc #40
  sta clear_poi
  lda clear_poi+1
  sbc #0
  sta clear_poi+1
  rts

.add:
  inc clear_y
  lda clear_y
  cmp #25
  bne .noedgeB
  dec clear_x
  lda #24
  sta clear_y
  lda #$ff
  sta clear_spdy
  lda clear_x
  cmp #8
  bne .rrrts
  lda #1
  sta screen_spd
  rts
.noedgeB:
  lda clear_poi
  clc
  adc #40
  sta clear_poi
  lda clear_poi+1
  adc #0
  sta clear_poi+1
  rts


;###
;### setpos
;###

 seg zp
pos_wait: dc.b 0
pos_rate: dc.b 0
pos_spr_to_spawn: dc.b 0

pos_spawn_x: dc.b 0
pos_spawn_x2: dc.b 0
pos_spawn_x_spd: dc.b 0
pos_spawn_x2_spd: dc.b 0

pos_spawn_ind_pos_x_spd: dc.b 0
pos_spawn_ind_pos_x2_spd: dc.b 0

pos_textcou: dc.b 0
 seg code

pos_en:     ds.b MAXSPR,0  ;This is TTL, time to live
pos_x:      ds.b MAXSPR,0
pos_x_spd:  ds.b MAXSPR,0
pos_x2:     ds.b MAXSPR,0
pos_x2_spd: ds.b MAXSPR,0
pos_y:      ds.b MAXSPR,0

pos_text:
  dc.b " ",0,99,-201,-5,1,1,0,"STAND BY WHILE LOADING AND DECRUNCHING              "
  dc.b 0,-40,50,6,1,0,1,"   THIS IS A VERY SPECIAL NIGHT         "
  dc.b 0,10,120,-3,-5,2,-2,"     WE ARE HERE TO ENJOY       "
  dc.b 0,0,50,6,2,1,-2,"   ROB HUBBARD    "
  dc.b 0,128,140,-2,1,2,2,"   BEN DAGLISH    "
  dc.b 0,18,140,3,-2,0,0,"   PRESS PLAY ON TAPE "
  dc.b "                        ",$ff,$ff

setpos_init:
  lda #9
;  lda #20
  sta pos_rate
  lda #0
  sta pos_wait
  lda #0
  sta pos_spr_to_spawn
  sta pos_spawn_x
  sta pos_spawn_x2
  lda #5
  sta pos_spawn_x_spd
  lda #15
  sta pos_spawn_x2_spd

  lda #01
  sta pos_spawn_ind_pos_x_spd
  lda #$fe
  sta pos_spawn_ind_pos_x2_spd


  ldx #MAXSPR                 ;Use all sprites
  stx numsprites
  dex
.initloop:
; txa
; and #$07
; clc
; adc #$21
;                lda #$3f
;  lda #$ff
; sta spry,x
  lda #charset_start  ;$c0   ;sprite at $
  sta sprf,x
  lda #0
  sta pos_en,x
  sta pos_x,x
  sta pos_x2,x
  sta pos_y,x
  dex
  bpl .initloop

  ldx #0
  lda #$c0
.ilop2:
  sta pos_y,x
  clc
  adc #$09
  inx
  cpx #MAXSPR
  bne .ilop2

  jsr setpos
  rts

no_spawn2:
  jmp .no_spawn

setpos:
  dec pos_wait
  bpl no_spawn2
  lda pos_rate
  sta pos_wait

;### Spawn a new character

  ldx pos_spr_to_spawn

  ldy pos_textcou
  lda pos_text,y
  bpl .normal_char
;Goto next part, reset counter if next char also is $ff
  iny
  lda pos_text,y
  bpl .next_part2
  ldy #0
.next_part2:
  sty pos_textcou
  lda #1
  sta part_finished
  rts
.normal_char:
  bne .nonewspd
  iny
  lda pos_text,y
  sta pos_spawn_x
  iny
  lda pos_text,y
  sta pos_spawn_x2
  iny
  lda pos_text,y
  sta pos_spawn_x_spd
  iny
  lda pos_text,y
  sta pos_spawn_x2_spd
  iny
  lda pos_text,y
  sta pos_spawn_ind_pos_x_spd
  iny
  lda pos_text,y
  sta pos_spawn_ind_pos_x2_spd
  iny
  lda pos_text,y

.nonewspd:
  cmp #' 
  bne .nospace
  lda #$40
.nospace:
  cmp #'6
  bne .no6
  lda #'Z+3
.no6:
  cmp #'4
  bne .no4
  lda #'Z+4
.no4:
  cmp #'.
  bne .nodot
  lda #'Z+1
.nodot:
  cmp #'!
  bne .noexkl
  lda #'Z+2
.noexkl:

  and #$3f
  clc
  adc #charset_start ;$c0
  sta sprf,x
  iny
  sty pos_textcou

  lda #$ff
  sta pos_en,x
  lda pos_spawn_x
  sta pos_x,x
  clc
  adc pos_spawn_x_spd
  sta pos_spawn_x

  lda pos_spawn_x2
  sta pos_x2,x
  clc
  adc pos_spawn_x2_spd
  sta pos_spawn_x2

  lda pos_spawn_ind_pos_x_spd
  sta pos_x_spd,x
;  lda $1000,x  ;"Random"
; and #$0f
; sec
; sbc #$7

  lda pos_spawn_ind_pos_x2_spd
  sta pos_x2_spd,x

  lda #$c0
  sta pos_y,x
  inx
  cpx #MAXSPR
  bne .no_wrap
  ldx #0
.no_wrap:
  stx pos_spr_to_spawn


.no_spawn: 

  ldx #MAXSPR-1 ;Use all sprites
.slop:
  lda pos_en,x
  bne .not_disabled
;  lda #$ff
; sta spry,x
  lda #$00
  sta sprx,x
  jmp .end_disabled
.not_disabled:
;  inc pos_x,x
  dec pos_en,x    ;decrement ttl
  lda pos_x,x
  clc
  adc pos_x_spd,x
  sta pos_x,x

  ldy pos_x,x
  lda sintbl,y
  eor #$80
  lsr
  sta sprx,x

;  inc pos_x2,x
;  inc pos_x2,x
  lda pos_x2,x
  clc
  adc pos_x2_spd,x
  sta pos_x2,x

  ldy pos_x2,x
  lda sintbl,y
  eor #$80
  lsr
  lsr
  clc
  adc #$28
  adc sprx,x
  sta sprx,x

.end_disabled:
  dec pos_y,x
  ldy pos_y,x
  lda sintbl,y
  eor #$80
  sta spry,x

.end_slop:
  dex
  bpl .slop
  rts



;setpos:
;  inc tmpcoux
;  ldy tmpcoux
;  ldx #0
;setloopx:
;  lda sintbl,y
;  eor #$80
;  lsr
;  sta sprx,x
;  inx
;  tya
;  clc
;  adc #$05
;  tay
;  cpx #MAXSPR
;  bne setloopx
;
;
;  inc tmpcoux2
;  inc tmpcoux2
;  ldy tmpcoux2
;  ldx #0
;setloopx2:
;  lda sintbl,y
;  eor #$80
;  lsr
;  lsr
;  clc
;  adc #$28
;  adc sprx,x
;  sta sprx,x
;  inx
;  tya
;  clc
;  sbc #$28
;  tay
;  cpx #MAXSPR
;  bne setloopx2
;
;  dec tmpcouy
;  ldy tmpcouy
;  ldx #0
;setloopy:
;  lda sintbl,y
;  eor #$80
;  sta spry,x
;;  lsr
;;  clc
;;  adc #$20
;;  adc spry,x
;;  sta spry,x
;  inx
;  tya
;  clc
;  adc #$09
;  tay
;  cpx #MAXSPR
;  bne setloopy
;  rts

;###
;### initsprites
;###
;Routine to init the sprite multiplexing system
initsprites:
  lda #$00
  sta sortedsprites
  sta sprupdateflag
  ldx #MAXSPR-1                   ;Init the order table with a
.is_orderlist:
  txa                             ;0,1,2,3,4,5... order
  sta sortorder,x
  dex
  bpl .is_orderlist
  rts


;###
;### IRQ_init
;###
;Routine to init the raster interrupt system
IRQ_init:     
 sei
 lda #$35                        ;Set all ROM off, IO on.
 sta $01
 lda #<IRQ_idle                   ;Set an idle-IRQ which will
 sta $0314                       ;be used when KERNAL is on
 lda #>IRQ_idle
 sta $0315

 lda #<nmi                       ;Set NMI, both the hardware
 sta $0318                       ;vector and the vector used by
 sta $fffa                       ;KERNAL
 lda #>nmi
 sta $0319
 sta $fffb

 lda #<IRQ_1            ;Set IRQ vector for first
 sta $fffe                       ;raster interrupt
 lda #>IRQ_1
 sta $ffff
;                lda #<IRQ_1
;                sta $0314
;                lda #>IRQ_1
;                sta $0315
 lda #$7f                    ;CIA interrupt off
 sta $dc0d
 lda #$01                    ;Raster interrupt on
 sta $d01a
 lda #27                     ;High bit of interrupt position = 0
 sta $d011
 lda #IRQ1LINE               ;Line where next IRQ happens
 sta $d012
 lda $dc0d                   ;Acknowledge IRQ (to be sure)
; cli
 rts




;NMI is used as "another raster irq" that enables the sprites
;and makes them white at the top of the screen.
nmi:
; cld
;  pha
  sta nmi_savea
; stx nmi_savex
; sty nmi_savey

; lda #0
; sta $3fff

; lda part_inverted
; eor #$01
  lda #1
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

; lda #3
; sta $d020
; sta $d021
  lda $dd0d ;ack
  lda #$7f                    ;CIA interrupt off
  sta $dd0d

  lda nmi_savea
; ldx nmi_savex
; ldy nmi_savey
;  pla
  rti


;###
;### IRQ
;###

IRQ1LINE  = $ff ;This is the place on screen where the sorting
                ;IRQ happens
IRQ2LINE  = $00 ;This is where sprite displaying begins...
IRQBORDER = $f8

IRQ_border:
 cld
 sta savea
 stx savex
 sty savey
 dec $d019                       ;Acknowledge raster interrupt
 lda #%00010011
 sta $d011
 lda #<IRQ_1                      ;Set up the sprite display IRQ
 sta $fffe
 lda #>IRQ_1
 sta $ffff
 lda #IRQ1LINE
 sta $d012

;waitticks = $146e
waitticks = $156e

;Setup NMI-timer irq for gowhite
  lda #<waitticks
  sta $dd04
  lda #>waitticks
  sta $dd05
  lda $dd0d ;ack NMI-timer irq
  lda #%10011011 ;Start timer
  sta $dd0e
  lda #$81       ;enable NMI
  sta $dd0d

  lda savea
  ldx savex
  ldy savey
  rti

        ;Raster interrupt 1. This is where sorting happens.

IRQ_1:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #$80                        ;Move all sprites
  sta $d001                       ;to the bottom to prevent
  sta $d003                       ;weird effects when sprite
  sta $d005                       ;moves lower than what it
  sta $d007                       ;previously was
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f

  lda #%00011011
;  lda #%00010011
  sta $d011

  lda sprupdateflag               ;New sprites to be sorted?
  beq irq1_nonewsprites
  lda #$00
  sta sprupdateflag
  lda numsprites                  ;Take number of sprites given
                                  ;by the main program
  sta sortedsprites               ;If its zero, dont need to
  bne irq1_beginsort              ;sort

irq1_nonewsprites:
  ldx sortedsprites
  cpx #$09
  bcc .irq1_notmorethan8
  ldx #$08
.irq1_notmorethan8:
  lda d015tbl,x                   ;Now put the right value to
  sta $d015                       ;$d015, based on number of
  beq .irq1_nospritesatall         ;sprites
                                  ;Now init the sprite-counter
  lda #$00                        ;for the actual sprite display
  sta sprirqcounter               ;routine
  lda #<IRQ_2                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>IRQ_2
  sta $ffff
; lda #10
;  lda part_inverted
  lda #14
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e
  jmp irq2_direct                 ;Go directly; we might be late
.irq1_nospritesatall:
 lda savea
 ldx savex
 ldy savey
 rti

irq1_beginsort:
;  lda #1
;  sta $d020
  ldx #MAXSPR
  dex
  cpx sortedsprites
  bcc .irq1_cleardone
  lda #$ff                        ;Mark unused sprites with the
.irq1_clearloop:
  sta spry,x                      ;lowest Y-coordinate ($ff);
  dex                             ;these will "fall" to the
  cpx sortedsprites               ;bottom of the sorted table
  bcs .irq1_clearloop
.irq1_cleardone:

  ldx #$00
irq1_sortloop:
  ldy sortorder+1,x               ;Sorting code. Algorithm
  lda spry,y                      ;ripped from Dragon Breed :-)
  ldy sortorder,x
  cmp spry,y
  bcs irq1_sortskip
  stx irq1_sortreload+1
irq1_sortswap:
  lda sortorder+1,x
  sta sortorder,x
  sty sortorder+1,x
  cpx #$00
  beq irq1_sortreload
  dex
  ldy sortorder+1,x
  lda spry,y
  ldy sortorder,x
  cmp spry,y
  bcc irq1_sortswap
irq1_sortreload:
  ldx #$00
irq1_sortskip:
  inx
  cpx #MAXSPR-1
  bcc irq1_sortloop

  ldx sortedsprites
  lda #$ff                       ;$ff is the endmark for the
  sta sortspry,x                 ;sprite interrupt routine

  ldx #$00
irq1_sortloop3:
  ldy sortorder,x                ;Final loop:
  lda spry,y                     ;Now copy sprite variables to
  sta sortspry,x                 ;the sorted table
  lda sprx,y
  sta sortsprx,x
  lda sprf,y
  sta sortsprf,x
  inx
  cpx sortedsprites
  bcc irq1_sortloop3
;  lda #0
; sta $d020
  lda #%00011011
  sta $d011
  jmp irq1_nonewsprites


;Make all sprites white at this raster line
;to simulate a "upper border" where sprites
;disappear

IRQ_gowhite:
; lda #1
; sta $d027
; sta $d028
; sta $d029
; sta $d02a
; sta $d02b
; sta $d02c
; sta $d02d
; sta $d02e
;The next time, do a "normal" IRQ_2, without whitening
; lda #5
; sta $d020
;  lda #<IRQ_2
;  sta $fffe
;  lda #>IRQ_2
;  sta $ffff

        ;Raster interrupt 2. This is where sprite displaying happens

IRQ_2:
  cld
  sta savea
  stx savex
  sty savey
irq2_direct:
  dec $d019                       ;Acknowledge raster interrupt
;  lda #6
;  sta $d021

  ldy sprirqcounter               ;Take next sorted sprite number
  lda sortspry,y                  ;Take Y-coord of first new sprite
  clc
  adc #$10                        ;16 lines down from there is
  bcc irq2_notover                ;the endpoint for this IRQ
  lda #$ff                        ;Endpoint cant be more than $ff
irq2_notover:
  sta tempvariable
irq2_spriteloop:
  lda sortspry,y
  cmp tempvariable                ;End of this IRQ?
  bcs irq2_endspr
  ldx physicalsprtbl2,y           ;Physical sprite number x 2
  sta $d001,x                     ;for X & Y coordinate
  lda sortsprx,y
;  asl
;Make sprites 256 positions centered
  clc
  adc #$30
  sta $d000,x
  bcc irq2_lowmsb
  lda $d010
  ora ortbl,x
  sta $d010
  jmp irq2_msbok
irq2_lowmsb:
  lda $d010
  and andtbl,x
  sta $d010
irq2_msbok:
  ldx physicalsprtbl1,y           ;Physical sprite number x 1
  lda sortsprf,y
  sta $07f8,x                     ;for color & frame
  iny
  bne irq2_spriteloop
irq2_endspr:
  cmp #$ff                        ;Was it the endmark?
  beq irq2_lastspr
  sty sprirqcounter
  sec                             ;That coordinate - $10 is the
  sbc #$10                        ;position for next interrupt
  cmp $d012                       ;Already late from that?
  bcc irq2_direct                 ;Then go directly to next IRQ
  sta $d012
; lda #0
; sta $d021
  lda savea
  ldx savex
  ldy savey
  rti

irq2_lastspr:
  lda #<IRQ_border                      ;Was the last sprite,
  sta $fffe                       ;go back to irq1
  lda #>IRQ_border                      ;(sorting interrupt)
  sta $ffff
  lda #IRQBORDER
  sta $d012
; lda #14
; sta $d021
  lda savea
  ldx savex
  ldy savey
  rti

sprx:
  ds.b MAXSPR,0                   ;Unsorted sprite table
spry:
  ds.b MAXSPR,0
sprf:
  ds.b MAXSPR,0

sortsprx:
  ds.b MAXSPR,0                   ;Sorted sprite table
sortspry:
  ds.b MAXSPR+1,0                 ;Must be one byte extra for the
                                                ;$ff endmark
sortsprf:
  ds.b MAXSPR,0


d015tbl:
  dc.b %00000000                  ;Table of sprites that are "on"
  dc.b %00000001                  ;for $d015
  dc.b %00000011
  dc.b %00000111
  dc.b %00001111
  dc.b %00011111
  dc.b %00111111
  dc.b %01111111
  dc.b %11111111

physicalsprtbl1:
  dc.b 0,1,2,3,4,5,6,7            ;Indexes to frame & color
  dc.b 0,1,2,3,4,5,6,7      ;registers
  dc.b 0,1,2,3,4,5,6,7
  dc.b 0,1,2,3,4,5,6,7
  dc.b 0,1,2,3,4,5,6,7
  dc.b 0,1,2,3,4,5,6,7
  dc.b 0,1,2,3,4,5,6,7
  dc.b 0,1,2,3,4,5,6,7

physicalsprtbl2:
  dc.b 0,2,4,6,8,10,12,14
  dc.b 0,2,4,6,8,10,12,14
  dc.b 0,2,4,6,8,10,12,14
  dc.b 0,2,4,6,8,10,12,14
  dc.b 0,2,4,6,8,10,12,14
  dc.b 0,2,4,6,8,10,12,14
  dc.b 0,2,4,6,8,10,12,14
  dc.b 0,2,4,6,8,10,12,14

andtbl:
  dc.b 255-1
ortbl:
  dc.b 1
  dc.b 255-2
  dc.b 2
  dc.b 255-4
  dc.b 4
  dc.b 255-8
  dc.b 8
  dc.b 255-16
  dc.b 16
  dc.b 255-32
  dc.b 32
  dc.b 255-64
  dc.b 64
  dc.b 255-128
  dc.b 128


 seg sprites
charset_start = $d0;

 org $3400

;This is a very nice sprite charset.
;Drawn in this text file... :)
sprites:
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %00111110,%00000000,%11111110
  dc.b %00000000,%00000000,%11111110
  dc.b %00011111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%10000000,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %00011111,%11111110,%11111111
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %11111111,%10000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%01111111,%00000000
  dc.b %01111111,%11111111,%11110000
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%10000001,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%10000001,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %11111110,%11111111,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000011,%11111111,%11000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00011111,%11111111,%11100000
  dc.b %00111111,%11111111,%11000000
  dc.b %00111111,%11111111,%10000000
  dc.b %01111111,%11111111,%00000000
  dc.b %01111111,%11111110,%00000000
  dc.b %01111111,%11111100,%00000000
  dc.b %11111111,%11111000,%00000000
  dc.b %01111111,%11111000,%00000000
  dc.b %01111111,%11111100,%00000000
  dc.b %01111111,%11111110,%00000000
  dc.b %00111111,%11111111,%00000000
  dc.b %00111111,%11111111,%10000000
  dc.b %00011111,%11111111,%11000000
  dc.b %00001111,%11111111,%11100000
  dc.b %00000011,%11111111,%11000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000001,%11111111
  dc.b %00000000,%00000000,%11111110
  dc.b %00000000,%00000000,%11111110
  dc.b %00000000,%00000000,%11111110
  dc.b %00000000,%00000000,%11111110
  dc.b %00000000,%00000000,%11111110
  dc.b %00000000,%11111111,%11111110
  dc.b %00001111,%11111111,%11111110
  dc.b %00111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%10000001,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%10000001,%11111110
  dc.b %00111111,%11111111,%01111110
  dc.b %00001111,%11111110,%00111111
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00001111,%11100000
  dc.b %11111111,%11111111,%00000000
  dc.b %11111111,%11110000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00000000,%11110000
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%11111111,%11111100
  dc.b %00011111,%11111111,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111110
  dc.b %11111111,%10000001,%11111100
  dc.b %11111111,%00000000,%01110000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%11111111,%10000000
  dc.b %11111111,%11111111,%11000000
  dc.b %11111111,%10000001,%10000000
  dc.b %11111111,%10000000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %01111110,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111100
  dc.b %11111111,%10000001,%11111000
  dc.b %11111111,%00000000,%11100000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00011111,%11100000
  dc.b %11111111,%00111111,%11111100
  dc.b %11111111,%00010000,%11111100
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%11111111,%11111100
  dc.b %00011111,%11111111,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %11111111,%10000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %01111111,%01111111,%00000000
  dc.b %01111111,%11111111,%11110000
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%10000001,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111111
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00111100,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%01111000,%00000000
  dc.b %00000000,%11111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111110,%00000000
  dc.b %00000000,%01111100,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00111100,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%01111000,%00000000
  dc.b %00000000,%11111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %01111000,%11111111,%00000000
  dc.b %11111110,%11111111,%00000000
  dc.b %11111111,%11111111,%00000000
  dc.b %01111111,%11111110,%00000000
  dc.b %00011111,%11111100,%00000000
  dc.b %00000000,%00000000,%00000000,0

;k
  dc.b %00000000,%00000000,%00000000
  dc.b %00011110,%00000000,%00000000
  dc.b %00111111,%00000000,%00000000
  dc.b %00111111,%10000000,%00000000
  dc.b %01111111,%10000000,%01111000
  dc.b %01111111,%00000000,%11111100
  dc.b %01111111,%00000001,%11111110
  dc.b %11111111,%00000011,%11111110
  dc.b %11111111,%00000111,%11111110
  dc.b %11111111,%10001111,%11111110
  dc.b %11111111,%11011111,%11111100
  dc.b %11111111,%11111111,%11110000
  dc.b %11111111,%11111111,%11000000
  dc.b %11111111,%11111111,%00000000
  dc.b %11111111,%11111111,%11000000
  dc.b %11111111,%01111111,%11110000
  dc.b %11111111,%00111111,%11111100
  dc.b %01111111,%00011111,%11111100
  dc.b %01111111,%00000111,%11111110
  dc.b %00111110,%00000001,%11111100
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111111,%01110000
  dc.b %00000000,%01111111,%11111000
  dc.b %00000000,%00111111,%11110000
  dc.b %00000000,%00011111,%11000000
  dc.b %00000000,%00000000,%00000000,0
;m
  dc.b %00000000,%00000000,%00000000
  dc.b %11100000,%00000000,%00000111
  dc.b %01110000,%00000000,%00001110
  dc.b %01111000,%00000000,%00011110
  dc.b %01111100,%00000000,%00111110
  dc.b %01111110,%00000000,%01111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%10000001,%11111110
  dc.b %01111111,%11000011,%11111110
  dc.b %01111111,%11100111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%01111110,%11111110
  dc.b %01111111,%00111100,%11111110
  dc.b %01111111,%00011000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111111
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %11111000,%01111111,%00000000
  dc.b %01111110,%11111111,%11110000
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%10000001,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111111
  dc.b %00000000,%00000000,%00000000,0


  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000011,%11111111,%11000000
  dc.b %00001111,%11111111,%00110000
  dc.b %00011111,%11111111,%00001000
  dc.b %00111111,%11111111,%00000100
  dc.b %00111111,%11111111,%00000100
  dc.b %01111111,%11111111,%00000010
  dc.b %01111111,%11111111,%00000010
  dc.b %11111111,%11111111,%00000001
  dc.b %11111111,%11111111,%00000001
  dc.b %11111111,%11111111,%00000001
  dc.b %01111111,%11111111,%00000010
  dc.b %01111111,%11111111,%00000010
  dc.b %00111111,%11111111,%00000100
  dc.b %00111111,%11111111,%00000100
  dc.b %00011111,%11111111,%00001000
  dc.b %00001111,%11111111,%00110000
  dc.b %00000011,%11111111,%11000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00000000,%00000000,0
;p
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%11110000,%11111100
  dc.b %11111111,%11111111,%11111100
  dc.b %11111111,%00001111,%11111000
  dc.b %11111111,%00000000,%11000000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%00000000,%00000000
  dc.b %01111111,%00000000,%00000000
  dc.b %00111110,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11110011,%11111110
  dc.b %11111111,%11100001,%11111111
  dc.b %11111111,%10000001,%11111111
  dc.b %11111111,%00000000,%11111111
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%00001100,%11111110
  dc.b %11111111,%00001110,%01111110
  dc.b %11111111,%00000111,%01111100
  dc.b %11111111,%00000011,%11111100
  dc.b %11111111,%00000001,%11111100
  dc.b %11111111,%00000000,%11110000
  dc.b %11111111,%10000001,%11111000
  dc.b %01111111,%11111111,%11011100
  dc.b %00011111,%11111111,%10001110
  dc.b %00000000,%00000000,%00000000,0
;r
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%11110000,%11111000
  dc.b %11111111,%11111111,%11100000
  dc.b %11111111,%00001111,%11110000
  dc.b %11111111,%00000001,%11111000
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00000001,%11111110
  dc.b %11111111,%00000011,%11111111
  dc.b %01111111,%00000001,%11111100
  dc.b %00011111,%00000000,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111110,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111100
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00000000,%01110000
  dc.b %01111111,%00000000,%00000000
  dc.b %00011111,%11110000,%00000000
  dc.b %0000011,%111111111,%11000000
  dc.b %00000000,%00001111,%11111000
  dc.b %00000000,%00000000,%11111100
  dc.b %00001110,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%11111111,%11111100
  dc.b %00011111,%11111111,%11110000
  dc.b %00000000,%00000000,%00000000,0
;t
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%10000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111111
  dc.b %11111111,%11111111,%11111111
  dc.b %11110000,%11111111,%01111110
  dc.b %01100000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11000000,%00000000
  dc.b %00001111,%11100001,%11110000
  dc.b %00111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111110
  dc.b %11111111,%11110011,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%00000000,%11111111
  dc.b %11111111,%00000000,%11111111
  dc.b %11111111,%00000000,%11111111
  dc.b %11111111,%00000000,%11111111
  dc.b %11111111,%00000000,%11111111
  dc.b %11111111,%10000000,%11111110
  dc.b %11111111,%10000000,%11111110
  dc.b %01111111,%10000000,%11111110
  dc.b %01111111,%11000001,%11111110
  dc.b %00111111,%11111111,%11111100
  dc.b %00001111,%11111111,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11000000,%00000000
  dc.b %00001111,%11100000,%00011000
  dc.b %00111111,%11110000,%00111100
  dc.b %01111111,%11110000,%00111100
  dc.b %01111111,%11110000,%00111110
  dc.b %11111111,%11100000,%01111110
  dc.b %11111111,%10000000,%01111100
  dc.b %11111111,%00000000,%01111100
  dc.b %11111111,%00000000,%11111000
  dc.b %01111111,%10000000,%11110000
  dc.b %01111111,%11000000,%11110000
  dc.b %01111111,%11000001,%11100000
  dc.b %00111111,%11100001,%11100000
  dc.b %00001111,%11100001,%11000000
  dc.b %00000011,%11100011,%11000000
  dc.b %00000001,%11110011,%10000000
  dc.b %00000000,%11110111,%10000000
  dc.b %00000000,%01111111,%00000000
  dc.b %00000000,%00111110,%00000000
  dc.b %00000000,%00000000,%00000000,0
;w
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11000000,%00000000
  dc.b %00001111,%11100001,%11110000
  dc.b %00111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111110
  dc.b %11111111,%11110011,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111111
  dc.b %11111111,%00000000,%11111111
  dc.b %11111111,%00001000,%11111111
  dc.b %11111111,%00001100,%11111111
  dc.b %11111111,%00011100,%11111111
  dc.b %11111111,%00111110,%11111111
  dc.b %11111111,%01111111,%11111110
  dc.b %11111111,%11110111,%11111110
  dc.b %11111111,%11110111,%11111110
  dc.b %11111111,%11100111,%11111110
  dc.b %01111111,%11100011,%11111100
  dc.b %00011111,%11000001,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11000000,%00000000
  dc.b %00001111,%11100001,%11110000
  dc.b %00111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111100
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00000001,%11111000
  dc.b %01111111,%00000001,%11110000
  dc.b %00011111,%11110011,%11100000
  dc.b %00000111,%11111111,%11100000
  dc.b %00000000,%11111111,%11110000
  dc.b %00000001,%11111111,%11111000
  dc.b %00000111,%11111111,%11111100
  dc.b %00111111,%11000111,%11111110
  dc.b %11111111,%11000001,%11111110
  dc.b %01111111,%11000001,%11111100
  dc.b %00011111,%10000000,%01110000
  dc.b %00000000,%00000000,%00000000,0
;y
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11000000,%00000000
  dc.b %00001111,%11100001,%11110000
  dc.b %00111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111110
  dc.b %11111111,%11110011,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %00011111,%11110000,%11111110
  dc.b %0000011,%111111111,%11111110
  dc.b %00000000,%00001111,%11111111
  dc.b %00000000,%00000000,%11111111
  dc.b %00001110,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%11111111,%11111100
  dc.b %00011111,%11111111,%11110000
  dc.b %00000000,%00000000,%00000000,0
;z
  dc.b %00000000,%00000000,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11111111,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %00111000,%00000000,%11111100
  dc.b %00000000,%00000000,%11111100
  dc.b %00000000,%00000000,%11111000
  dc.b %00000000,%00011111,%11100000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000001,%11110000,%00000000
  dc.b %00000111,%10000000,%01111000
  dc.b %00111111,%00000000,%11111100
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%11111111,%11111100
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00111100,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111100,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%01111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111110,%00000000
  dc.b %00000000,%11111110,%00000000
  dc.b %00000000,%01111100,%00000000
  dc.b %00000000,%00110000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00111100,%00000000
  dc.b %00000000,%00000000,%00000000,0
;
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00001111,%11111111,%11110000
  dc.b %00111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111100
  dc.b %01111111,%11111111,%11111110
  dc.b %11111111,%11110011,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00000000,%01110000
  dc.b %11111111,%00000000,%00000000
  dc.b %11111111,%11110000,%00000000
  dc.b %11111111,%11111111,%00000000
  dc.b %11111111,%00001111,%11110000
  dc.b %11111111,%00000000,%11111100
  dc.b %11111111,%00000000,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %01111111,%11111111,%11111100
  dc.b %00011111,%11111111,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%11000000,%00000000
  dc.b %00001111,%11100001,%11110000
  dc.b %00111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111100
  dc.b %01111111,%11110011,%11111110
  dc.b %11111111,%11110011,%11111110
  dc.b %11111111,%11100001,%11111110
  dc.b %11111111,%10000001,%11111110
  dc.b %11111111,%00000000,%11111110
  dc.b %01111111,%00000000,%11111110
  dc.b %00011111,%11110000,%11111110
  dc.b %0000011,%111111111,%11111110
  dc.b %00000000,%00001111,%11111111
  dc.b %00000000,%00000000,%11111111
  dc.b %00000000,%00000000,%11111110
  dc.b %00000000,%00000000,%11111110
  dc.b %00000000,%00000001,%11111110
  dc.b %00000000,%00000001,%11111100
  dc.b %00000000,%00000000,%11110000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000010,%00000000
  dc.b %00000000,%00000001,%00000000
  dc.b %00000000,%00000001,%10000000
  dc.b %00000000,%01000000,%11100000
  dc.b %00000000,%00110000,%11110000
  dc.b %00000000,%00011110,%01011000
  dc.b %00000000,%00001111,%11001100
  dc.b %00000000,%01100110,%01100110
  dc.b %00000000,%11110011,%00100011
  dc.b %00000000,%10011001,%10000000
  dc.b %00000001,%10001100,%11000000
  dc.b %00110001,%10001100,%01100000
  dc.b %01111001,%10001100,%00000000
  dc.b %11000000,%11011000,%00000000
  dc.b %11000000,%01110000,%00000000
  dc.b %01100011,%00000000,%00000000
  dc.b %00111110,%00000000,%00000000
  dc.b %11011100,%00000000,%00000000
  dc.b %11000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00001111,%10001110,%00000011
  dc.b %00111111,%11001110,%00000111
  dc.b %01111000,%01101110,%00000111
  dc.b %01110000,%00001110,%00000111
  dc.b %01110000,%00001110,%00000111
  dc.b %11100000,%00001110,%00000111
  dc.b %11100000,%00001110,%00000111
  dc.b %11100000,%00001110,%00000111
  dc.b %11100000,%00001110,%00000111
  dc.b %11111111,%00001110,%00000111
  dc.b %11111111,%11000111,%11111111
  dc.b %11110000,%11100000,%00000111
  dc.b %11100000,%11100000,%00000111
  dc.b %11100000,%11100000,%00000111
  dc.b %11100000,%11100000,%00000111
  dc.b %11110000,%11100000,%00000111
  dc.b %11110000,%11100000,%00000111
  dc.b %00111100,%11000000,%00000111
  dc.b %00001111,%00000000,%00000111
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000011,%11000000,%01110000
  dc.b %00001111,%11100001,%11111100
  dc.b %00111111,%11110011,%11111110
  dc.b %01111111,%11111011,%11111110
  dc.b %01111111,%11111111,%11111111
  dc.b %11111111,%11111111,%11111111
  dc.b %11111111,%11111111,%11111111
  dc.b %01111111,%11111111,%11111110
  dc.b %01111111,%11111111,%11111110
  dc.b %00111111,%11111111,%11111100
  dc.b %00111111,%11111111,%11111100
  dc.b %00011111,%11111111,%11111000
  dc.b %00001111,%11111111,%11111000
  dc.b %00000111,%11111111,%11110000
  dc.b %00000011,%11111111,%11100000
  dc.b %00000001,%11111111,%11000000
  dc.b %00000000,%11111111,%10000000
  dc.b %00000000,%01111111,%00000000
  dc.b %00000000,%00011110,%00000000
  dc.b %00000000,%00000000,%00000000,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00111100,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00111000,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%11111111,%00000000
  dc.b %00000000,%01111110,%00000000
  dc.b %00000000,%00111100,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000,0

; dc.b %00000000,%00000000,%00000000
; dc.b %00000001,%10000000,%00000000
; dc.b %00001111,%11000011,%11100000
; dc.b %00111111,%11100111,%11110000
; dc.b %01111111,%11100111,%11110001
; dc.b %01111111,%11100111,%11111001
; dc.b %11111111,%11100111,%11111011
; dc.b %11111111,%11000011,%11111011
; dc.b %11111111,%00000011,%11111101
; dc.b %11111110,%00000001,%11111101
; dc.b %11111110,%00010001,%11111100
; dc.b %11111110,%00011001,%11111100
; dc.b %11111110,%00111001,%11111100
; dc.b %11111110,%01111101,%11111101
; dc.b %11111110,%11111111,%11111011
; dc.b %11111111,%11101111,%11111011
; dc.b %11111111,%11101111,%11111011
; dc.b %11111111,%11001111,%11111011
; dc.b %01111111,%11000111,%11110001
; dc.b %00011111,%10000011,%11100000
; dc.b %00000000,%00000000,%00000000,0
;
; dc.b %00000000,%00000000,%00000000
; dc.b %00000011,%11111100,%00000000
; dc.b %00111111,%11111111,%11000001
; dc.b %11111111,%11111111,%11110011
; dc.b %11111111,%11111111,%11110011
; dc.b %11111111,%11111111,%11111001
; dc.b %11111111,%11111111,%11111000
; dc.b %11111110,%00000111,%11111000
; dc.b %11111100,%00000011,%11111001
; dc.b %11111100,%00000011,%11111011
; dc.b %11111000,%00000011,%11111011
; dc.b %00000000,%00000011,%11111011
; dc.b %01111111,%11111111,%11111011
; dc.b %11111111,%11111111,%11111011
; dc.b %11111110,%00000111,%11111011
; dc.b %11111110,%00000011,%11111011
; dc.b %11111100,%00000011,%11111011
; dc.b %11111110,%00000111,%11111011
; dc.b %11111111,%11111111,%11111011
; dc.b %01111111,%11111011,%11111101
; dc.b %00000000,%00000000,%00000000,0
;
; dc.b %00000000,%00000000,%00000000
; dc.b %11100000,%00111111,%11100000
; dc.b %11111000,%11111111,%11111000
; dc.b %11111001,%11111111,%11111100
; dc.b %11111011,%11111111,%11111110
; dc.b %11111011,%11111111,%11111111
; dc.b %11110011,%11111111,%11111111
; dc.b %00000011,%10001111,%11111111
; dc.b %11100001,%00001111,%11100111
; dc.b %11111000,%00001111,%11100010
; dc.b %11111100,%00001111,%11100000
; dc.b %11111100,%00001111,%11100000
; dc.b %11111100,%00001111,%11100000
; dc.b %11111100,%00011111,%11100000
; dc.b %11111100,%00011111,%11100000
; dc.b %11111100,%00011111,%11100000
; dc.b %11111100,%00011111,%11100000
; dc.b %11111100,%00011111,%11000000
; dc.b %11111000,%00001111,%11000000
; dc.b %11110000,%00000111,%10000000
; dc.b %00000000,%00000000,%00000000,0
;
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111
; dc.b %11111111,%11111111,%11111111,0


 seg sprites_to_draw_in
 org $3e00
spr_to_draw_in:
 ds.b $40*7,$00
 ds.b $3f,$00

 seg 3fff
 org $3fff
 dc.b 00

 seg spr_ytab
 org $4000

up_sprites:

up_tmpsprites1 = * + $2000 ;which is $6000-$61ff
up_tmpsprites2 = * + $2100


nof_ytables = 0 ;which is 256, really...
spr_ytables:
  ds.b 256*32,$0  ;256 tables with 32 bytes in each

  seg enlargeytable
  org $6000

;Room for 16 tables with 256 bytes in each
;This table is just an example:
ytable:
ytable_00:
  ds.b $100,$0
ytable_01:
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0

  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
  ds.b $100,$0
ytable_end:

;$7000

 seg seg8000
 org $8000


;###
;###
;###

; org $6000
;This endscroller charset was done by Shaman of Taboo
up_charset:
;@
 dc.b %00001111
 dc.b %00111000
 dc.b %01110000
 dc.b %01110000
 dc.b %11100111
 dc.b %11101100
 dc.b %11100000
 dc.b %11100111
 dc.b %11101100
 dc.b %11101100
 dc.b %11101100
 dc.b %11100111
 dc.b %01110000
 dc.b %01110000
 dc.b %00111100
 dc.b %00001111

 dc.b %11100000
 dc.b %01111000
 dc.b %00011100
 dc.b %00011100
 dc.b %11001110
 dc.b %01101110
 dc.b %01101110
 dc.b %11101110
 dc.b %01101110
 dc.b %01101110
 dc.b %01101110
 dc.b %10111100
 dc.b %00000000
 dc.b %00011100
 dc.b %00111100
 dc.b %11110000
 
;a
 dc.b $00,$00,$7e,$e7,$e7,$07,$7f,$e7
 dc.b $e7,$e7,$e7,$e7,$7b,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;b
 dc.b $e0,$e0,$fe,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$de,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;c
 dc.b $00,$00,$7e,$e7,$e7,$e0,$e0,$e0
 dc.b $e0,$e0,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;d
 dc.b $07,$07,$7f,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7b,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;e
 dc.b $00,$00,$7e,$e7,$e7,$e7,$ff,$e0
 dc.b $e0,$e0,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;f
 dc.b $38,$70,$70,$70,$70,$70,$fc,$70
 dc.b $70,$70,$70,$70,$70,$70,$70,$70
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;g
 dc.b $00,$00,$7e,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7f,$07,$e7,$7e
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;h
 dc.b $e0,$e0,$fe,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;i
 dc.b $e0,$e0,$e0,$00,$e0,$e0,$e0,$e0
 dc.b $e0,$e0,$e0,$e0,$70,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;j
 dc.b $70,$70,$70,$00,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$70,$70,$70,$e0
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;k
 dc.b $e0,$e0,$e7,$e7,$e6,$fc,$e6,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;l
 dc.b $e0,$e0,$e0,$e0,$e0,$e0,$e0,$e0
 dc.b $e0,$e0,$e0,$e0,$e0,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;m
 dc.b $00,$00,$dd,$e6,$e6,$e6,$e6,$e6
 dc.b $e6,$e6,$e6,$e6,$e6,$00,$00,$00
 dc.b $00,$00,$e0,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$70,$00,$00,$00
;n
 dc.b $00,$00,$de,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;o
 dc.b $00,$00,$7e,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;p
 dc.b $00,$00,$fe,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$fe,$e0,$e0,$e0
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;q
 dc.b $00,$00,$7f,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7f,$07,$07,$07
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;r
 dc.b $00,$00,$de,$e7,$e7,$e0,$e0,$e0
 dc.b $e0,$e0,$e0,$e0,$e0,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;s
 dc.b $00,$00,$7f,$e7,$e7,$e0,$e0,$7e
 dc.b $07,$07,$e7,$e7,$fe,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;t
 dc.b $70,$70,$f8,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$38,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;u
 dc.b $00,$00,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;v
 dc.b $00,$00,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$66,$3c,$18,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;w
 dc.b $00,$00,$e6,$e6,$e6,$e6,$e6,$e6
 dc.b $e6,$e6,$e6,$e6,$7f,$00,$00,$00
 dc.b $00,$00,$70,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$e0,$00,$00,$00
;x
 dc.b $00,$00,$e7,$e7,$e7,$e7,$e7,$7e
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;y
 dc.b $00,$00,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7f,$07,$e7,$7e
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;z
 dc.b $00,$00,$ff,$e7,$07,$0e,$1c,$38
 dc.b $70,$e0,$e7,$e7,$ff,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;[
 dc.b $f8,$e0,$e0,$e0,$e0,$e0,$e0,$e0
 dc.b $e0,$e0,$e0,$e0,$e0,$f8,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;o + /
 dc.b $00,$00,$7e,$e7,$e7,$e7,$ef,$ff
 dc.b $f7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;]
 dc.b $f8,$38,$38,$38,$38,$38,$38,$38
 dc.b $38,$38,$38,$38,$38,$f8,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;o + " = 
 dc.b $e7,$00,$7e,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;
 dc.b $3c,$00,$7e,$e7,$e7,$07,$7f,$e7
 dc.b $e7,$e7,$e7,$e7,$7b,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;" "
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;!
 dc.b $e0,$e0,$e0,$e0,$e0,$e0,$e0,$e0
 dc.b $e0,$00,$e0,$e0,$e0,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;"
 dc.b $e7,$e7,$e7,$63,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;" - endquote
 dc.b $e7,$e7,$e7,$c6,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;$
 dc.b $18,$18,$7e,$e7,$e7,$e0,$7e,$07
 dc.b $07,$e7,$e7,$7e,$18,$18,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;%
 dc.b $e3,$e3,$e6,$06,$0c,$0c,$18,$18
 dc.b $30,$30,$60,$67,$c7,$c7,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;& (empty)
;e'
 dc.b $0c,$30,$7e,$e7,$e7,$e7,$ff,$e0
 dc.b $e0,$e0,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;'
 dc.b $e0,$e0,$e0,$c0,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;(
 dc.b $70,$e0,$e0,$e0,$e0,$e0,$e0,$e0
 dc.b $e0,$e0,$e0,$e0,$e0,$70,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;)
 dc.b $e0,$70,$70,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$70,$e0,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
; = a + "
 dc.b $66,$00,$7e,$e7,$e7,$07,$7f,$e7
 dc.b $e7,$e7,$e7,$e7,$7b,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;+
 dc.b $00,$00,$00,$00,$38,$38,$fe,$38
 dc.b $38,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;,
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$e0,$e0,$e0,$c0,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;-
 dc.b $00,$00,$00,$00,$00,$00,$fe,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;.
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$e0,$e0,$e0,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;/
 dc.b $03,$03,$06,$06,$0c,$0c,$18,$18
 dc.b $30,$30,$60,$60,$c0,$c0,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;0
 dc.b $7e,$e7,$e7,$e7,$e7,$ef,$ff,$f7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;1
 dc.b $1c,$3c,$7c,$dc,$1c,$1c,$1c,$1c
 dc.b $1c,$1c,$1c,$1c,$1c,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;2
 dc.b $7e,$e7,$e7,$e7,$07,$0e,$1c,$38
 dc.b $70,$e0,$e7,$e7,$ff,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;3
 dc.b $7e,$e7,$e7,$e7,$07,$07,$1e,$07
 dc.b $07,$07,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;4
 dc.b $e7,$e7,$e7,$e7,$e7,$e7,$7f,$07
 dc.b $07,$07,$07,$07,$07,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
;5
 dc.b $ff,$e7,$e7,$e0,$e0,$e0,$fe,$07
 dc.b $07,$07,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e0,$e0,$e0,$fe,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $ff,$e7,$07,$07,$07,$07,$1f,$07
 dc.b $07,$07,$07,$07,$07,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$80,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e7,$e7,$e7,$7e,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e7,$e7,$e7,$7f,$07
 dc.b $07,$07,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$e0,$e0,$e0,$00,$e0
 dc.b $e0,$e0,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$e0,$e0,$e0,$00,$e0
 dc.b $e0,$e0,$60,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$fe,$00,$fe
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e7,$07,$07,$1e,$38
 dc.b $38,$00,$38,$38,$38,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e7,$e7,$e7,$ff,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $fe,$e7,$e7,$e7,$e7,$e7,$fe,$e7
 dc.b $e7,$e7,$e7,$e7,$fe,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e0,$e0,$e0,$e0,$e0
 dc.b $e0,$e0,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $fe,$e7,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$fe,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $fe,$e0,$e0,$e0,$e0,$e0,$f8,$e0
 dc.b $e0,$e0,$e0,$e0,$fe,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $fe,$e0,$e0,$e0,$e0,$e0,$f8,$e0
 dc.b $e0,$e0,$e0,$e0,$e0,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e0,$e0,$e0,$ef,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $e7,$e7,$e7,$e7,$e7,$e7,$ff,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $f8,$70,$70,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$f8,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $70,$70,$70,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$70,$70,$70,$e0
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $e7,$e7,$e7,$e7,$e6,$fc,$e6,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $e0,$e0,$e0,$e0,$e0,$e0,$e0,$e0
 dc.b $e0,$e0,$e0,$e0,$fc,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7f,$e6,$e6,$e6,$e6,$e6,$e6,$e6
 dc.b $e6,$e6,$e6,$e6,$e6,$00,$00,$00
 dc.b $e0,$70,$70,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$70,$00,$00,$00
 dc.b $fe,$e7,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $fe,$e7,$e7,$e7,$e7,$e7,$fe,$e0
 dc.b $e0,$e0,$e0,$e0,$e0,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7e,$07,$03,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $fe,$e7,$e7,$e7,$e7,$e7,$fe,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $7e,$e7,$e7,$e7,$e0,$e0,$7e,$07
 dc.b $07,$e7,$e7,$e7,$fe,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $ff,$1c,$1c,$1c,$1c,$1c,$1c,$1c
 dc.b $1c,$1c,$1c,$1c,$1c,$00,$00,$00
 dc.b $80,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $e7,$e7,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$e7,$e7,$7f,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $e7,$e7,$e7,$e7,$e7,$e7,$e7,$e7
 dc.b $e7,$e7,$66,$3c,$18,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $e6,$e6,$e6,$e6,$e6,$e6,$e6,$e6
 dc.b $e6,$e6,$e6,$e6,$7f,$00,$00,$00
 dc.b $70,$70,$70,$70,$70,$70,$70,$70
 dc.b $70,$70,$70,$70,$e0,$00,$00,$00
 dc.b $e7,$e7,$e7,$e7,$e7,$e7,$7e,$e7
 dc.b $e7,$e7,$e7,$e7,$e7,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $e7,$e7,$e7,$e7,$e7,$e7,$7f,$07
 dc.b $07,$07,$e7,$e7,$7e,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $ff,$e7,$e7,$e7,$07,$0e,$1c,$38
 dc.b $70,$e7,$e7,$e7,$ff,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00
 dc.b $00,$00,$00,$00,$00,$00,$00,$00


; org $5e00
up_charwidths:
;a-z
 dc.b $10, $09, $09, $09, $09, $09, $07, $09
 dc.b $09, $05, $05, $09, $04, $0d, $09, $09
 dc.b $09, $09, $09, $09, $06, $09, $09, $0d
 dc.b $09, $09, $09, $06, $09, $06, $09, $09 ;xyz[o]
;$20-$3f
 dc.b $06, $04, $09, $09, $09, $09, $09, $04 ; !"#%&   ;&=
 dc.b $05, $05, $09, $08, $04, $08, $04, $09
 dc.b $09, $07, $09, $09, $09, $09, $09, $0a
 dc.b $09, $09, $04, $04, $00, $08, $00, $09
;A-Z
 dc.b $00, $09, $09, $09, $09, $08, $08, $09
 dc.b $09, $06, $05, $09, $07, $0d, $09, $09
 dc.b $09, $09, $09, $09, $0a, $09, $09, $0d
 dc.b $09, $09, $09, $00, $00, $00, $00, $00
;a-z
 dc.b $10, $09, $09, $09, $09, $09, $07, $09
 dc.b $09, $05, $05, $09, $04, $0d, $09, $09
 dc.b $09, $09, $09, $09, $06, $09, $09, $0d
 dc.b $09, $09, $09, $06, $09, $06, $09, $09 ;xyz[o]


;###
;### up-scroller
;### 


 seg.u zp_up
 org $02
up_toggle:             dc.b 0  
up_current_row:        dc.b 0  
up_current_rowpixel:   dc.b 0  
up_irq_counter:        dc.b 0
up_sprites_ypos:       dc.b 0  
up_current_displayrow: dc.b 0  
up_textpoi:            dc.b 0,0
up_thisrow_sprx:       dc.b 0  
up_current_xdraw:      dc.b 0  
up_underline:          dc.b 0  
up_drawpoi:            dc.b 0,0
up_draw_sprline:       dc.b 0  
up_tmp:                dc.b 0  
up_thischar_width:     dc.b 0  
up_charpoi:            dc.b 0,0

up_sincou:             dc.b 0
up_cou:                dc.b 0
up_cou2:                dc.b 0

up_20:
 ds.b $10,0 ;Byte 0-15 from the char
up_30:
 dc.b 0     ;#0
up_31:
 ds.b $10,0 ;Byte 16-31 from the car
up_41:
 dc.b 0     ;#0
up_42:
 ds.b $11,0 ;Byte 32-47 set to #0

up_pause: dc.b 0

; org $4000
;up_sprites:
; ds.b $1e00,0

 seg seg8000

;###
;###
;###

;This is the endscroller:
start_up:
  sei
  lda #$35
  sta $01
  lda #$7f
  sta $dc0d
  lda $dc0d
  jsr justplay_init
  ldx #$2e
.d000loop:
  lda up_d000defaults,x
  sta $d000,x
  dex
  bpl .d000loop
  inx
  stx up_irq_counter
  lda $dd00
  and #$fc
  ora #$02
  sta $dd00

  lda #0
  ldx #0
.looop:
  sta up_sprites+$0000,x
  sta up_sprites+$0100,x
  sta up_sprites+$0200,x
  sta up_sprites+$0300,x
  sta up_sprites+$0400,x
  sta up_sprites+$0500,x
  sta up_sprites+$0600,x
  sta up_sprites+$0700,x
  sta up_sprites+$0800,x
  sta up_sprites+$0900,x
  sta up_sprites+$0a00,x
  sta up_sprites+$0b00,x
  sta up_sprites+$0c00,x
  sta up_sprites+$0d00,x
  sta up_sprites+$0e00,x
  sta up_sprites+$0f00,x
  sta up_sprites+$1000,x
  sta up_sprites+$1100,x
  sta up_sprites+$1200,x
  sta up_sprites+$1300,x
  sta up_sprites+$1400,x
  sta up_sprites+$1500,x
  sta up_sprites+$1600,x
  sta up_sprites+$1700,x
  sta up_sprites+$1800,x
  sta up_sprites+$1900,x
  sta up_sprites+$1a00,x
  sta up_sprites+$1b00,x
  sta up_sprites+$1c00,x
  sta up_sprites+$1d00,x
  sta up_sprites+$1e00,x
  sta up_sprites+$1f00,x
  inx
  bne .looop

  lda #$00
  sta up_current_row
  lda #$14
  sta up_current_rowpixel
  sei
  lda #<up_irq1
  sta $fffe
  lda #>up_irq1
  sta $ffff
  lda #$fc   ;set nmi to rti
  sta $fffa
  lda #$ff
  sta $fffb
  lda #$40   ;rti
  sta $fffc
  lda #<up_text
  sta up_textpoi
  lda #>up_text
  sta up_textpoi+1

  lda #$40
  sta up_thisrow_sprx
  lda #$0
  sta up_current_xdraw
  sta up_underline
  sta up_sincou
  sta up_cou
  sta up_cou2
  sta up_pause
  jsr up_move

  lda #$01
  sta $d019
  cli

up_ever:
  lda up_current_row
  clc
  adc #$0e
  cmp #$0f
  bcc .go
  sbc #$0f
.go:
  sta up_draw_sprline
  asl
  adc #$40
  sta up_drawpoi+1
  lda #$00
  sta up_drawpoi
  lda up_thisrow_sprx
  ldx up_draw_sprline
  sta up_xpositions,x
  lda #$00
  sta up_current_xdraw
  sta up_underline
  tax
.clr:
  sta up_tmpsprites1,x
  sta up_tmpsprites2,x
  inx
  bne .clr
up_nextchar:
  ldy #$00
  lda (up_textpoi),y

;Check if the text is ended, #$ff
  cmp #$ff
  bne .no_end
  jmp up_end_part

.no_end:
;Check if the row is ended, #$fe
  cmp #$f8
  bne .no_pause
;  lda #$80
  sta up_pause
  inc up_textpoi
  bne .nowrpa
  inc up_textpoi+1
.nowrpa
  jmp up_nextchar
.no_pause:
  cmp #$fe
  bne .no_endrow
  ldy #$00
.lo98:
  lda up_tmpsprites1,y
  sta (up_drawpoi),y
  iny
  bne .lo98
  inc up_drawpoi+1
.lo99:
  lda up_tmpsprites2,y
  sta (up_drawpoi),y
  iny
  bne .lo99
  inc up_textpoi
  bne .nowre
  inc up_textpoi+1
.nowre:
  lda up_current_row
.wait_next_line:
  cmp up_current_row
  beq .wait_next_line
  jmp up_ever

;Check if #$f9, sets x-position
.no_endrow:
  cmp #$f9
  bne .no_setx
  iny
  lda (up_textpoi),y
  ldx up_draw_sprline
  sta up_thisrow_sprx
  sta up_xpositions,x
  lda #$02
  jsr up_inc_textpoi_with_a
  jmp up_nextchar

;Check if #$fa, underline on/off
.no_setx:
  cmp #$fa
  bne .no_underline
  lda up_underline
  eor #$80
  sta up_underline
  inc up_textpoi
  bne .nowr
  inc up_textpoi+1
.nowr:
  jmp up_nextchar

;Check if #$fb, sets xdraw
.no_underline:
  cmp #$fb
  bne .no_set_sprsx
  iny
  lda (up_textpoi),y
  sta up_current_xdraw
  lda #$02
  jsr up_inc_textpoi_with_a
  jmp up_nextchar

;Check if #$fd, right align text
.no_set_sprsx:
  cmp #$fd
  bne .no_rightmargin
  sty up_tmp
  iny
.nextchar:
  lda (up_textpoi),y
  iny
  tax
  bmi .nochar
  lda up_charwidths,x
  clc
  adc up_tmp
  sta up_tmp
.nochar:
  cpx #$fe
  bne .nextchar
  lda #$c0
  sec
  sbc up_tmp
  sta up_current_xdraw
  inc up_textpoi
  bne .nowr2
  inc up_textpoi+1
.nowr2:
  jmp up_nextchar

;Check if #$fc, center align text

.no_rightmargin:
  cmp #$fc
  bne .no_center
  sty up_tmp
  iny
.nextchar2:
  lda (up_textpoi),y
  iny
  tax
  bmi .nochar2
  lda up_charwidths,x
  clc
  adc up_tmp
  sta up_tmp
.nochar2:
  cpx #$fe
  bne .nextchar2
  lda #$c0
  sec
  sbc up_tmp
  lsr
  sta up_current_xdraw
  inc up_textpoi
  bne .nowr3
  inc up_textpoi+1
.nowr3:
  jmp up_nextchar

;Draw character:
.no_center:
  cmp #$60
  bcc .no_lower_case
  and #$1f
.no_lower_case:

  sta up_tmp
  tax
  lda up_charwidths,x
  sta up_thischar_width
  lda #$00
  sta up_charpoi
  txa
  lsr
  ror up_charpoi
  lsr
  ror up_charpoi
  lsr
  ror up_charpoi
  adc #>up_charset
  sta up_charpoi+1

  ldy #$0f
.lo1:
  lda (up_charpoi),y
  sta up_20,y
  dey
  bpl .lo1
  lda up_charpoi
  clc
  adc #$10
  sta up_charpoi

  ldy #$0f
.lo2:
  lda (up_charpoi),y
  sta up_31,y
  dey
  bpl .lo2

  ldy #$10
  lda #$00
.lo3:
  sta up_42,y
  dey
  bpl .lo3
  lda #$00
  sta up_30
  sta up_41

;Check if this should be underlined
  bit up_underline
  bpl .no_draw_underline
  ldx up_thischar_width
  lda up_underline1,x
  sta up_30
  lda up_underline2,x
  sta up_41

;Shift char sideways
.no_draw_underline:
  lda up_current_xdraw
  and #$07
  beq .noshift
  tay
.next_round:
  ldx #$10
.next_row:
  lsr up_20,x
  ror up_31,x
  ror up_42,x
  dex
  bpl .next_row
  dey
  bne .next_round

.noshift:
  lda up_current_xdraw
  lsr
  lsr
  lsr
  tax
  stx up_tmp
  lda up_pointers_lsb,x
  sta up_charpoi
  lda up_pointers_msb,x
  sta up_charpoi+1

;Copy char into sprite
  ldx #$00
  ldy #$00
.lo20:
  lda up_20,x
  ora (up_charpoi),y
  sta (up_charpoi),y
  inx
  iny
  iny
  iny
  cpx #$11
  bne .lo20

  ldx up_tmp
  inx
  cpx #$18
  beq up_finish_char

  lda up_pointers_lsb,x
  sta up_charpoi
  lda up_pointers_msb,x
  sta up_charpoi+1
  ldx #$00
  ldy #$00
.lo_31:
  lda up_31,x
  ora (up_charpoi),y
  sta (up_charpoi),y
  inx
  iny
  iny
  iny
  cpx #$11
  bne .lo_31

  ldx up_tmp
  inx
  inx
  cpx #$18
  beq up_finish_char
  lda up_pointers_lsb,x
  sta up_charpoi
  lda up_pointers_msb,x
  sta up_charpoi+1
  ldx #$00
  ldy #$00
.lo_42:
  lda up_42,x
  ora (up_charpoi),y
  sta (up_charpoi),y
  inx
  iny
  iny
  iny
  cpx #$11
  bne .lo_42

up_finish_char:
  lda up_thischar_width
  clc
  adc up_current_xdraw
  sta up_current_xdraw
  inc up_textpoi
  bne .nowr8
  inc up_textpoi+1
.nowr8:
  jmp up_nextchar

up_underline1:
 dc.b $00, $80, $c0, $e0, $f0, $f8, $fc, $fe
 dc.b $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

up_underline2:
 dc.b $00, $00, $00, $00, $00, $00, $00, $00
 dc.b $00, $80, $c0, $e0, $f0, $f8, $fc, $fe, $ff

;Pointer to sprite
up_pointers_lsb:
 dc.b $00,$01,$02,$40,$41,$42,$80,$81,$82,$c0,$c1,$c2
 dc.b $00,$01,$02,$40,$41,$42,$80,$81,$82,$c0,$c1,$c2

up_pointers_msb:
 ds.b 12,>up_tmpsprites1
 ds.b 12,>up_tmpsprites2


up_inc_textpoi_with_a:
  clc
  adc up_textpoi
  sta up_textpoi
  bcc .nowr9
  inc up_textpoi+1
.nowr9:
  rts

up_move:
  inc up_cou
  bne .hh
  inc up_cou+1
.hh
  lda up_cou2
  and #1
  bne .g1
  lda up_cou
  eor #$ff
  tax
  lda costbl,x
  cmp #$80
  ror
  sta up_sincou
.g1:
  rts

up_irq1:
  bit $dc0d
  bmi .noirq2
  jmp up_irq2
.noirq2:
  cli
  pha
  txa
  pha
  lda $d001
  clc
  adc #$15
  ldx up_sprites_ypos
  bne .sety
  cmp #$24
  bcc .sety
  sbc #$38
.sety:
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f
;set spritepointers
  lda up_current_displayrow
  asl
  asl
  asl
  tax
  stx $5ff8
  inx
  stx $5ff9
  inx
  stx $5ffa
  inx
  stx $5ffb
  inx
  stx $5ffc
  inx
  stx $5ffd
  inx
  stx $5ffe
  inx
  stx $5fff
;setx:
  ldx up_current_displayrow
  lda up_xpositions,x
  bne .use_this_x

  lda $d001
  clc
  adc up_sincou
  tax
  lda sintbl,x
  adc #$80
  lsr

.use_this_x:
  ldx #0
  clc
  adc #$18
  sta $d000
  adc #$18
  sta $d002
  adc #$18
  sta $d004
  adc #$18
  sta $d006
  adc #$18
  sta $d008
  adc #$18
  sta $d00a
  bcc .no_3msb
  ldx #$e0
  stx $d010
  clc
.no_3msb:
  adc #$18
  sta $d00c
  bcc .no_2msb
  ldx #$c0
  stx $d010
  clc
.no_2msb:
  adc #$18
  sta $d00e
  bcc .no_1msb
  ldx #$80
.no_1msb:
  stx $d010

;next:
  ldx up_current_displayrow
  inx
  cpx #$f
  bne .nowra
  ldx #0
.nowra:
  stx up_current_displayrow
  ldx up_sprites_ypos
  inx
  stx up_sprites_ypos
  cpx #$0d
  bne .no_new_row

  lda #$7f
  sta $dc0d
  lda $dc0d

  ldx up_pause
  beq .up_running
  dex
  stx up_pause
  jmp .no_new_row

.up_running:

  ldx up_current_rowpixel
  dex
  stx up_current_rowpixel
  bpl .no_new_row
  ldx #$14
  stx up_current_rowpixel
  ldx up_current_row
  inx
  cpx #$0f
  bne .nowrb
  ldx #0
.nowrb:
  stx up_current_row

.no_new_row:
  lda up_sprites_ypos
  cmp #$01
  bne .no_music
.music:
  tya
  pha
  jsr up_move
  jsr music_play
  pla
  tay
.no_music:
  pla
  tax
  pla
  rti

up_irq2:
  pha
  lda up_irq_counter
  bne up_irq3
  lda #$80
  sta $d011
  lda #$1f
  sta $d012
  lda #$01
  sta up_irq_counter
  sta $d019
  pla
  rti

up_irq3:
  cmp #$01
  bne up_irq4
  lda #$26
  sta $d012
  inc up_irq_counter
  lda #$01
  sta $d019
  nop
  nop
  nop
  lda #$ff
  sta $7fff
  pla
  rti

up_irq4:
  cmp #$02
  beq .up_irq6
  jmp up_irq5

.up_irq6:
  lda #$08
  sta $d011
  lda up_current_rowpixel
  sec
  sbc #$06
  bcs .h
  adc #$38
.h:
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f
  txa
  pha
  lda up_current_row
  asl
  asl
  asl
  tax
  stx $5ff8
  inx
  stx $5ff9
  inx
  stx $5ffa
  inx
  stx $5ffb
  inx
  stx $5ffc
  inx
  stx $5ffd
  inx
  stx $5ffe
  inx
  stx $5fff

  ldx up_current_row
  lda up_xpositions,x
  bne .use_this_x2

  lda $d001
  clc
  adc up_sincou
  tax
  lda sintbl,x
  adc #$80
  lsr

.use_this_x2:
;  ldx up_current_row
;  lda up_xpositions,x
  ldx #$00
  clc
  adc #$18
  sta $d000
  adc #$18
  sta $d002
  adc #$18
  sta $d004
  adc #$18
  sta $d006
  adc #$18
  sta $d008
  adc #$18
  sta $d00a
  bcc .noo_3msb
  ldx #$e0
  clc
.noo_3msb:
  adc #$18
  sta $d00c
  bcc .noo_2msb
  ldx #$c0
  clc
.noo_2msb:
  adc #$18
  sta $d00e
  bcc .noo_1msb
  ldx #$80
.noo_1msb:
  stx $d010
  lda up_current_rowpixel
  sec
  sbc #$0b
  bcs .k
  adc #$38
.k:
  ldx #$2a
  stx $dc04
  ldx #$05
  stx $dc05
  ldx #$91
.wait_raster:
  cmp $d012
  bne .wait_raster
  stx $dc0e

  lda #$81
  sta $dc0d
  lda $dc0d
  lda #$00
  sta up_sprites_ypos
  ldx up_current_row
  inx
  cpx #$0f
  bne .nowrc
  ldx #$00
.nowrc:
  stx up_current_displayrow
  lda #$0e
  sta $d012
  inc up_irq_counter
  lda #$01
  sta $d019
  pla
  tax
  pla
  rti

up_irq5:
  lda #$f8
  sta $d012
  lda #$00
  sta up_irq_counter
  lda #$01
  sta $d019
  lda #$00
  sta $7fff
  pla
  rti

;###
;###
;###

up_end_part:
  lda #0
  sta $dc0e ;stop timers
  lda #$7f
  sta $dc0d
  lda $dc0d
  jmp next_demo_part





;###
;### fade
;###

 subroutine

;This is the only "known bug" in this demo.
;At the start of this part, the lower part of the screen
;goes black between the newly opened lower border and
;the normal sideborders.
;I tried to remove it, but I didn't have time to
;fix it properly. So, it will stay as it is, and
;if you have a suggestion of how to fix it, please
;tell me!

;This routine is changing the last byte of the bank
;for the "inner" part of the screen, and then with
;four "unlimited enlarged i y-direction" sprites,
;the text in the border is displayed. These sprites
;change contents by changing their sprite pointers
;to the 256 different calculated sprites in memory.
start_fade:
  sei
  lda #$35
  sta $01
  lda #$0e
  sta $d021
  sta $d020
;Set bank
  lda #$0b
  sta $d011
  lda #0
  sta $7fff
  sta $47fc
  sta $47fd
  sta $47fe
  sta $47ff
  lda #$15
  sta $d018
  lda #$02
  sta $dd00

  lda #$d0
  sta $d010
  lda #$00
  sta $d015
  lda #$c8
  sta $d016
  sta $d017
  lda #0
  sta $d01d

  jsr justplay_init

  ldx #0
  lda #$e
.clrsc:
  sta $d800,x
  sta $d900,x
  sta $da00,x
  sta $daf7,x
  inx
  bne .clrsc

  ldx #$07
.loo:
  lda fade_sprs,x
  sta $d008,x
  dex
  bpl .loo

;Set sprite colours
  ldx #$03
  lda #$00
.loo2:
  sta $d02b,x
  dex
  bpl .loo2

  lda #0
  tax
.loa:
  sta fade_screen_tmp-4,x
  sta fade_screen_tmp+$fc,x
  inx
  bne .loa

  lda #$1b
  sta $d011


;Precalc all sprites for all possible values:
  lda #$40
  sta .dst+2
  lda #0
  sta .dst+1

  lda #>fade_image
  sta .move+2
  lda #<fade_image
  sta .move+1

  ldx #$00
.next_spr:
  txa
  ldy #$3f
.dst:
  sta $4000,y
  dey
  bpl .dst
  lda .dst+1
  clc
  adc #$40
  bcc .nowra
  inc .dst+2
.nowra:
  sta .dst+1
  inx
  bne .next_spr

  sei
  lda #$f9
.wait_raster:
  cmp $d012
  bne .wait_raster

  lda #$13
  sta $d011
  lda #$30
.wait_raster2:
  cmp $d012
  bne .wait_raster2


;.wait_raster3:
;  lda $d012
;  ora $d011
;  bmi .wait_raster3
  lda #$08
  sta $d011
  lda #$f0
  sta $d015
;  lda #$0e
;  sta $d021
;  sta $d020
  jmp start_fade_loop

;fade_sprs: dc.b $e0,$07,$00,$07,$58,$07,$70,$07
fade_sprs: dc.b $e0,$0f,$00,$0f,$58,$0f,$70,$0f

 org $9300
start_fade_loop:
;  ldy #$02
  ldy #$0a
.w1:
  cpy $d012
  bne .w1
  ldx #$09
.w2:
  dex
  bne .w2
  cmp ($00,x)
  iny
  cpy $d012
  beq .w3
  cmp $00,x
.w3:
  ldx #$09
.w4:
  dex
  bne .w4
  cmp ($00,x)
  iny
  cpy $d012
  beq .w5
  bit $00
.w5:
  ldx #$0a
.w6:
  dex
  bne .w6
  nop
  iny
  cpy $d012
  bne .w7
.w7:
  ldx #$0b
.w8:
  dex
  bne .w8
  cmp $00,x

  ldy #$00
;  ldx #$0e
  ldx #$0c+8
  lda #$00
.loop1:
  sta fade_screen_tmp-3,x
  sty $d016
  sta $7fff
  lda #$f8
  sta $d016
  sty $d017
  sta $d017
  lda fade_screen_tmp,x
  sta $47fc
  sta $47fd
  sta $47fe
  sta $47ff
  inx
  bne .loop1

  sta fade_screen_tmp+$00fd
.loop2:
  nop
  sty $d016
  sta $7fff
  lda #$f8
  sta $d016
  lda fade_screen_tmp+$0100,x
  sta $47fc
  sta $47fd
  sta $47fe
  sta $47ff
  sty $d011
  sta fade_screen_tmp+$00fe,x
  inx
;  cpx #$26
  cpx #$26
  bne .loop2

  cmp $00
.loop3:
  sty $d016
  sta $7fff
  lda #$f8
  sta $d016
  sty $d017
  sta $d017
  lda fade_screen_tmp+$0100,x
  sta $47fc
  sta $47fd
  sta $47fe
  sta $47ff
  cmp $00
  inx
;  cpx #$35
;  cpx #$27
  cpx #$2a
  bne .loop3

;  ldx #$12
  ldx #$8
.move:
  lda fade_image,x
  sta fade_screen_tmp+$0124,x
  dex
  bpl .move
  lda #$08
  sta $d011

  ldx .move+2
  lda .move+1
  clc
  adc #$02
  bcc .nowr
  inx
  stx .move+2
.nowr:
  sta .move+1

  cmp #<(fade_image + $2a0)
  bne .noend
  cpx #>(fade_image + $2a0)
  beq .end_fade
.noend:
  jsr music_play

  jmp start_fade_loop

.end_fade:
  lda #0
  sta $d020
  sta $d021
  jmp next_demo_part

; org $0e00

 org $93fc
 ds.b 4,0
fade_screen_tmp:
 ds.b $1fc,0

fade_image:
 dc.b %00111000
 dc.b %01000111
 dc.b %11100000
 dc.b %00011000
 dc.b %00000100
 dc.b %00000010
 dc.b %00000001
 dc.b %10000000
 dc.b %01000000
 dc.b %01000000
 dc.b %00100000
 dc.b %00100000
 dc.b %00100000
 dc.b %00010000
 dc.b %00010000
 dc.b %00010000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10111011
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %00101000
 dc.b %00101000
 dc.b %00101000
 dc.b %00111000
 dc.b %00010000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10101000
 dc.b %10101000
 dc.b %10001000
 dc.b %10001000
 dc.b %00000000
 dc.b %10001000
 dc.b %11011000
 dc.b %01110000
 dc.b %00100000
 dc.b %01110000
 dc.b %11011000
 dc.b %10001000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %11111000
 dc.b %11110000
 dc.b %00100000
 dc.b %01000000
 dc.b %00100000
 dc.b %11110000
 dc.b %11111000
 dc.b %00000000
 dc.b %11110000
 dc.b %11111000
 dc.b %00101000
 dc.b %00101000
 dc.b %00101000
 dc.b %11111000
 dc.b %11110000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %00100000
 dc.b %00100000
 dc.b %00100000
 dc.b %11111000
 dc.b %11111000
 dc.b %00000000
 dc.b %01110000
 dc.b %11111000
 dc.b %10001000
 dc.b %10001000
 dc.b %10001000
 dc.b %11111000
 dc.b %01110000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %00010000
 dc.b %00100000
 dc.b %01000000
 dc.b %11111000
 dc.b %11111000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10101000
 dc.b %10101000
 dc.b %10001000
 dc.b %10001000
 dc.b %00000000
 dc.b %00011000
 dc.b %00111000
 dc.b %11100000
 dc.b %11100000
 dc.b %00111000
 dc.b %00011000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %00001000
 dc.b %00001000
 dc.b %11111000
 dc.b 111111000
 dc.b %00001000
 dc.b %00001000
 dc.b %00000000
 dc.b %01111000
 dc.b %11111000
 dc.b %10000000
 dc.b %10000000
 dc.b %10000000
 dc.b %11111000
 dc.b %01111000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %00101000
 dc.b %00101000
 dc.b %00001000
 dc.b %00001000
 dc.b %00000000
 dc.b %00011000
 dc.b %00111000
 dc.b %01100000
 dc.b %11000000
 dc.b %01100000
 dc.b %00111000
 dc.b %00011000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10101000
 dc.b %10101000
 dc.b %10001000
 dc.b %10001000
 dc.b %00000000
 dc.b %10010000
 dc.b %10111000
 dc.b %10101000
 dc.b %10101000
 dc.b %11101000
 dc.b %01001000
 dc.b %00000000
 dc.b %10010000
 dc.b %10111000
 dc.b %10101000
 dc.b %10101000
 dc.b %11101000
 dc.b %01001000
 dc.b %00000000
 dc.b %01110000
 dc.b %11111000
 dc.b %10001000
 dc.b %10001000
 dc.b %10001000
 dc.b %11111000
 dc.b %01110000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %00010000
 dc.b %00100000
 dc.b %01000000
 dc.b %11111000
 dc.b %11111000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10101000
 dc.b %10101000
 dc.b %10101000
 dc.b %11111000
 dc.b %01010000
 dc.b %00000000
 dc.b %10001000
 dc.b %11111000
 dc.b %11111000
 dc.b %10001000
 dc.b %00000000
 dc.b %00001000
 dc.b %00001000
 dc.b %11111000
 dc.b %11111000
 dc.b %00001000
 dc.b %00001000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10000000
 dc.b %10000000
 dc.b %10000000
 dc.b %10000000
 dc.b %00000000
 dc.b %10001000
 dc.b %11111000
 dc.b %11111000
 dc.b %10001000
 dc.b %00000000
 dc.b %00011000
 dc.b %00111000
 dc.b %01100000
 dc.b %11000000
 dc.b %01100000
 dc.b %00111000
 dc.b %00011000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10101000
 dc.b %10101000
 dc.b %10001000
 dc.b %10001000
 dc.b %00000000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %00111000
 dc.b %00100000
 dc.b %00100000
 dc.b %00100000
 dc.b %11111000
 dc.b %11111000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10001000
 dc.b %10001000
 dc.b %10001000
 dc.b %11111000
 dc.b %01110000
 dc.b %00000000
 dc.b %11111000
 dc.b %11111000
 dc.b %10101000
 dc.b %10101000
 dc.b %10001000
 dc.b %10001000
 dc.b %00000000
 dc.b %11111000
 dc.b %11110000
 dc.b %00100000
 dc.b %01000000
 dc.b %00100000
 dc.b %11110000
 dc.b %11111000
 dc.b %00000000
 dc.b %01110000
 dc.b %11111000
 dc.b %10001000
 dc.b %10001000
 dc.b %10001000
 dc.b %11111000
 dc.b %01110000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00000000
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10000011
 dc.b %00111000
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10111011
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10111011
 dc.b %01111100
 dc.b %11101110
 dc.b %11000111
 dc.b %10111011
 dc.b %01111100
 dc.b %11111110
 dc.b %11111110
 dc.b %11111110
 dc.b %11111110
 dc.b %11111101
 dc.b %11111101
 dc.b %11111101
 dc.b %11111011
 dc.b %11111011
 dc.b %11110111
 dc.b %11101111
 dc.b %10011111
 dc.b %01111110
 dc.b %11110001
 dc.b %00001110
 ds.b 314,%11111111

;###
;###
;###

;72a0-72af x-position fr raderna
up_xpositions:
 dc.b $40,$40,$40,$40,$40,$40,$40,$40
 dc.b $40,$40,$40,$40,$40,$40,$40

;72af-72de defaultvrden fr $d000-$d02e
up_d000defaults:
 dc.b $00, $00, $00, $00, $00, $00, $00, $00
 dc.b $00, $00, $00, $00, $00, $00, $00, $00
 dc.b $00, $7b, $f8, $00, $00, $ff, $08, $00
 dc.b $70, $01, $01, $ff, $00, $00, $00, $00
 dc.b $00, $00, $00, $00, $00, $00, $00, $01
 dc.b $01, $01, $01, $01, $01, $01, $01



;###
;### textdisplay border-routine
;###
 org $9900
irq_textdisplay_border_raster = $45
irq_textdisplay_border: subroutine
  sta savea
  stx savex
  sty savey
  tsx           ; 2
  lda #$00
  sta $d017
  lda #$18  ;spr data at $07f8
  lda $d018

  nop
  bit $24       ; 5

  lda #<irq_textdisplay_bordera
  sta $fffe
  lda #>irq_textdisplay_bordera
  sta $ffff

  lda #$c1         ; waste at least 12 cycles
  lda $d010      ; (up to 64 cycles delay allowed here)
  lda #%11111111
  sta $d015
  inc $d012     ; At this stage, $d012 has already been incremented by one.
  lda #1
  sta $d019     ; acknowledge the first raster interrupt
  cli           ; enable interrupts (the second interrupt can now occur)


;Total cycles = 2+5*9-1 = 46 cycles
;  ldy #9  ;#2
;  dey     ;#2
;  bne *-1 ;#3 if succeeds      ; delay

;cycles = 2+8*4 = 34
  lda #$01  ;#2
  lda $d028 ;spr #1 colour
  lda $d029 ;spr #2 colour
  lda $d02a ;spr #3 colour
  lda $d02b ;spr #4 colour
  lda $d02c ;spr #5 colour
  lda $d02d ;spr #6 colour
  lda #1
  lda $d027 ;#4 ;spr #0 colour
  lda $d02e ;spr #7 colour
  nop
  nop
  nop
  nop
  nop


  nop           ; The second interrupt will occur while executing these
  nop           ; two-cycle instructions.
  nop
  nop
  nop
  nop

.error:
;  inc $d020
  jmp .error


irq_textdisplay_bordera:
;Now, get the bytes away from the stack that we won't use
  pla           ; 4
  nop           ; 2
  pla           ; 4
  nop           ; 2
  pla           ; 4
  nop           ; 2
  nop
  nop           ; 4
  nop           ; 2
  nop
  bit $24       ; 5


  lda #<irq_spr1
  lda $fffe
  lda #>irq_spr1
  lda $ffff
  ldx $d012
  nop
  bit $24       ; 6569, 63 cycles/line
  cpx $d012     ; The comparison cycle is executed CYCLES or CYCLES+1 cycles
                ; after the interrupt has occurred.
  beq *+2       ; Delay by one cycle if $d012 hadn't changed.
                ; Now exactly CYCLES+3 cycles have passed since the interrupt.


  ldx #16
.l:
  dex
  bne .l

  nop
  ldx #1
  stx $d019     ; acknowledge the raster interrupt
  nop
  nop
  lda #14
  lda $d020
  lda #5
  sta rows
  ldx #$15 ;First row shall not be opened
  nop
.loop1:
  nop
  dec $d016
  inc $d016

  nop
  nop
  lda $d011
  clc
  adc #5
  and #$07
  ora #$18
  sta $d011
  nop
  nop
  bit $24
  dex
  bne .loop1

;  lda #0
;  sta $d020

  ldx textdisplay_rownr
  cpx #2
  beq .textdisplay_endirq
  inx
  stx textdisplay_rownr
  jsr textdisplay_set
  lda #<irq_textdisplay_border    ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_textdisplay_border
  sta $ffff
  lda #$1b
  sta $d011
  lda savea
  ldx savex
  ldy savey
  rti

.textdisplay_endirq:
  lda #%00011011
  sta $d011
  lda #<irq_text2                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_text2
  sta $ffff
  lda #irq_text2_raster
  sta $d012
;  lda #%00000011
;  sta $dd00
  lda savea
  ldx savex
  ldy savey
  rti


textdisplay_ypos:
  dc.b $19,$41,$69

textdisplay_xpos_lsb_spd:
  dc.b $1,$1,$1

textdisplay_xpos_lsb:
  dc.b $b4,$b4,$04
textdisplay_xpos_msb:
  dc.b $00,$00,$01

textdisplay_xpos_cou:
  dc.b $00

;This isn't used for anything...
textdisplay_7fff:
  dc.b %11111111
  dc.b %11111111
  dc.b %11111110
  dc.b %11111110
  dc.b %11111100
  dc.b %11111100
  dc.b %11111000
  dc.b %11111000
  dc.b %11110000
  dc.b %11110000
  dc.b %11100000
  dc.b %11100000
  dc.b %11000000
  dc.b %11000000
  dc.b %10000000
  dc.b %10000000
  dc.b %00000000

textdisplay_text:
  dc.b #"T" - $40 + charset_start
  dc.b #"I" - $40 + charset_start
  dc.b #"B" - $40 + charset_start
  dc.b #"H" - $40 + charset_start
  dc.b #"S" - $40 + charset_start
  dc.b #"Y" - $40 + charset_start
  dc.b #"I" - $40 + charset_start
  dc.b #"C" - $40 + charset_start
  dc.b #charset_start
  dc.b #"S" - $40 + charset_start
  dc.b #"O" - $40 + charset_start
  dc.b #charset_start
  dc.b #"D" - $40 + charset_start
  dc.b #"D" - $40 + charset_start
  dc.b #charset_start
  dc.b #"E" - $40 + charset_start
  dc.b #"E" - $40 + charset_start
  dc.b #charset_start
  dc.b #"M" - $40 + charset_start
  dc.b #"D" - $40 + charset_start
  dc.b #charset_start
  dc.b #"O" - $40 + charset_start
  dc.b #charset_start
  dc.b #charset_start

textdisplay_x:
  dc.b $f3,$f3,$00
  dc.b $13,$0f,$00
  dc.b $26,$40,$f3
  dc.b $38,$58,$f3
  dc.b $68,$70,$f3
  dc.b $81,$88,$f3
  dc.b $9a,$a0,$f3
  dc.b $b3,$b8,$f3

textdisplay_rownr: dc.b 0

textdisplay_set:
  lda textdisplay_ypos,x
  sta $d001
  sta $d003
  sta $d005
  sta $d007
  sta $d009
  sta $d00b
  sta $d00d
  sta $d00f
  sec
  sbc #$4
  sta $d012

  lda #0
  sta $d017
  sta $d01d

  lda #1
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

  lda textdisplay_text,x
  sta $07f8  
  lda textdisplay_text+3,x
  sta $07f9  
  lda textdisplay_text+6,x
  sta $07fa  
  lda textdisplay_text+9,x
  sta $07fb  
  lda textdisplay_text+12,x
  sta $07fc  
  lda textdisplay_text+15,x
  sta $07fd  
  lda textdisplay_text+18,x
  sta $07fe  
  lda textdisplay_text+21,x
  sta $07ff  

  lda #$ff
  sta $d015

  lda textdisplay_x,x
  bne .normalx
  inc textdisplay_xpos_cou
  ldx textdisplay_xpos_cou
  lda sintbl,x
  bpl .oo
  eor #$ff
.oo:
  sta $d000
  clc
  adc #$19
  sta $d002
  lda #%00000000
  sta $d010
  rts
.normalx:
  sta $d000
  lda textdisplay_x+3,x
  sta $d002
  lda textdisplay_x+6,x
  sta $d004
  lda textdisplay_x+9,x
  sta $d006
  lda textdisplay_x+12,x
  sta $d008
  lda textdisplay_x+15,x
  sta $d00a
  lda textdisplay_x+18,x
  sta $d00c
  lda textdisplay_x+21,x
  sta $d00e

  lda #%00000001
  sta $d010
  rts

;###
;### image_scroller
;###

 seg zp
image_scroller_poi:
  dc.b 0,0
image_scroller_x:
  dc.b 0
image_scroller_nextspr:
  dc.b 0
 seg seg8000

image_scroller_init:
  lda #>image_scroller_text
  sta image_scroller_poi+1
  lda #<image_scroller_text
  sta image_scroller_poi
  lda #0
  sta image_scroller_x

  ldx #$07
  lda #charset_start
.cllr:
  sta $07f8,x
  dex
  bpl .cllr

  jmp precalc_imagescroll_ydist

image_scroller: subroutine
  lda image_scroller_x
  sec
image_scroll_spd = *+1
  sbc #$04
  bcc .wrap
  sta image_scroller_x
  rts
.wrap:
  clc
  adc #$34
  sta image_scroller_x
  ldx #$00
.move:
  lda $07f9,x
  sta $07f8,x
  inx
  cpx #$07
  bne .move
;  lda image_scroller_nextspr
;  sta $07fe

.another:
  ldy #$08
  lda (image_scroller_poi),y
  bne .no_wrap

;  lda #>image_scroller_text
;  sta image_scroller_poi+1
;  lda #<image_scroller_text
;  sta image_scroller_poi
  lda #1
  sta part_finished
  lda #' 
;  jmp .another

.no_wrap:
  cmp #' 
  bne .nospace
  lda #0
.nospace:
  and #$3f
  clc
  adc #charset_start
;  sta image_scroller_nextspr
  sta $07ff

  inc image_scroller_poi
  bne .nowrr
  inc image_scroller_poi+1
.nowrr
  rts


image_scroller_setx:
  lda image_scroller_x
  lsr
  lsr
  tax
  lda imagescroll_xpostable+0,x
  sta image_scroll_xposes
  lda imagescroll_xpostable+$d,x
  sta image_scroll_xposes+1
  lda imagescroll_xpostable+$1a,x
  sta image_scroll_xposes+2
  lda imagescroll_xpostable+$27,x
  sta image_scroll_xposes+3
  lda imagescroll_xpostable+$34,x
  sta image_scroll_xposes+4
  lda imagescroll_xpostable+$41,x
  sta image_scroll_xposes+5
  lda imagescroll_xpostable+$4e,x
  sta image_scroll_xposes+6
  lda imagescroll_xpostable+$5b,x
  sta image_scroll_xposes+7
  lda imagescroll_xpostable+$68,x
  sta image_scroll_xposes+8

  lda image_scroller_x
  lsr
  lsr
  ;Now, a is 0 - $c (13 different x-positions)
  clc
  adc #>imagescroll_ydist
  sta ydistpoi+1
  rts

image_scroll_xposes:
  ds.b 9,0



 seg zp
precalc_poi:
  dc.b 0,0
precalc_tmp:
  dc.b 0
precalc_round:
  dc.b 0
precalc_zeroes:
  dc.b 0
 seg seg8000

imagescroll_ydist = $7200

precalc_imagescroll_ydist: subroutine
  ldx #0
  lda #$ff
.clear:
  sta imagescroll_ydist+$000,x
  sta imagescroll_ydist+$100,x
  sta imagescroll_ydist+$200,x
  sta imagescroll_ydist+$300,x
  sta imagescroll_ydist+$400,x
  sta imagescroll_ydist+$500,x
  sta imagescroll_ydist+$600,x
  sta imagescroll_ydist+$700,x
  sta imagescroll_ydist+$800,x
  sta imagescroll_ydist+$900,x
  sta imagescroll_ydist+$a00,x
  sta imagescroll_ydist+$b00,x
  sta imagescroll_ydist+$c00,x
  inx
  bne .clear

  ldy #0
  sty precalc_round
  lda #$fe
  sta .bitmask

.next_bitmask:
  lda #>imagescroll_ydist
  sta precalc_poi+1
  lda #<imagescroll_ydist
  sta precalc_poi

.more2:
  lda precalc_round
  tax
  ldy precalc_ypos,x
;  lda precalc_xscale,x
;  tax
  sec
  sbc #$34
  bmi .now
  eor #$ff
.now:
  asl
  asl
;  sec
;  sbc #$10
  tax


;  ldy #1
  sta precalc_tmp
  lda #20
  sta precalc_zeroes

.more:
  txa
  clc
  adc precalc_tmp
  sta precalc_tmp
  lda #0
  adc #0
  beq .no_carry
  lda #$0
  dec precalc_zeroes
  bpl .apply
  lda #$ff
  jmp .apply
.no_carry:
  lda #$ff

.apply:
;  eor #$ff
.bitmask = *+1
  ora #$fe
  and (precalc_poi),y
  sta (precalc_poi),y

  iny
  cpy #$60
  bne .more

  inc precalc_round
  inc precalc_poi+1
  lda precalc_poi+1
  cmp #>imagescroll_ydist + $d
  bne .more2

  sec
  rol .bitmask
  lda precalc_round
  cmp #$68
  bne .next_bitmask
  rts

precalc_ypos:
  dc.b $00,$00,$00,$00,$00,$00,$00,$00
  dc.b $00,$00,$00,$00,$00,$00,$00,$00
  dc.b $00,$01,$01,$01,$01,$02,$02,$03
  dc.b $03,$04,$04,$05,$06,$07,$08,$09
  dc.b $09,$0a,$0b,$0c,$0d,$0e,$10,$11
  dc.b $13,$15,$17,$19,$1b,$1d,$1f,$21
  dc.b $23,$25,$28,$26,$24,$22,$21,$1f
  dc.b $1d,$1c,$1a,$18,$17,$15,$14,$13
  dc.b $12,$11,$10,$0f,$0e,$0d,$0c,$0b
  dc.b $0a,$09,$08,$07,$06,$05,$04,$03
  dc.b $03,$03,$02,$02,$02,$01,$01,$01
  dc.b $01,$00,$00,$00,$00,$00,$00,$00
  dc.b $00,$00,$00,$00,$00,$00,$00,$00
  dc.b $00,$00,$00,$00,$00,$00,$00,$00

 org $9d00
image_d011:
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$14,$15,$16,$17  ;to open the lower border
  dc.b $10,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f
  dc.b $18,$19,$1a,$1b,$1c,$1d,$1e,$1f

imagescroll_xpostable:
  dc.b $c4,$cc,$d3,$da,$e1,$e7,$ee,$f4,$03,$09,$0f,$15,$1b ;spr0
  dc.b $21,$27,$2d,$33,$39,$3f,$45,$4a,$4f,$54,$59,$5e,$63 ;spr1
  dc.b $68,$6c,$70,$74,$78,$7c,$80,$84,$88,$8b,$8f,$93,$96 ;spr2
  dc.b $98,$9b,$9d,$a0,$a2,$a4,$a6,$a8,$aa,$ac,$ae,$b0,$b2 ;spr3
  dc.b $b4,$b6,$b8,$ba,$bc,$be,$c0,$c2,$c4,$c6,$c9,$cb,$ce ;spr4
  dc.b $d0,$d3,$d6,$d9,$dc,$df,$e2,$e5,$e8,$eb,$ee,$f1,$f4 ;spr5
  dc.b $f7,$fa,$fd,$01,$05,$09,$0d,$12,$16,$1a,$1e,$22,$26 ;spr6
  dc.b $2b,$33,$38,$3d,$41,$46,$4b,$51,$56,$5c,$62,$68,$6e ;spr7
  dc.b $81,$81,$81,$c1,$c1,$c1,$c1,$c1,$c0,$c0,$c0,$c0,$c0 ;d010


;###
;### up-text
;###

up_text:
 dc.b $f9,$40,$fc,"You have been watching",$fe
 dc.b $fc,"the Bit-Live 4 demo",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $fc,"Released on the",$fe
 dc.b $fc,"13th of September 2003",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,1,$fa,"Coding by:",$fa,$fe
 dc.b $f9,$80,$fd,"Pex Mahoney Tufvesson",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,1,$fa,"Music:",$fa,$fe
 dc.b $f9,$80,$fd,"Spellbound",$fe
 dc.b $fd,"by Rob Hubbard",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,1,$fa,"This charset by:",$fa,$fe
 dc.b $f9,$80,$fd,"Shaman of Taboo",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,1,$fa,"Press play on tape:",$fa,$fe
 dc.b $f9,$1,"Keyboard:",$fe
 dc.b $f9,$80,$fd,"Andre Tischer Poulsen",$fe
 dc.b $f9,$1,"Bass:",$fe
 dc.b $f9,$80,$fd,"Uffe Friis Lichtenberg",$fe
 dc.b $f9,$1,"Guitar:",$fe
 dc.b $f9,$80,$fd,"Jesper Holm Olsen",$fe
 dc.b $f9,$1,"Guitar:",$fe
 dc.b $f9,$80,$fd,"Martin Koch",$fe
 dc.b $f9,$1,"Keyboard:",$fe
 dc.b $f9,$80,$fd,"Theo Engell-Nielsen",$fe
 dc.b $f9,$1,"Drums:",$fe
 dc.b $f9,$80,$fd,"S","z"+2,"ren Trautner Madsen",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,1,$fa,"Inspiration from:",$fa,$fe
 dc.b $f9,$80,$fd,$fa,"Stable Raster IRQ",$fe
 dc.b $fd,"by Fungus 1996",$fe
 dc.b " ",$fe
 dc.b $f9,$80,$fd,$fa,"Sprite multiplexing",$fe
 dc.b $fd,"by Lasse ","z"+4,"z"+4,"rni",$fe
 dc.b " ",$fe
 dc.b $f9,$80,$fd,$fa,"Altered States",$fe
 dc.b $fd,"by Taboo 1994",$fe
 dc.b $fd,"Endscroll code by MMS",$fe
 dc.b " ",$fe

 dc.b $f9,1,$fa,"Tools used:",$fa,$fe
 dc.b $f9,$80,$fd,$fa,"CodeWright 6.0b",$fe
 dc.b $fd,"text editor for win32",$fe
 dc.b " ",$fe
 dc.b $fd,$fa,"Photoshop 7.0",$fe
 dc.b $fd,"graphics for win32",$fe
 dc.b " ",$fe
 dc.b $fd,$fa,"PuCrunch 8.3.2002",$fe
 dc.b $fd,"by Pasi Ojala",$fe
 dc.b $fd,"Packer for win32",$fe
 dc.b " ",$fe
 dc.b $fd,$fa,"XRay64 v0.1 beta3",$fe
 dc.b $fd,"by Jakub Prezes",$fe
 dc.b $fd,"Wozniakowski",$fe
 dc.b $fd,"Graphics conv for win32",$fe
 dc.b " ",$fe
 dc.b $fd,$fa,"Dasm v2.12.04",$fe
 dc.b $fd,"by Mattew Dillon",$fe
 dc.b $fd,"and Olaf 'Rhialto' Seibert",$fe
 dc.b " ",$fe
 dc.b $fd,$fa,"WinVice v1.12",$fe
 dc.b $fd,"Commodore64 emulator",$fe
 dc.b " ",$fe

 dc.b $fd,$fa,"CCS64",$fe
 dc.b $fd,"by Per H","z"+5,"kan Sundell",$fe
 dc.b $fd,"Commodore64 emulator",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe

 dc.b $f9,$40,$fc,"Thanks to you all",$fe
 dc.b $fc,"for making these",$fe
 dc.b $fc,"wonderful tools.",$fe
 dc.b " ",$fe
 dc.b $fc,"Without them, this demo",$fe
 dc.b $fc,"would never been done!",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe


 dc.b $f9,0,$fa,$fc,"Greetings to:",$fa,$fe
 dc.b " ",$fe
 dc.b $f9,$40,$fc,"(analphabetic order)",$fe
 dc.b " ",$fe
 dc.b $f9,0,$fc,"Melina Ejeson",$fe
 dc.b $fc,"John 'Ziphoid' Carehag",$fe
 dc.b $fc,"Carsten 'O2' Ohlsson",$fe
 dc.b $fc,"Peter 'Yodelking' Sand&n",$fe
 dc.b $fc,"Marcus 'Makke' Nilsson",$fe
 dc.b $fc,"Slaygon/Censor",$fe
 dc.b $fc,"Gluemaster",$fe
 dc.b $fc,"Jan Lund Thomsen",$fe
 dc.b $fc,"Anders Berkeman",$fe
 dc.b $fc,"Karin ","z"+4,"jehagen",$fe
 dc.b $fc,"Sofia ","z"+5,"berg",$fe
 dc.b $fc,"Elsa Persson",$fe
 dc.b $fc,"Hanna Kappelin",$fe
 dc.b $fc,"Johannes Linden",$fe
 dc.b $fc,"Kent 'Trace' Walden",$fe
 dc.b $fc,"Rauli Sulanko",$fe
 dc.b $fc,"Holger Lagerfeldt",$fe
 dc.b $fc,"Glyn R. Brown",$fe
 dc.b $fc,"Rob Hubbard",$fe
 dc.b $fc,"Neil 'Tas' Carr",$fe
 dc.b $fc,"Markus 'LMan' Klein",$fe
 dc.b $fc,"Stefan Buhrstr","z"+4,"m",$fe
 dc.b $fc,"Stello Doussis",$fe
 dc.b $fc,"Anders Dellson",$fe
 dc.b $fc,"Ben Daglish",$fe
 dc.b $fc,"Deck/Scoopex",$fe
 dc.b $fc,"Thomas Detert",$fe
 dc.b $fc,"Marcus Nelson",$fe
 dc.b $fc,"LaLa",$fe
 dc.b $fc,"Wobbler",$fe
 dc.b $fc,"Jesper Johansson",$fe
 dc.b $fc,"Chris Abbott",$fe
 dc.b $fc,"TMB/Creators",$fe
 dc.b $fc,"Michael 'DjLizard' Cooke",$fe
 dc.b $fc,"Stefan 'Zingo' Andersen",$fe
 dc.b $fc,"Linus Walleij",$fe
 dc.b $fc,"Marcel Donn&",$fe
 dc.b $fc,"Robert Nilsson",$fe
 dc.b $fc,"Max Levin",$fe
 dc.b $fc,"Peter M","z"+4,"rck",$fe
 dc.b $fc,"Rafael Dyll",$fe
 dc.b $fc,"Johan N",")"+1,"sholm",$fe
 dc.b $fc,"Reyn Ouwehand",$fe
 dc.b $fc,"Mermaid/Creators",$fe
 dc.b $fc,"Slumgud",$fe
 dc.b $fc,"Erik Persson",$fe
 dc.b $fc,"Octave sounds",$fe
 dc.b $fc,"Sonic Wanderer",$fe
 dc.b $fc,"Janek Rooney",$fe
 dc.b $fc,"Larsec",$fe
 dc.b $fc,"Mikael Ericsson Duffy",$fe
 dc.b $fc,"Eliot",$fe
 dc.b $fc,"Che 'THC' Lalic",$fe
 dc.b $fc,"Per Bolmstedt",$fe
 dc.b $fc,"David Whittaker",$fe
 dc.b " ",$fe
 dc.b $f9,$40,$fc,"...and those I forgot!",$fe

 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe

 dc.b $f9,1,$fa,"Visit these sites:",$fa,$fe
 dc.b $f9,$80,$fd,"http://remix64.com",$fe
 dc.b $fd,"remix.kwed.org",$fe
 dc.b " ",$fe

 dc.b $f9,1,$fa,"Download sourcecode",$fa,$fe
 dc.b $f9,$80,$fd,"http://mahoney.c64.org",$fe
 dc.b " ",$fe

 dc.b $f9,1,$fa,"Contact me:",$fa,$fe
 dc.b $f9,$80,$fd,"mahoney",0,"c64.org",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe

 dc.b $f9,$40,$fc,"Have a noise night!",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe

 dc.b $f8," ",$fe
 dc.b " ",$fe
 dc.b " ",$fe

 dc.b $f9,$1,"I can feel it...",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,$40,$fc,"it's coming...",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,$80,$fd,"..the restart is near",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,1,"I see the light...",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,$40,$fc,"at the end of the tunnel",$fe
 dc.b " ",$fe
 dc.b " ",$fe
 dc.b $f9,$80,$fd,"IT'S A TRAIN...!!",$fe
 dc.b " ",$fe
 dc.b $f9,$80,$fd,"TSCHHHHOOOUUUK splat!",$fe
 dc.b " ",$fe
 dc.b " ",$fe

 dc.b $20,$ff

;Here is the pu-crunched ppot-image
multicol4bitmap_pu:
  incbin ppot_multicol4bitmap.pu
multicol4bitmap_pu_end:  ;The length is $0f25

;This is the part of the image that is in the upper
;border:
Sprites_for_the_ppot_image:
;leftmost
  dc.b %00010101,%01000100,%01010000
  dc.b %00100010,%10101000,%10001000
  dc.b %01010101,%01010010,%10010001
  dc.b %00001010,%10101000,%00101010
  dc.b %00010101,%00010110,%01010100
  dc.b %00101010,%10101010,%10001010
  dc.b %00010101,%01010101,%01010101
  dc.b %00001000,%00100010,%10101000
  dc.b %01010101,%01010101,%01010100
  dc.b %00001110,%10101010,%10111001
  dc.b %01010101,%01010110,%01010100
  dc.b %00101010,%00101010,%00101010
  dc.b %01010101,%01010111,%00010101
  dc.b %00101110,%10101010,%10101000
  dc.b %01010101,%01010101,%00010001
  dc.b %00101010,%00101010,%10111010
  dc.b %01010101,%01010111,%01010101
  dc.b %00001110,%10101010,%10101010
  dc.b %01010101,%01010101,%01010101
  dc.b %00101110,%10101010,%10110010
  dc.b %01010101,%10101101,%01010101,0

  dc.b %00100000,%00000000,%00000000
  dc.b %10001010,%10000000,%00000000
  dc.b %00000001,%00000001,%00000100
  dc.b %01001000,%10000010,%10000000
  dc.b %00000101,%00010001,%00010000
  dc.b %00100000,%10101000,%01000100
  dc.b %00000101,%00000100,%00000001
  dc.b %10010000,%00000000,%10010000
  dc.b %00000100,%00010001,%01000101
  dc.b %01001000,%10000010,%10000000
  dc.b %00000000,%00000100,%00010101
  dc.b %10001010,%00001010,%10000000
  dc.b %00000000,%01000000,%01001001
  dc.b %00010010,%00100000,%10000010
  dc.b %00000100,%01001001,%00100001
  dc.b %10100010,%00000010,%01001010
  dc.b %00000001,%00010000,%00010001
  dc.b %00101000,%10000001,%00000000
  dc.b %01000100,%01001000,%00100101
  dc.b %10000010,%10000000,%10010000
  dc.b %00001001,%01010101,%10100100,0

  dc.b %01000000,%00000000,%00000000
  dc.b %00100000,%00000000,%00000000
  dc.b %01010000,%00000000,%00000000
  dc.b %10100000,%00000000,%00000000
  dc.b %01001000,%00000000,%00000000
  dc.b %10100100,%00100010,%00000000
  dc.b %00101001,%10001000,%00000000
  dc.b %01000000,%00000010,%00000100
  dc.b %00100100,%01001001,%01000000
  dc.b %01011000,%00100010,%00000000
  dc.b %10100001,%10000100,%00001000
  dc.b %01010100,%00100010,%00100000
  dc.b %01100000,%10001000,%00000100
  dc.b %11011010,%00010010,%01000000
  dc.b %01110000,%01000101,%00000001
  dc.b %10100001,%10001000,%00000000
  dc.b %01111000,%00100010,%00010000
  dc.b %10100101,%10001000,%00000100
  dc.b %01110101,%01010101,%01000001
  dc.b %10101000,%00000010,%00100000
  dc.b %11110101,%01010101,%01000101,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000010,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00001000,%00000001,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00100001,%10001010,%00000000
  dc.b %10001000,%00000001,%00100000
  dc.b %01000101,%00100010,%01000101
  dc.b %10000000,%00000001,%00100000
  dc.b %00100001,%01001001,%00000000
  dc.b %10001000,%00000010,%10010000
  dc.b %00100001,%10000101,%00000100
  dc.b %10000100,%00000010,%10100000
  dc.b %01000000,%01000001,%10000101
  dc.b %00001001,%00001001,%00001000
  dc.b %10000100,%01000010,%10100000
  dc.b %00010001,%10010001,%00000000
  dc.b %01000100,%00000101,%01010010
  dc.b %10001000,%00101010,%10000000
  dc.b %00010001,%01000101,%01000100,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %10010000,%00100001,%00000000
  dc.b %00000001,%10001000,%00001000
  dc.b %10001000,%00000010,%00100000
  dc.b %00100001,%10001000,%00000100
  dc.b %10000000,%00100010,%00100000
  dc.b %01010100,%01010101,%01001001
  dc.b %00000001,%10001000,%00000010
  dc.b %10001000,%00100010,%00100000
  dc.b %00010001,%10001000,%00000100
  dc.b %01000001,%01000101,%00000001
  dc.b %10001000,%00100010,%00100000
  dc.b %00010001,%00000101,%00000100,0

  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00100010,%10001000,%00000100
  dc.b %00000000,%00000000,%00000000
  dc.b %01000001,%01000100,%01000000
  dc.b %00001000,%00000010,%00000000
  dc.b %00010000,%00000000,%00010000
  dc.b %01000001,%10010000,%00000100
  dc.b %00001000,%00100010,%00100000
  dc.b %00100001,%10001000,%00000100
  dc.b %01010101,%01000101,%00010101
  dc.b %10001000,%00100010,%00100000
  dc.b %01010101,%01010101,%01010101,0
;rightmost:
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00010000,%00000000
  dc.b %00000000,%10000000,%00000000
  dc.b %00010000,%00000000,%00000000
  dc.b %10000000,%00000000,%00000000
  dc.b %00000001,%00000000,%00000000
  dc.b %00000000,%00100000,%00000000
  dc.b %01000000,%00000000,%00000000
  dc.b %00000000,%00001000,%00000000
  dc.b %00010000,%00000000,%00000000
  dc.b %00000010,%00100000,%00000000
  dc.b %00000001,%00000000,%00000000
  dc.b %10000000,%10000000,%00000000
  dc.b %00000101,%00010000,%00000000
  dc.b %10000000,%10000000,%00000000
  dc.b %00010001,%00000100,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00010000,%00000000,%00000000
  dc.b %10000000,%01000100,%00000000
  dc.b %00010001,%00000000,%00000000,0
 ;leftmost, above the rest
  dc.b %00010101,%00010101,%00000000
  dc.b %00001010,%10001000,%00100000
  dc.b %00000101,%00000100,%00000000
  dc.b %00000010,%00000010,%10000000
  dc.b %00000001,%00010000,%00010001
  dc.b %00001000,%00100010,%10001000
  dc.b %00010001,%00000100,%00000000
  dc.b %00000000,%10000000,%01000000
  dc.b %00000100,%00100001,%00000100
  dc.b %00000000,%00001000,%10000000
  dc.b %01010000,%00000001,%00000000
  dc.b %00001010,%00100000,%00000000
  dc.b %00000001,%01000101,%01000100
  dc.b %00010000,%10000000,%00000000
  dc.b %01000101,%00000010,%00000001
  dc.b %00000000,%00100100,%10000000
  dc.b %00001001,%00000001,%00000000
  dc.b %00100010,%00100010,%10100010
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000,0


image_scroller_text:
  dc.b "        NEXT ON STAGE","Z"+8," "
  dc.b "SOREN TRAUTNER MADSEN  "
  dc.b "BEN DAGLISH  "
  dc.b "UFFE FRIIS LICHTENBERG  "
  dc.b "ROB HUBBARD  "
  dc.b "THEO ENGELL NIELSEN  "
  dc.b "JESPER HOLM OLSEN  "
  dc.b "ANDRE TISCHER POULSEN  "
  dc.b "MARTIN KOCH  "
  dc.b "      ",0

;Yes I know this takes a lot of memory.... but it
;was the fastest way of coding it... :)
text:
  dc.b "                                        "
  dc.b "    **** COMMODORE 64 BASIC V2 ****     "
  dc.b "                                        "
  dc.b " 64K RAM SYSTEM  38911 BASIC BYTES FREE "
  dc.b "                                        "
  dc.b "READY.                                  "
  dc.b "                "

textpressspace:
  dc.b " PRESS SPACE TO START!                  "
  dc.b "              / PEX 'MAHONEY' TUFVESSON "


pexyoung_init: subroutine
  sei
  lda #0
  sta $d015

  lda #$0b
  sta $d011     ; set the raster interrupt location
  lda #<irq_young_border
  sta $fffe
  lda #>irq_young_border
  sta $ffff
  lda #<nmi_spr                       ;Set NMI, both the hardware
  sta $fffa                       ;KERNAL
  lda #>nmi_spr
  sta $fffb
  lda #irq_young_border_raster
  sta $d012
  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d
  ldx #1
  stx $d01a     ; enable raster interrupt
  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts

  ldx #0
  lda #$0
.l:
  sta $7c00,x
;  sta $7d00,x
;  sta $7e00,x
  sta $7ef7,x
  inx
  bne .l

  ldx #0
.l5:
  lda #$16
  sta $7c50,x
  sta $7d00,x
  sta $7e00,x
  sta $7e70,x
  inx
  bne .l5

  cli
  rts

irq_young0_raster = $3f
irq_young0:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%00111000 ;bitmap mode, 25 rows, noblank
  sta $d011
  lda #<irq_young_border    ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_young_border
  sta $ffff
  lda #irq_young_border_raster
  sta $d012
  lda #%00000010
  sta $dd00

;0 = $4000
;8 = $6000
;15 = $7c00
  lda #%11111000
  sta $d018     ;Screen should be at $7c00 - bitmap at $6000
  nop
  lda #$1
  sta $d020

  jsr music_play

  lda savea
  ldx savex
  ldy savey
  rti


irq_young_border_raster = $df
irq_young_border:
  cld
  sta savea
  stx savex
  sty savey
  dec $d019                       ;Acknowledge raster interrupt
  lda #%00111000
  sta $d011
  lda #<irq_young0                      ;Set up the sprite display IRQ
  sta $fffe
  lda #>irq_young0
  sta $ffff
  lda #irq_young0_raster
  sta $d012
  lda #%00000010
  sta $dd00
  lda #0
  sta $d020
  lda #%11001000
  sta $d016

  lda savea
  ldx savex
  ldy savey
  rti

;This is the image of me as a baby
pex_young_pu:
  incbin pex_young.pu
pex_young_pu_end:  ;The length is $0f25

;###
;### ppot_image faderut:
;###

  jmp ppot_image_faderut

;image_wait: dc.b 0
;imaclr_poi: dc.b 0,0

;This is the routine to erase the ppot image:
ppot_image_faderut: subroutine
.wai:
  lda IRQ_finished
  beq .wai
  lda #0
  sta IRQ_finished

  inc image_wait
  bne .nowr
  inc image_wait+1
.nowr:
  lda image_wait+1
  cmp #3
  bne .wai
  lda image_wait
  cmp #$46
  bne .wai

  lda #0
  sta imaclr_poi
  sta imaclr_poi+1

.fadelop:
  lda IRQ_finished
  beq .fadelop

  lda #0
  sta IRQ_finished
  sta imaclr_poi
  sta imaclr_poi+1
.toohigh:
  jsr new_random
  clc
  adc image_wait+1
  cmp #$ae
  bcs .toohigh

  cmp #$99
  bcc .erase_bitmap

  sec
  sbc #$99
  sta imaclr_poi
  asl
  clc
  adc imaclr_poi
  tay
  lda #0
  sta $7000,y
  sta $7001,y
  sta $7002,y
  sta $7040,y
  sta $7041,y
  sta $7042,y
  sta $7080,y
  sta $7081,y
  sta $7082,y
  sta $70c0,y
  sta $70c1,y
  sta $70c2,y
  sta $7100,y
  sta $7101,y
  sta $7102,y
  sta $7140,y
  sta $7141,y
  sta $7142,y
  sta $7180,y
  sta $7181,y
  sta $7182,y
  sta $71c0,y
  sta $71c1,y
  sta $71c2,y
  jmp .nextframe

.erase_bitmap:
;  and #$7f   ;a is the y position
  sec
  sbc #4
  cmp #$ff
  beq .zero
  cmp #$fe
  beq .zero
  cmp #$fd
  beq .zero
  cmp #$fc
  bne .nozero

.zero:
  lda #0
.nozero:

  pha
  lsr
  lsr
  lsr
;a is the row number, multiply by 320 = 256 + 64
  sta imaclr_poi+1 ;that was 256
  lsr
  ror imaclr_poi
  lsr
  ror imaclr_poi
  clc
  adc #$40
  adc imaclr_poi+1
  sta imaclr_poi+1 ;that was 64

  pla
  and #$07         ;now add position in row
  ora imaclr_poi
  sta imaclr_poi

  ldx #40
.more:
  ldy #0
  lda (imaclr_poi),y
  pha
  pha
  pha
  and #$03
  tay
  lda newcoltable0,y
  sta image_wait
  pla
  lsr
  lsr
  and #$03
  tay
  lda newcoltable2,y
  ora image_wait
  sta image_wait
  pla
  lsr
  lsr
  lsr
  lsr
  and #$03
  tay
  lda newcoltable4,y
  ora image_wait
  sta image_wait
  pla
  lsr
  lsr
  lsr
  lsr
  lsr
  lsr
  and #$03
  tay
  lda newcoltable6,y
  ora image_wait
  ldy #0
  sta (imaclr_poi),y

  lda imaclr_poi
  clc
  adc #8
  sta imaclr_poi
  lda imaclr_poi+1
  adc #0
  sta imaclr_poi+1
  dex
  bne .more
  
.nextframe:
  lda part_finished
  bne .ennd
  jmp .fadelop

.ennd:
  jmp next_demo_part

;Colour 03 becomes 02
;Colour 02 becomes 01
;Colour 01 becomes 00
;Colour 00 becomes 00
newcoltable0: dc.b %00000000,%00000000,%00000001,%00000010
newcoltable2: dc.b %00000000,%00000000,%00000100,%00001000
newcoltable4: dc.b %00000000,%00000000,%00010000,%00100000
newcoltable6: dc.b %00000000,%00000000,%01000000,%10000000


;###
;### varglad2_sprs
;###

;This is the routine for showing the part of the
;Melina image in the upper border.
varglad2_sprs_init:
  lda #$32-21 ;spritey
  sta $d00b
  sta $d00d
  sta $d00f
  lda #$32-21-21 ;spritey
  sta $d009
  lda #$80
  sta $d010
  lda #$60
  sta $d00a
  lda #$78
  sta $d00c
  lda #$68
  sta $d008
  lda #$1a
  sta $d00e

  ldx #$80
  stx $7bfc
  inx
  stx $7bfd
  inx
  stx $7bfe
  inx
  stx $7bff

  ldx #0
.mo:
  lda varglad2_sprs,x
  sta $6000,x
  inx
  bne .mo
  rts

varglad2_sprs:
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00000000,%00000000,%00001000
  dc.b %00000000,%00000000,%10000000
  dc.b %00000000,%00010001,%00000000
  dc.b %00000000,%10000010,%00000000
  dc.b %00000000,%00000000,%00010001
  dc.b %00000001,%00001010,%00000000
  dc.b %00000000,%01000000,%00000000
  dc.b %00000010,%00100100,%10000100
  dc.b %00000100,%01000010,%00100000
  dc.b %00001010,%00000000,%10000010
  dc.b %00000001,%00100100,%00010000
  dc.b %00101000,%10000001,%00000000
  dc.b %01000100,%01001000,%00000010
  dc.b %00010010,%00000010,%10010000
  dc.b %10000101,%00010001,%01000001,0

  dc.b %00000000,%00010001,%00000100
  dc.b %00000010,%10000100,%01010000
  dc.b %00000001,%00100001,%00000100
  dc.b %00000000,%10001010,%10010010
  dc.b %00000010,%00010001,%00101001
  dc.b %00000000,%10100010,%01010010
  dc.b %00000100,%00010000,%10101000
  dc.b %00000001,%01000100,%01010100
  dc.b %00000100,%00010001,%10100000
  dc.b %00001001,%01000010,%01010100
  dc.b %00000100,%10010100,%10101001
  dc.b %00001010,%00101010,%11010000
  dc.b %00000000,%01000101,%01101000
  dc.b %00010010,%00101110,%11010010
  dc.b %00000100,%01010101,%10101000
  dc.b %00100010,%00001010,%11010000
  dc.b %00000001,%01011101,%10100100
  dc.b %00100100,%10101010,%11010000
  dc.b %01001000,%01110111,%01100101
  dc.b %10000010,%10101010,%11110000
  dc.b %00100100,%01011101,%10100100,0

  dc.b %00100000,%00000010,%00000000
  dc.b %10000010,%10000000,%00000000
  dc.b %00000001,%00000000,%00000000
  dc.b %01000000,%00000000,%00000000
  dc.b %10000001,%00000000,%00000000
  dc.b %00100000,%00000000,%00000000
  dc.b %00000100,%00000100,%00000000
  dc.b %10000000,%00000000,%00010000
  dc.b %00001000,%00000000,%00000000
  dc.b %01000000,%10000000,%00000000
  dc.b %00000100,%00000000,%00000000
  dc.b %10010000,%00001000,%10000000
  dc.b %00000000,%00000000,%00000000
  dc.b %00100010,%00000000,%00000000
  dc.b %00000000,%01000000,%00000000
  dc.b %10010000,%00000000,%00001000
  dc.b %00000001,%00000000,%00000000
  dc.b %10001000,%00000001,%00000000
  dc.b %00100000,%00000000,%00000000
  dc.b %10000000,%10000000,%00000000
  dc.b %01001000,%00000000,%00000100,0

  dc.b %00100000,%00010000,%00000000
  dc.b %10000001,%00000000,%00000000
  dc.b %00000000,%00000001,%00000100
  dc.b %01000100,%10000000,%10000000
  dc.b %00000001,%00010001,%00010000
  dc.b %10000010,%00101000,%01000000
  dc.b %00000001,%00000100,%00000001
  dc.b %10010000,%01000000,%10010000
  dc.b %00000001,%00010001,%01000100
  dc.b %01001000,%10100010,%10000000
  dc.b %00000001,%01000100,%00010100
  dc.b %10001000,%00101010,%10000010
  dc.b %00000000,%01010000,%01001000
  dc.b %01010010,%00101000,%10000000
  dc.b %00000100,%01010001,%00100001
  dc.b %10100010,%00101010,%01001000
  dc.b %00000000,%00010101,%00010001
  dc.b %00100010,%00101010,%00000100
  dc.b %01001000,%01010101,%00100000
  dc.b %00100000,%00101010,%10010101
  dc.b %00010010,%01010101,%10100000,0








;###
;### end
;###

 seg zap
 org $e000
;This is Rob Hubbard's Spellbound music;
 incbin spell_e000.sid

 seg varglad
 org $f000
varglad_pu:
 incbin varglad.pu
varglad2_pu:
 incbin varglad2.pu

;###
;###
;###

;Shows the "Var glad fast du r svensk"-sticker.
;This is "Be happy, eventhough you are Swedish" in english.
;This is moved with a simple interruptless FLD-routine.
  nop
start_varglad1: subroutine
  sei
  lda #$35
  sta $01
  lda #0
  sta $d015
  sta $d020
  sta $d021

  lda #$0b
  sta $d011     ; set the raster interrupt location
  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d

  lda #$60  ;opcode for rts -> decrunch is fastest
  sta decrunch_slow

  jsr music_play
  jsr justplay_init

  ldx #>(varglad_pu+2)
  ldy #<(varglad_pu+2)
  jsr pu

  ldx #10
  stx $d01a     ; disable raster interrupt
  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts

  ldx #0
.l8:
  lda #$7
  sta $7c00,x
  sta $7d00,x
  sta $7e00,x
  sta $7ef8,x
  inx
  bne .l8

  sei

  lda #%00111000 ;bitmap mode, 25 rows, noblank
  sta $d011
  lda #%00000010
  sta $dd00
  lda #%11110000
  sta $d018     ;Screen should be at $7c00 - bitmap at $4000
  lda #%11001000
  sta $d016

fldmain:
  lda #$bc
  sta fld_tmp

fld:
  lda $d011
  bpl fld
.f1:
  lda $d011
  bmi .f1
  lda #$2a
.f2:
  cmp $d012
  bne .f2

  lda fld_tmp
  cmp #$4
  beq fldmain2

fld_start:
  ldx fld_tmp
  ldy $d012
fld_bounce:
  ldy $d012
.fwa:
  cpy $d012
  beq .fwa
  dey
  tya
  and #$07
;  ora $10
  ora #$38
  sta $d011
  dex
  bne fld_bounce


;  ldy #$15
;  sty $d018
  lda fld_tmp
  sec
  sbc #$08
;  sbc #$00
  sta fld_tmp
  jsr music_play
  jmp fld

fldmain2:
;  lda #$1b
  lda #%00111000 ;bitmap mode, 25 rows, noblank
  sta $d011

  lda #$cd    ;How long shall the varglad-image be displayed?
  sta fld_tmp
.f8:
  lda $d011
  bpl .f8
.f9:
  lda $d011
  bmi .f9
  jsr music_play
  dec fld_tmp
  bne .f8

  lda #$04
  sta fld_tmp

fld2: subroutine
  lda $d011
  bpl fld2
.f1:
  lda $d011
  bmi .f1
  lda #$2a
.f2:
  cmp $d012
  bne .f2

  lda fld_tmp
  cmp #$c4
  beq .fldmain2

.fld_start:
  ldx fld_tmp
  ldy $d012
.fld_bounce:
  ldy $d012
.fwa:
  cpy $d012
  beq .fwa
  dey
  tya
  and #$07
;  ora $10
  ora #$38
  sta $d011
  dex
  bne .fld_bounce


;  ldy #$15
;  sty $d018
  lda fld_tmp
  clc
  adc #$08
  sta fld_tmp
  jsr music_play
  jmp fld2

.fldmain2:
.ever:

  lda #$0b
  sta $d011
  jsr music_play
  jmp next_demo_part


;###
;###
;###

;Shows Melina's face on a white background:
start_varglad2: subroutine
  sei
  ldx #1
  stx $d020
  stx $d021
  lda #$0b
  sta $d011     ; set the raster interrupt location

.waaa:
  lda $d011
  bmi .waaa
  jsr varglad2_sprs_init
  ldx #0
  stx $d015
  stx $d017
  stx $d01d

  lda #$7f
  sta $dc0d     ; disable timer interrupts
  sta $dd0d

  lda #$60  ;opcode for rts -> decrunch is fastest
  sta decrunch_slow

  jsr justplay_init

  ldx #>(varglad2_pu+2)
  ldy #<(varglad2_pu+2)
  jsr pu

  ldx #10
  stx $d01a     ; disable raster interrupt
  lda $dc0d     ; acknowledge CIA interrupts
  lsr $d019     ; and video interrupts

  ldx #0
.l8:
  lda #$11
  sta $7800,x
  sta $7900,x
  sta $7a00,x
  sta $7af8,x
  inx
  bne .l8
  lda #39
  sta whitex

  lda #$0
  sta whitex_colcou
  sta $7fff
  lda whitex_colcou_tbl
  sta current_col
.loopwhitex:
  lda #$fa
.no:
  cmp $d012
  bne .no
  lda #%00110011 ;bitmap mode, 25 rows, noblank
  sta $d011
  ldx #9
.ll4:
  dex
  bne .ll4
  lda #%00010011 ;bitmap mode, 25 rows, noblank
  sta $d011

  lda whitex
  tay
  lda #$78
  sta whitex_poi+1
  lda #0
  sta whitex_poi
  ldx #25
.lo:
current_col = *+1
  lda #$10
  sta (whitex_poi),y
  lda whitex_poi
  clc
  adc #40
  sta whitex_poi
  lda whitex_poi+1
  adc #0
  sta whitex_poi+1
  dex
  bne .lo
  dec whitex
  lda whitex
  cmp #7
  bne .loop_nonewcol
  lda #39
  sta whitex
  inc whitex_colcou
  ldx whitex_colcou
  lda whitex_colcou_tbl,x
  sta current_col
  bpl .ok_col

.end_part:
  lda #$0b
  sta $d011
  jmp next_demo_part

.loop_nonewcol:
.ok_col:
  lda #$80
  sta $7bfc

  lda $7822
  sta $d02e

  lda $780a
  sta $d02c

  lda $780b
  sta $d02b

  lda $780d
  sta $d02d


;  lda #$30
;.f8:
;  cmp $d012
;  bne .f8

  lda #$00
.f9:
  cmp $d012
  bne .f9

  lda #%11110000
  sta $d015

  lda #$32
.fa:
  cmp $d012
  bne .fa
  lda #%00000010
  sta $dd00
  lda #%11100000
  sta $d018     ;Screen should be at $7800 - bitmap at $4000
  lda #%11001000
  sta $d016
  ldx #6
.ll6:
  dex
  bne .ll6

  lda #%00111011 ;bitmap mode, 25 rows, noblank
  sta $d011

  lda #0
  sta $d015

  jsr music_play
  jmp .loopwhitex

;The colours that the face will have:
whitex_colcou_tbl:
  dc.b $1d,$15,$10,$10,$10,$1b,$1c,$1f,$11,$ff

;The last byte is at $fff9. Which is pretty good, considering
;that $fffa is used as an interrupt-vector in the 6510-processor!

;This is the end, as it should be. Thanks for watching, and
;please have fun! / Pex "Mahoney" Tufvesson
